/* FILE: process.c              Copyright (c), Symmetric Research 1998-2000
 *
 * Watch for a PAR24B .out file, calling a threshold detector when it
 * appears.
 *
 * If the threshold exceeds the specified value, the .out is renamed
 * with a timestamp filename, otherwise deleted.
 *
 * Note that the processing is put to sleep for intervals to give other
 * processes a chance to execute.
 *
 *
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>

#ifdef SROS_LINUX
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>          /* STDIN_FILENO define */
#else
#include <sys\types.h>
#include <sys\stat.h>
#include <conio.h>
#include <io.h>
#endif

#include <fcntl.h>
#include <time.h>

#include "simp24b.h"      /* <<< required for the HDR layout ... */


#ifdef SROS_LINUX
#define O_BINARY O_FSYNC
#endif








/* REV MESSAGE:
 *
 *
 *
 */

void Rev( void) {

        /* Print the rev info. */

        printf( "PAR24B process : (c) SR 1998-2000 : Rev {01/05/00}\n");

}


#ifdef SROS_LINUX

int _filelength( int fh ) {

        struct stat buffer;

        if ( fstat( fh, &buffer ) == -1 ) {
                printf( "Could not get file stats\n" );
                return( -1 );
                }

        return( buffer.st_size );
}

int SetNonBlock( int value ) {

        int oldflags, result;

        oldflags = fcntl( STDIN_FILENO, F_GETFL, 0 );

        if (oldflags == -1)
                return( -1 );

        if (value != 0)
                oldflags |= O_NONBLOCK;
        else
                oldflags &= ~O_NONBLOCK;

       result = fcntl( STDIN_FILENO, F_SETFL, oldflags );
       return( result );
}

#endif // SROS_LINUX



/* ERRORS:
 *
 *
 *
 *
 */

char *ErrorMsg[] = {

        "ERROR: %s                                                 \n",
        "                                                          \n",
        "  process hdrfile outfile threshold      ... where        \n",
        "                                                          \n",
        "    hdrfile     = binary .hdr file from simp24b/scope24b  \n",
        "    outfile     = binary .out file from simp24b/scope24b  \n",
        "    threshold   = abs integer threshold level (eg 2345)   \n",

        NULL,

        };

void Error( char *msg) {

        char *Chop( char *s);  /* see bottom of file for code ... */


        /* Display the rev. */

        Rev();


        /* Print the command line usage message and quit. */

        printf( Chop( ErrorMsg[0]), msg           );
        printf( Chop( ErrorMsg[1])                );
        printf( Chop( ErrorMsg[2])                );
        printf( Chop( ErrorMsg[3])                );
        printf( Chop( ErrorMsg[4])                );
        printf( Chop( ErrorMsg[5])                );
        printf( Chop( ErrorMsg[6])                );

        exit( 1);

}
char *Chop( char *s) {

        char *p;


        /* Chop spaces ' ' off the end of a string and add '\n'. */

        p = s;

        while ( *p )

                p++;


        if ( *--p == '\n' ) {

                while ( *--p == ' ' )

                        /* back up */ ;

                *++p = '\n';
                *++p = '\0';

                }

        return( s);
}

int GetKeyCheck( void ) {

        int c;

        /* Get a keypress if ready. */

#ifdef SROS_LINUX
        c = getchar();
#else        
        c = EOF;
        if ( kbhit() ) {
                c = getch();
                if (c == 0xD) printf("\n"); /* add newline if char was ENTER */
                }
#endif

        return( c );
}


int SrSleep( int ms ) {

        /* Sleep for specifed number of milliseconds.
         * Returns 0 for success, -1 for error.
         *
         * Linux nanosleep takes structure with seconds and nanoseconds.
         * No sleep function is available for MSDOS, so return immediately.
         */

#ifdef SROS_LINUX
        
        struct timespec twait, tleft;
        tleft.tv_sec  = tleft.tv_nsec = 0;
        twait.tv_sec  = (long)(ms / 1000);
        twait.tv_nsec = (ms - (twait.tv_sec*1000)) * 1000000;
        return( nanosleep( &twait, &tleft ) );

#elif defined( SROS_MSDOS )
        
        return( 0 );

#else        
        _sleep( ms );
        return( 0 );
#endif
        
}






/* HDR FILE READ:
 *
 *
 *
 */

struct HDR_data HDR;

void ReadHdrFile( char *HdrFilename) {

        int FhHdrFile;



        /* Read in the .hdr data file. */

        FhHdrFile = open( HdrFilename, O_RDONLY | O_BINARY);

        if ( !FhHdrFile )

                Error( "unable to open .hdr file");

        if ( read( FhHdrFile, &HDR, sizeof( HDR)) != sizeof( HDR) )

                Error( "invalid .hdr file (1)");

        close( FhHdrFile);



        /* Examine the Id string. */

        if ( strcmp( HDR.Id, "SR PAR24B") != 0 )

                Error( "invalid .hdr file (2)");

}






/* OUT DATA FILE READ:
 *
 * The .out file is processed in chunks.  This function reads in a buffer
 * of nvalues.
 *
 *
 *
 */

int GetOutData( int FhOutFile, long *values, int nvalues, int *nread) {

        int nbytes;


        nbytes = read( FhOutFile, values, nvalues*HDR.WordSize);

        if ( nbytes )

                { *nread = nbytes/HDR.WordSize; return( 1); }

        else
                return( 0);

}







/* DATA BUFFER:
 *
 * Buffer for processing the .out data file.
 *
 *
 *
 */

#define NVALUES         5000

long VALUES[ NVALUES];





/* PAR24B ABSOLUTE VALUE:
 *
 * The PAR24B drivers return 24 bit signed or offset values.  It is assumed
 * the program is running the signed default.
 *
 * This function computes the absolute value of the 24 bit twos compliment
 * signed PAR24B values.
 *
 *
 *
 */

long Par24bAbs( long v) {


        /* Compute the absolute value of a 24 bit PAR24B signed value. */

        if ( (unsigned long)v > 0x007FFFFF )

                v = 0x01000000 - v;

        return( v);

}






/* THRESHOLD DETECTOR:
 *
 * This function compares the entire .out file against the threshold.
 *
 * Even when the function finds the threshold is exceeded, it continues
 * on to scan all the data in the file so it runs in a predictable amount
 * of time.
 *
 * Note that the data is actually multiplexed, but it doesn't matter for
 * a simple threshold scan.
 *
 * It is assumed the PAR24B DataCoding is SIGNED.
 *
 *
 *
 *
 */

#define EXCEEDED                1
#define NOT_EXCEEDED            0

int CheckThreshold( int FhOutFile, unsigned long Threshold) {

        int status, nread, i; long v;


        /* Compare the .out file against the threshold. */

        status = NOT_EXCEEDED;

        while ( GetOutData( FhOutFile, VALUES, NVALUES, &nread) ) {

                for ( i = 0 ; i < nread ; i++ ) {

                        if ( Par24bAbs( VALUES[i]) >= Threshold )

                                status = EXCEEDED;
                        }
                }


        return( status);

}








/* MAIN:
 *
 *
 *
 *
 */

int main( int argc, char *argv[]) {

        char
                *OutFilename,
                *HdrFilename,
                *StrThreshold;

        int FhOutFile;

            unsigned long OutFileSize;

        unsigned long Threshold;

            int status; char Newname[32];



#ifdef SROS_LINUX

        /* Allow getchar to return even if a key has not been pressed. */

        if ( SetNonBlock( 1 ) == -1 )
               Error( "Can't set non-blocking read ...");
#endif



        /* Get the command line arguments. */

        if ( argc != 4 )

                Error( "wrong number of command line arguments");

        HdrFilename    = argv[1];
        OutFilename    = argv[2];
        StrThreshold   = argv[3];





        /* Convert the command line threshold argument to an integer value. */

        if ( sscanf( StrThreshold, "%ld", &Threshold) != 1 )

                Error( "invalid threshold value");



        printf( "Starting threshold test.  Press a key to exit.\n\n" );


        /* Wait for the .out file to appear. */

        printf( "Waiting for %s data file to appear ...\n", OutFilename);

polling_loop:



        /* Try to open it. */

        FhOutFile = open( OutFilename, O_RDONLY | O_BINARY);


        if ( FhOutFile == -1 /* == file not found */ ) {


                /* Give the user a chance to quit ... */

                if ( GetKeyCheck() != EOF ) {
                        printf( "Processing stopped.\n ");
                        exit( 0);
                        }


                else /* sleep for awhile and continue polling. */

                        { SrSleep( 100 /* = time in ms */); goto polling_loop;}

                }
                






        /* Found the .out file, start processing ... */





        /* Read the header file. */

        ReadHdrFile( HdrFilename);


        /* Check .out filesize against .hdr info. */

        OutFileSize = HDR.Channels * HDR.RecordPts * HDR.WordSize;

        if ( _filelength( FhOutFile) != OutFileSize )

                Error( ".out filesize != .hdr info");






        /* Call the threshold detector, renaming if exceeded. */

        printf( "Found %s, thresh test ... ", OutFilename);


        status = CheckThreshold( FhOutFile, Threshold);

            close( FhOutFile);



        if ( status == EXCEEDED ) {

                sprintf( Newname, "%08lX.OUT", time( NULL));

                    rename( OutFilename, Newname);

                printf( "EXCEEDED, renamed to %s\n", Newname);

                }

        else
                printf( "not exceeded\n");






        /* Delete the source .out file. */

        remove( OutFilename);



        /* Continue polling. */

        goto polling_loop;


        return( 0 );
}
