/* FILE: sr2asc.c             Copyright (c), Symmetric Research 1998-2001
 *
 * This program reads in a binary .hdr and a binary .out file generated
 * by one of the SR acquisition programs and writes out the information
 * in an ASCII text version.  This file can be compiled for use with 
 * either the SR PAR24B or PAR4CH A/D board.
 *
 *
 *
 */

#include <stdio.h>





//****** A/D Specific Data ******


#if defined( ATOD_IS_PAR4CH )

#define APPHDRID        "SR PAR4CH"
#define APPNAME         "4ch"
#define ATODCHANNELS    4

        

#elif defined( ATOD_IS_PAR24B )

#define APPHDRID        "SR PAR24B"
#define APPNAME         "24b"
#define ATODCHANNELS    8


#else

#pragma message( "Valid A/D options are ATOD_IS_PAR4CH or ATOD_IS_PAR24B." )
#error Invalid A/D option defined.

#define APPHDRID        "SR ??????"
#define APPNAME         "???"
#define ATODCHANNELS    8

#endif


/* Header structure
 *
 *
 *
 */

#define MAXCHANNELS             ATODCHANNELS
#define MAXTITLE                12

struct HDR_data {

          char Id[32];

          long Channels;
          long RecordPts;
          long WordSize;
          long DataCoding;

        double SamplingRate;
        double Gain;
        double FilterCoeff;
        double FilterScale;

        struct channel_data { long channel; char title[ MAXTITLE ]; }

                        ChannelList[ MAXCHANNELS ];
        };

struct HDR_data HDR;

#define HDRSIZE         sizeof( struct HDR_data )




/* Sign extension constants
 *
 * PAR4CH data can be acquired in 24-bit signed or offset format.
 * When stored as a 32-bit number, the 24-bit signed format data must
 * be sign extended if the 24th bit is set.
 *
 */

#define NEG24BIT        0x00800000
#define SIGNEXTEND      0xFF000000
#define NOSIGNEXTEND    0x00000000




/* ERROR FUNCTIONS:
 *
 * Command line and file opening errors.
 *
 *
 */

void ErrorUsage( void ) {

        /* Give a command line usage message and quit. */

        printf( "USAGE: %3s2asc binary.hdr binary.out data.asc\n", APPNAME );

        exit( 1 );

}
void ErrorFile( char *fmt, char *filename ) {

        /* Give an error message and quit. */

        printf( fmt, filename );

        exit( 1 );

}






/* BINARY INPUT DATA BUFFER:
 *
 *
 *
 */

#define	NCHANNELS	ATODCHANNELS

long DataBuffer[ NCHANNELS ];

#define NBYTES          sizeof( DataBuffer )







/* MAIN PROGRAM:
 *
 * Open the input and output files and translate to ASCII.
 *
 *
 */

int main( int argc, char *argv[] ) {

        char *HeaderName; FILE *HeaderFile;
	char *BinaryName; FILE *BinaryFile;
        char *AsciiName;  FILE *AsciiFile;

	int i; long Nsample, SetTopBits;




	/* Process the command line. */

	if ( argc != 4 )

                ErrorUsage();



	
	/* Open input binary header file. */

        HeaderName = argv[1];
	HeaderFile = fopen( HeaderName, "rb" );

	if ( !HeaderFile )

                ErrorFile( "Can't open Header input file %s", HeaderName );




	/* Open input binary data file. */

        BinaryName = argv[2];
	BinaryFile = fopen( BinaryName, "rb" );

	if ( !BinaryFile )

                ErrorFile( "Can't open binary input file %s", BinaryName );




        /* Open the output ASCII data file. */

        AsciiName = argv[3];
	AsciiFile = fopen( AsciiName, "w" );

	if ( !AsciiFile )

                ErrorFile( "Can't open ASCII output file %s", AsciiName );




        /* Convert the header file to ASCII. */

        fread( &HDR, 1, HDRSIZE, HeaderFile );
        
        fprintf( AsciiFile, "Header:\n" );
        fprintf( AsciiFile, "           Id = %s\n",  HDR.Id           );
        fprintf( AsciiFile, "     Channels = %ld\n", HDR.Channels     );
        fprintf( AsciiFile, "    RecordPts = %ld\n", HDR.RecordPts    );
        fprintf( AsciiFile, "     WordSize = %ld\n", HDR.WordSize     );
        fprintf( AsciiFile, "   DataCoding = %ld\n", HDR.DataCoding   );
        fprintf( AsciiFile, " SamplingRate = %f\n",  HDR.SamplingRate );
        fprintf( AsciiFile, "         Gain = %f\n",  HDR.Gain         );
        fprintf( AsciiFile, "  FilterCoeff = %f\n",  HDR.FilterCoeff  );
        fprintf( AsciiFile, "  FilterScale = %f\n",  HDR.FilterScale  );

        for ( i = 0 ; i < HDR.Channels ; i++ )

                fprintf( AsciiFile, "   Channel[%1ld] = %s\n",

                                 HDR.ChannelList[i].channel,
                                 HDR.ChannelList[i].title

                               );



        
        /* Prepare data column titles. */
        
        fprintf( AsciiFile, "\nData:\n" );
        
        for ( i = 0 ; i < HDR.Channels ; i++ )

                fprintf( AsciiFile, "%8s ", HDR.ChannelList[i].title );
        
        fprintf( AsciiFile, "   Index\n" );



        /* Prepare sign extension constant for signed vs offset data. */

        if (HDR.DataCoding == 1)
                SetTopBits = SIGNEXTEND;
        else
                SetTopBits = NOSIGNEXTEND;



        


	/* Convert the binary data file to ASCII. */

	Nsample = 0;

	while ( fread( DataBuffer, 1, NBYTES, BinaryFile ) == NBYTES ) {


                /* Convert the samples for each channel at this time point. */

		for ( i = 0 ; i < NCHANNELS ; i++ ) {

                        /* Sign extend 24 bit data to 32 bits. */
                        
                        if (DataBuffer[i] & NEG24BIT)
                                DataBuffer[i] |= SetTopBits;
                        
			fprintf( AsciiFile, "%8ld ", DataBuffer[i] );
                        }



                /* Add a final column with the time index. */

		fprintf( AsciiFile, "%8ld\n", Nsample );

		Nsample++;


		}




	/* Close files and return success. */
	
        fclose( HeaderFile );
	fclose( BinaryFile );
        fclose( AsciiFile );

        exit( 0 );

}
