/* FILE: par24b.c                 Copyright (c), Symmetric Research, 2001
 *
 * This is the NT/Win2K version of PAR24B.DLL.  It acts as a shell or 
 * wrapper around the kernel mode device driver par24b.sys that does all 
 * the actual work.
 * 
 * The function calls in this DLL agree with those in the Win9x and DOS
 * par24b.c.  Applications that work with those DLLs will also work with
 * this one without modification.
 * 
 * However, in order for this NT/Win2K DLL to work, the underlying device
 * driver par24b.sys must be installed.  See the driver directory for
 * more details.
 * 
 *
 */

#include <windows.h>   /* for windef.h -> winnt.h -> defines GENERIC_READ etc */
#include <winioctl.h>  /* for device driver I/O control macros                */
#include <stdio.h>


/* INCLUDE PAR24B.H:
 *
 * Note that for building the library, _DLLEXPORT or _STATIC should be
 * defined from the compile command line.  
 *
 * See the makefile and par24b.h for details.
 *
 */

#include "par24bkd.h"  /* for device driver I/O control constants */
#include "par24b.h"    /* for EPP and PAR24B prototypes           */




/* Global variables */

HANDLE hndFile         = NULL;    /* Driver handle */
ULONG  lastDriverError = 0;


/* Error strings that must match defines in par4ch.h */
char *FAILURE_WORD[] = {

    "NONE",
    "PORT ADDRESS",
    "PORT MODE",
    "NO FIFO SIZE",
    "DATA CODING",
    "SAMPLING RATE",
    "GAIN",
    "CALIBRATION",
    "A/D READBACK",
    "DEVICE DRIVER",

    NULL,

    };



/* RELEASE REV INFO:
 *
 * Allow the user to determine the library rev they are working with.  See
 * par24b.h for more details.
 *
 */

VARTYPE( INT ) PAR24B_Rev = PAR24B_REV;

FUNCTYPE( INT ) PAR24B_GetRev( void ) {

        /* Return the library rev number. */

        return( PAR24B_Rev );
}




/* KEYBOARD HELPER FUNCTIONS:

These are needed to help smooth over keyboard behavior differences
between Linux and Windows.

GetKey has two behaviors.  If wait is true, it will wait for getch to
return a character.  Otherwise, it will use kbhit to check for an available
character.  If one is there, getch reads and returns it.  Otherwise, EOF
is returned.

GetKeyWait and GetKeyCheck are just handy shortcut variations.

*/

FUNCTYPE( INT) GetKey( int wait ) {

        int c;

        /* Get a ready keypress or wait until one is ready if requested. */

#ifdef SROS_LINUX

        c = getchar();
        while ( (c == EOF) && wait )
                c = getchar();
        
#else

        c = EOF;
        if ( kbhit() || wait ) {
                c = getch();
                if (c == 0xD) printf("\n"); /* add newline if char was ENTER */
                }
#endif

        return( c );
}

FUNCTYPE( INT) GetKeyCheck( void ) { return( GetKey( KEY_CHECK ) ); }
FUNCTYPE( INT) GetKeyWait(  void ) { return( GetKey( KEY_WAIT  ) ); }




/* PAR24B DEVICE DRIVER ACCESS FUNCTIONS:
 *
 * These functions open and close the communication connection between
 * the DLL and the NT device driver par24b.sys.
 *
 * It is recommended that application programs begin by calling PAR24B_Open 
 * to initiate communication and end by calling PAR24B_Close to terminate
 * driver communication.
 *
 * However, to maintain compatibility with existing programs, PAR24B_Init
 * will automatically call PAR24B_Open if necessary.  Also, the functionality
 * of PAR24B_Close is automatically performed by the operating system when 
 * the application terminates.
 *
 */

FUNCTYPE( INT ) PAR24B_Open( void ) {

        /* Only allow one open at a time */
        
        if (hndFile != NULL) {
                lastDriverError = ERROR_SERVICE_ALREADY_RUNNING;
                return( 1 );
                }

        
        /* Open device driver */

        hndFile = CreateFile(
                        DLL_DEVICE_NAME,                    /* "file" name */
                        GENERIC_READ    | GENERIC_WRITE,    /* access mode */
                        FILE_SHARE_READ | FILE_SHARE_WRITE, /* share mode  */
                        NULL,                               /* security    */
                        OPEN_EXISTING,                      /* create mode */
                        0,                                  /* file attrib */
                        NULL                                /* copy from   */
                        );

        if (hndFile == INVALID_HANDLE_VALUE) {
                lastDriverError = GetLastError();
                return( 0 );
                }

        lastDriverError = 0;
        return( 1 );
}

FUNCTYPE( INT ) PAR24B_Close( void ) {

        /* Only close open drivers */

        if (hndFile == NULL) {
                lastDriverError = ERROR_SERVICE_DISABLED;
                return( 1 );
                }


        /* Close driver */
        
        if (!CloseHandle(hndFile))
                return( 0 );


        /* Reset handle */
        
        hndFile = NULL;
        
        lastDriverError = 0;
        return( 1 );
}



/* DEVICE DRIVER IO HELPER FUNCTION:
 *
 * This function calls DeviceIoControl to communicate information between
 * the DLL functions and the kernel mode device driver.
 *
 * After the device driver has been opened with the CreateFile function,
 * it is ready to provide its services.  Common services such as reading
 * and writing data are requested with the ReadFile and WriteFile
 * functions.  More specialized services are requested using the
 * DeviceIoControl function.
 *
 * The second argument to the DeviceIoControl function is an IO control
 * code that indicates which service is being requested.  The available
 * control codes are given in par24bkd.h.
 *
 * The DeviceIoControl function allows small amounts of data to be 
 * exchanged between the caller and the device driver.  Arguments three
 * and four give the location and size of the data buffer passed to the 
 * device driver, while arguments five and size give the location and 
 * size of the data buffer passed back from the device driver.  The
 * seventh argument indicates how much of the returned buffer was
 * actually filled.
 *
 */

FUNCTYPE( INT) DevIoReadWrite( 

        ULONG IoCtlCode, 
        PVOID pValueIn,  
        DWORD InSize,
        PVOID pValueOut, 
        DWORD OutSize  

        ) {


        BOOL  code;
        DWORD bytesReturned;


        /* Device driver must be opened first. */
        
        if (hndFile == NULL) {
                lastDriverError = ERROR_SERVICE_DISABLED;
                return( 0 );
                }

        
        
        /* Request specialized services from device driver. */

        code = DeviceIoControl(
                       hndFile,          /* Handle to device               */
                       IoCtlCode,        /* IO Control code                */
                       pValueIn,         /* Ctrl data to driver            */
                       InSize,           /* Length of data in bytes        */
                       pValueOut,        /* Buffer from driver             */
                       OutSize,          /* Length of buffer in bytes      */
                       &bytesReturned,   /* Bytes placed in output buffer  */
                       NULL              /* NULL = wait till I/O completes */
                       );

        
        /* Failure output. */

        if (!code) {
                lastDriverError = GetLastError();
                return( 0 );
                }
        
        lastDriverError = 0;
        return( 1 );
}



/* PAR24B INITIALIZATION:
 *
 * This function must be called to initialize the PAR24B board before
 * starting execution.  It takes care of all initialization.  No other
 * functions need to be called to initialize the PAR24B board.
 *
 * During initialization the HI7190 control regster is programmed with
 * values setting the sampling rate etc.  All HI7190s are programmed
 * with the same values.
 *
 * See par24b.h for defined constants to use with Dc, Fp, Gain, CalMode,
 * ErrorCode etc.  To convert floating point sampling rates to Fp, use
 * the helper function PAR24B_SpsToFp.
 *
 * The function will also return the true size of the FIFO on board.
 * Pass a NULL pointer for this parameter if you wish to skip this part
 * of the calculation.  This is NOT recommended since the correct FIFO
 * size is required for acquisition. 
 *
 * If your program specifies the FIFO size separately, compare it against
 * the number returned by PAR24B_Init as a validity check.  Using the
 * wrong FifoSize and Nvalues with PAR24B_GetData will result in system
 * errors.  Use the FifoSizeToNvalues function to determine the proper
 * value to use with PAR24B_GetData.
 *
 *
 */


FUNCTYPE( INT ) PAR24B_Init(

             int PortAddress,
             int PortMode,
    unsigned int *FifoSize,

             int Dc,
             int Fp,
             int Gain,
             int CalMode,
            long *HI7190_CrValue,

             int *ErrorCode
             ) {


        INIT24BPARMS Parm;
        int          iret;



        /* Set up parameters to be passed to driver. */

        Parm.PortAddress      = PortAddress;
        Parm.PortMode         = PortMode;
        Parm.Dc               = Dc;
        Parm.Fp               = Fp;
        Parm.Gain             = Gain;
        Parm.CalMode          = CalMode;
        Parm.FifoSize         = 0;
        Parm.HI7190_CrValue   = 0;
        Parm.ErrorCode        = 0;

        if (FifoSize)
                Parm.FifoSize = 1;

        

        /* Device driver must be opened first */
        
        if (hndFile == NULL) {

                if ( !PAR24B_Open() ) {
                        if (ErrorCode)
                                *ErrorCode = PAR24B_IERROR_DRIVER_NOT_OPEN;
                        return( 0 );
                        }
                }

        

        /* Request initialization service. */

        iret = DevIoReadWrite( 
                              IOCTL_PAR24B_INIT,
                              &Parm, 
                              sizeof(Parm), 
                              &Parm, 
                              sizeof(Parm) 
                              );

        /* On NT, this should always return 1 because STATUS_SUCCESS
         * is sent from the driver to force the Parm output values
         * to be returned.  If not, there is some unknown driver 
         * error.
         */
         if (!iret)
                Parm.ErrorCode = PAR24B_IERROR_DRIVER_NOT_OPEN;


        /* Prepare for return by filling in pointer variables */

        if (ErrorCode)
                *ErrorCode      = Parm.ErrorCode;
        if (FifoSize)
                *FifoSize       = Parm.FifoSize;
        if (HI7190_CrValue)
                *HI7190_CrValue = Parm.HI7190_CrValue;


        if (Parm.ErrorCode == PAR24B_IERROR_SUCCESS)
                return( 1 );
        else
                return( 0 );
}


FUNCTYPE( VOID ) PAR24B_SpsToFp( double Sps, int *Fp ) {

	/* Use this helper function to compute the value of Fp to use
	 * with PAR24B_Init, where:
	 *
	 *	Sps = desired sampling rate in samples per second
	 *
	 *
	 * Note that the allowed range of fp is limited.  Only values
	 * between 10 and 2047 are acceptable for the HI7190.  This
	 * corresponds to sampling rates of:
	 *
	 *
	 *      Fp = 10    ->  sampling rate (Sps) = 1953.125 Hz
	 *      Fp = 2047  ->  sampling rate (Sps) =    9.541 Hz
	 *
	 *
	 * PAR24B_Init checks the value of Fp to make sure it is within
	 * this range.
	 *
	 *
	 *
	 *
	 * Also note that because Fp is an integer, the actual sampling
	 * rate will not be exactly the requested Sps.  Use the formula:
	 *
	 *         Sps = XTAL/(512*Fp)
	 *
	 * to get the exact sampling rate corresponding to Fp.  See the
	 * Harris HI7190 spec sheet for formulas relating Fp to Sps.
	 *
	 *
	 */

#define PAR24B_XTAL	(10.0E+6)

	*Fp = (int)(PAR24B_XTAL/(512.0*Sps));
}

FUNCTYPE( VOID ) PAR24B_FpToSps( int Fp, double *Sps ) {

	*Sps = PAR24B_XTAL/(512.0*Fp);
}




/* PAR24B EXECUTION CONTROL FUNCTIONS:
 *
 *
 *
 *
 */

FUNCTYPE( VOID ) PAR24B_Start( void ) {

        DevIoReadWrite( 
                        IOCTL_PAR24B_START, 
                        NULL, 
                        0, 
                        NULL, 
                        0
                        );
}

FUNCTYPE( VOID ) PAR24B_Stop( void ) {

        DevIoReadWrite( 
                        IOCTL_PAR24B_STOP, 
                        NULL, 
                        0, 
                        NULL, 
                        0 
                        );
}





/* PAR24B FIFO FUNCTIONS: (U230/720X)
 *
 *
 *
 *
 */


FUNCTYPE( INT ) PAR24B_IsEmpty( void ) {

        ULONG fifostatus;

        DevIoReadWrite( 
                        IOCTL_PAR24B_IS_EMPTY, 
                        NULL, 
                        0, 
                        &fifostatus, 
                        sizeof(ULONG) 
                        );

        return( fifostatus );
}


FUNCTYPE( INT ) PAR24B_IsReady( void ) {

        ULONG fifostatus;

        DevIoReadWrite( 
                        IOCTL_PAR24B_IS_READY, 
                        NULL, 
                        0, 
                        &fifostatus, 
                        sizeof(ULONG) 
                        );

        return( fifostatus );
}


FUNCTYPE( INT ) PAR24B_Overflow( void ) {

        ULONG fifostatus;

        DevIoReadWrite(
                        IOCTL_PAR24B_OVERFLOW, 
                        NULL,
                        0,
                        &fifostatus,
                        sizeof(ULONG)
                        );

        return( fifostatus );
}

FUNCTYPE( INT ) PAR24B_ReadFifo( long *Values ) {

        if ( DevIoReadWrite( 
                             IOCTL_PAR24B_READ_FIFO, 
                             NULL, 
                             0, 
                             Values, 
                             8*sizeof(LONG)
                             ) )

                return( 8 );

        else
                return( 0 );

}

FUNCTYPE( UINT ) PAR24B_GetData( long *Values, unsigned int Nvalues ) {

        BOOL    code;
        DWORD   bytesReturned;
        DWORD   sizeval;
        UINT    nread;

        lastDriverError = 0;
        sizeval         = Nvalues * sizeof(long);


        /* Device driver must be opened first */
        
        if (hndFile == NULL) {
                lastDriverError = ERROR_SERVICE_DISABLED;
                return( 0 );
                }

        
        code = ReadFile(
                        hndFile,         /* Handle to device               */
                        Values,          /* Buffer to receive data.        */
                        sizeval,         /* Number of bytes to read.       */
                        &bytesReturned,  /* Bytes read.                    */
                        NULL             /* NULL = wait till I/O completes */
                        );

        nread = bytesReturned / sizeof(long);


        
        /* Failure output */

        if (!code) {
                lastDriverError = GetLastError();
                return( 0 );
                }


        /* Return */


	return( nread ); /* # of longs just read */
}

FUNCTYPE( VOID ) PAR24B_FifoSizeToNvalues(

        unsigned int FifoSize,
        unsigned int *Nvalues

        ) {


        /* Use this function to convert the FifoSize reported by PAR24B_Init
         * to an array dimension Nvalues for use with PAR24B_GetData.
         *
         * Nvalues is the number of longs that are ready to be transferred
         * from the PAR24B FIFO to PC memory by PAR24B_GetData when IsReady
         * returns true.
         *
         * An explanation of the formula is:
         *
         * The FifoSize is divided by 2 because the IsReady function
         * returns true when the FIFO is half full.
         *
         * Then, because there are 8 serial streams going into each bit of
         * the byte wide FIFO, it takes 24 FIFO bytes to store one sample
         * from each of the A/Ds.  Ie, (FifoSize/2)/24 is the number of
         * samples per channel stored in the FIFO when IsReady returns
         * true.
         *
         * Finally, each 24 bit acquired value from the FIFO is stored in a
         * 32 bit long, and there are 8 channels worth of data.
         *
         *
         */

        *Nvalues = ((FifoSize/2)/24)*8;
}




/* PAR24B BOARD LEVEL USER PROGRAMMABLE BYTE: (UWR/U220/574)
 *
 * This function programs the on board user programmable byte register
 * whose values are available on header J220.  The lowest bit is also
 * indicated by the yellow LED.
 *
 * Note that PAR24B_Init must be called somewhere before this function
 * to initialize the PC's parallel port.
 *
 *
 */


FUNCTYPE( VOID ) PAR24B_UserByte( int Value ) {

        ULONG Val;

        Val = (ULONG)Value;

        DevIoReadWrite( 
                        IOCTL_PAR24B_USER_BYTE, 
                        &Val, 
                        sizeof(ULONG), 
                        NULL, 
                        0 
                        );
}



