// FILE: indriver.c              Copyright (c), Symmetric Research, 1999-2001
//
// Program to install par24b kernel mode driver.
//
// This is done by starting the Service Control Manager, creating the
// driver service, adding driver information to the registry, and then
// starting the service.  Additional registry entries are also added to
// allow Par24b driver messages to be properly interpreted by the
// system log EventViewer.
//
// Refer to the readme.txt file for information about how to install
// the device driver manually.
//


#include <windows.h>
#include <stdio.h>
#include "par24bkd.h"   // for REG_DRIVER constants


// Defines

#define MAX_STR 256


// Helper function prototype

void ErrorMessage( char *msg, int fatal );


// Global variables shared by main and helper function

SC_HANDLE SCManager, SCService;




void main( int argc, char**argv ) {

        HKEY     Par24bKey;
        BOOL     Ret;
        LONG     Code, Code1, Code2;
        ULONG    UserAddress, UserMode;
        ULONG    PortAddress, PortMode;
        DWORD    TagId, Disposition, TypesSupported;
        LPCTSTR  DriverName, DriverDesc, LoadGroup;
        LPCTSTR  ParameterKeyName, EventLogKeyName;
        CHAR     DriverPath[MAX_STR], EventMessageFile[MAX_STR];

        
        
        // Banner message

        printf( "\nPAR24B Kernel Mode Driver Installation Program\n");
        



        // Get port address and mode from user

        if ( (argc < 3) || (argc > 4) ) {
                printf( "\nUsage: indriver 0x### [EPP|BPP]\n\n" );
                printf( "       0x### is a port address such as ...\n" );
                printf( "           0x378, 0x278, 0x268, 0x26C, or 0x27C\n" );
                printf( "       EPP indicates enhanced parallel port mode\n" );
                printf( "       BPP indicates standard bi-directional mode\n\n" );
                printf( "A typical command line is:\n\n" );
                printf( "       > indriver 0x378 EPP\n" );
                exit( 1 );
                }

        
        sscanf( argv[1], "0x%x", &UserAddress );

        if ( (UserAddress != 0x378) &&
             (UserAddress != 0x278) &&
             (UserAddress != 0x268) &&
             (UserAddress != 0x26C) &&
             (UserAddress != 0x27C) &&
             (UserAddress != 0x3BC) ) {                
                printf( "Address must be 0x378, 0x278, etc\n");
                exit( 1 );
                }

        
        if ( stricmp( argv[2], "EPP" ) == 0 )
                UserMode = 0x1L;
        else if ( stricmp( argv[2], "BPP" ) == 0 )
                UserMode = 0x0L;
        else {
                printf( "Parallel port mode must be EPP or BPP\n");
                exit( 1 );
                }

                
        


        // Open Service Control Manager

        SCManager = NULL;
        SCService = NULL;
        
        SCManager = OpenSCManager(
                        NULL,                  // use local machine
                        NULL,                  // use SERVICES_ACTIVE_DATABASE
                        SC_MANAGER_ALL_ACCESS  // full access permission
                        );

        if (!SCManager)
                ErrorMessage("Could not create service manager", 1);
        



        // Create the device driver service.
        //
        // This involves creating a registry key with several 
        // values and adding the service to the services database.  
        // The registry entries can be made by hand.  But in that
        // case, you must reboot the machine before the service 
        // will be included in the services database.  Once the 
        // service is available, it can be started.  The pertinent 
        // registry entries are as follows (notes in parenthesis are
        // just information and are not included in the registry):
        //
        // HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\Par24b
        //   DisplayName  = Par24b A/D Device Driver    (DriverDesc)
        //   ErrorControl = 0x00000001                  (SERVICE_ERROR_NORMAL)
        //   Group        = Data Acquisition Devices    (LoadGroup)
        //   ImagePath    = \??\C:\sr\par24b\driver\par24b.sys  (\??\C:DriverPath)
        //   Start        = 0x00000002                  (SERVICE_AUTO_START)
        //   Tag          = 0x00000001                  (TagId)
        //   Type         = 0x00000001                  (SERVICE_KERNEL_DRIVER)
        //

        GetCurrentDirectory( MAX_STR, DriverPath );
        lstrcat( DriverPath,  "\\par24b.sys");
        
        DriverName  = REG_DRIVER_NAME;
        DriverDesc  = REG_DRIVER_DESC;
        LoadGroup   = REG_DRIVER_GROUP;
        TagId       = 0x1L;
        
        SCService = CreateService(
                        SCManager,             // Service Control Manager
                        DriverName,            // name of device driver
                        DriverDesc,            // description of driver
                        SERVICE_ALL_ACCESS,    // full access permission
                        SERVICE_KERNEL_DRIVER, // type is a kernel driver
                        SERVICE_AUTO_START,    // starts at end of booting
                        SERVICE_ERROR_NORMAL,  // give warning, but still boot
                        DriverPath,            // location of .sys file
                        LoadGroup,             // loads in custom A/D group
                        &TagId,                // load order within group
                        NULL,                  // depends on no other services
                        NULL,                  // use default ServiceStartName
                        NULL                   // no password
                       );

        if (!SCService)
                ErrorMessage("Could not create device driver service", 1);




        // Add registry entries for device specific parameters

        ParameterKeyName =
                  "System\\CurrentControlSet\\Services\\Par24b\\Parameters";
        
        PortAddress      = UserAddress;
        PortMode         = UserMode;

        Code = RegCreateKeyEx(
                       HKEY_LOCAL_MACHINE,      // put under this key
                       ParameterKeyName,        // at this location
                       0,                       // reserved
                       NULL,                    // object type of key
                       REG_OPTION_NON_VOLATILE, // keep value when reboot
                       KEY_ALL_ACCESS,          // full access permission
                       NULL,                    // key handle not inheritable
                       &Par24bKey,              // handle to key
                       &Disposition             // indicates key new or existing
                      );

        if (Code != ERROR_SUCCESS)
                ErrorMessage("Failed to create parameter key", 1);


        Code1 = RegSetValueEx(
                    Par24bKey,                      // use this key
                    REG_PARM_PORTADDR,              // value name
                    0,                              // reserved
                    REG_DWORD,                      // value data type
                    &(CONST BYTE)PortAddress,       // ptr to value contents
                    sizeof(ULONG)                   // contents size
                   );
        
        Code2 = RegSetValueEx(
                    Par24bKey,                      // use this key
                    REG_PARM_PORTMODE,              // value name
                    0,                              // reserved
                    REG_DWORD,                      // value data type
                    &(CONST BYTE)PortMode,          // ptr to value contents
                    sizeof(ULONG)                   // contents size
                   );

        
        if ( (Code1 != ERROR_SUCCESS) ||
             (Code2 != ERROR_SUCCESS) )
                printf("Failed writing parameter key values to registry\n");
        
                       
        Code = RegCloseKey( Par24bKey );
        
        if (Code != ERROR_SUCCESS) 
                ErrorMessage("Failed to close parameter key", 1);
        

        

        // Add registry entries for interpreting EventViewer entries
        //
        // Note that these entries are NOT deleted when rmdriver.exe
        // is run.  This is so that System Log entries relating to the
        // Par24b driver can still be interpreted even when the driver
        // is no longer running.  Except for making the System Log
        // entries less informative, deleting these registry entries
        // by hand produces no bad effects.
        //

        EventLogKeyName  =
              "System\\CurrentControlSet\\Services\\EventLog\\System\\Par24b";
        
        TypesSupported   = 0x7L;
        lstrcpy( EventMessageFile, DriverPath );
        lstrcat( EventMessageFile, ";%SystemRoot%\\System32\\IoLogMsg.dll");
                         
        Code = RegCreateKeyEx(
                       HKEY_LOCAL_MACHINE,      // put under this key
                       EventLogKeyName,         // at this location
                       0,                       // reserved
                       NULL,                    // object type of key
                       REG_OPTION_NON_VOLATILE, // keep value when reboot
                       KEY_ALL_ACCESS,          // full access permission
                       NULL,                    // key handle not inheritable
                       &Par24bKey,              // handle to key 
                       &Disposition             // indicates key new or existing
                      );

        if (Code != ERROR_SUCCESS) 
                ErrorMessage("Failed creating Event Log Key", 1);


        Code1 = RegSetValueEx(
                    Par24bKey,                      // use this key
                    "EventMessageFile",             // value name
                    0,                              // reserved
                    REG_EXPAND_SZ,                  // value data type
                    (CONST BYTE*)EventMessageFile,  // ptr to value contents
                    strlen(EventMessageFile)        // contents size
                    );
        
                              
        Code2 = RegSetValueEx(
                    Par24bKey,                      // use this key
                    "TypesSupported",               // value name
                    0,                              // reserved
                    REG_DWORD,                      // value data type
                    &(CONST BYTE)TypesSupported,    // ptr to value contents
                    sizeof(DWORD)                   // contents size
                   );
        
        if ( (Code1 != ERROR_SUCCESS) ||
             (Code2 != ERROR_SUCCESS) )
                printf("Failed writing event log key values to registry\n");
        

        Code = RegCloseKey( Par24bKey );
        if (Code != ERROR_SUCCESS)
                ErrorMessage("Failed closing Event Log Key", 1);




        // Indicate install successful
        
        printf( "\n%s driver has been successfully installed.\n", DriverName );

        
        
        // Start Service

        Ret = StartService( SCService,    // service identifier
                            0,             // number of arguments
                            NULL           // pointer to arguments
                          );

        if (!Ret)
                ErrorMessage("Failed to start PAR24B device driver service", 1);
        else
                printf( "\n%s driver was started using port address 0x%X.\n",
                        DriverName, PortAddress );
        

        // Close handles (leaving service loaded and running)

        CloseServiceHandle( SCService );
        CloseServiceHandle( SCManager );


}

void ErrorMessage( char *msg, int fatal ) {

        DWORD  Err;
        LPVOID lpMessageBuffer;
        
        // Display local message, error number, and system message
        
        Err = GetLastError();

        FormatMessage(
                      FORMAT_MESSAGE_ALLOCATE_BUFFER |
                      FORMAT_MESSAGE_FROM_SYSTEM,
                      NULL,
                      Err,
                      MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
                      (LPTSTR) &lpMessageBuffer,
                      0,
                      NULL
                     );

        printf( "\n%s.  Error #%d\n\n%s\n", msg, Err, lpMessageBuffer );

        LocalFree( lpMessageBuffer );


        
        // Give additional explanation for common errors
        
        if (Err == ERROR_SERVICE_EXISTS) { // #1073
                printf( "\tNote: Use rmdriver.exe to clear out\n" );
                printf( "\tthe previous version and try again.\n" );
                }
        
        if (Err == ERROR_ACCESS_DENIED) { // #5
                printf( "\tThis error indicates the Par24b device driver could not start\n" );
                printf( "\tbecause no port was found at the requested port address.\n" );
                printf( "\t\n" );
                printf( "\tPossible causes include:\n" );
                printf( "\t 1. An incorrect port address was given.\n" );
                printf( "\t 2. The Printer Port and Parallel Class Enumerator drivers\n" );
                printf( "\t    needed for Win2K are disabled or uninstalled.\n" );
                printf( "\t 3. The port is already in use by a printer driver.\n" );
                printf( "\t\n" );
                printf( "\tCheck the BIOS CMOS setup, the Device Manager and the Printers\n" );
                printf( "\tfolder to help determine and correct the problem.  Then use\n" );
                printf( "\trmdriver to clear out the failed install of the Par24b device\n" );
                printf( "\tdriver and try again.\n" );
                }
        
        if (Err == ERROR_NO_SYSTEM_RESOURCES) { // #1450
                printf( "\tThis error most likely indicates that the Par24b\n" );
                printf( "\tdevice driver could not start because another\n" );
                printf( "\tdevice was using the requested port addresses.\n" );
                printf( "\t\n" );
                printf( "\tOnce the conflict is resolved, you can start\n" );
                printf( "\tthe Par24b device driver by typing:\n" );
                printf( "\tnet start par24b\n" );
                printf( "\t\n" );
                printf( "\tPlease examine the System Log in the EventViewer\n" );
                printf( "\tfor more details on the resource conflict.  To\n" );
                printf( "\taccess the EventViewer, type eventvwr at the\n" );
                printf( "\tCommandPrompt or select Programs > Administrative\n" );
                printf( "\tTools > EventViewer from the Start menu.\n" );
                }


        // Clean up and exit on fatal errors
        
        if (fatal) {
                if (SCService)  CloseServiceHandle( SCService );
                if (SCManager)  CloseServiceHandle( SCManager );
                exit( 1 );
                }
        
}
