// FILE: rmdriver.c                Copyright (c), Symmetric Research, 1999
//
// Program to remove par24b kernel mode driver.
//
// This is done by starting the Service Control Manager, opening,
// stopping, and deleting the driver service.  On deletion, driver
// information that was added to the registry under the driver's key
// is also deleted.  Additional registry entries, added to allow 
// Par24b driver messages to be properly interpreted by the system
// log EventViewer, are not deleted so that existing system log messages
// can still be deciphered.
//
// Refer to the readme.txt file for information about how to remove
// the device driver manually.


#include <windows.h>
#include <stdio.h>

#include "par24bkd.h"   // for REG_DRIVER constants


// Helper function prototype

void ErrorMessage( char *msg, int fatal );


// Global variables shared by main and helper function

SC_HANDLE SCManager, SCService;



void main( int argc, char**argv ) {

        BOOL            Ret;
        LPCTSTR         DriverName;
        SERVICE_STATUS  ServiceStatus;
        char            Choice;

        
        // Check for proper number of command line arguments

        if (argc != 2 ) {

                printf( "This function removes the PAR24B driver.\n" );
                printf( "  Usage: rmdriver [y]\n" );
                printf( "         if no command line 'y' is found, " );
                printf( "an interactive prompt is given\n\n" );
                printf( "Do you want to remove the PAR24B driver? (y or n):  ");

                scanf( "%c", &Choice );
                }
        else
                Choice = *argv[1];


        if ( (Choice != 'y') && (Choice != 'Y') )
                exit( 0);




        // Open Service Control Manager

        SCManager = NULL;
        SCService = NULL;
        
        SCManager = OpenSCManager(
                        NULL,                  // use local machine
                        NULL,                  // use SERVICES_ACTIVE_DATABASE
                        SC_MANAGER_ALL_ACCESS  // full access permission
                        );

        if (!SCManager)
                ErrorMessage("Could not create service manager", 1);
        



        // Open the device driver service
        
        DriverName = REG_DRIVER_NAME;
        
        SCService = OpenService(
                        SCManager,             // Service Control Manager
                        DriverName,            // name of device driver
                        SERVICE_ALL_ACCESS     // full access permission
                        );

        if (!SCService)
                ErrorMessage("Could not open device driver service", 1);




        // Stop Service if not already stopped

        Ret = QueryServiceStatus(
                             SCService,              // device driver service
                             &ServiceStatus          // status
                            );
        
        if (!Ret)
                ErrorMessage("Could not query device driver status", 0);
        
        else if (ServiceStatus.dwCurrentState != SERVICE_STOPPED) {

                Ret = ControlService(
                             SCService,              // device driver service
                             SERVICE_CONTROL_STOP,   // request STOP
                             &ServiceStatus          // status
                            );

                if (!Ret)
                        ErrorMessage("Could not stop device driver", 0);
                else
                        printf( "\n%s driver has been stopped.\n", DriverName );
                }

        
        
        // Delete Service
        //
        // This function both removes the device driver from the
        // services database and removes the driver's registry key.
        // Because the entire key is deleted, both the registry entries
        // automatically generated by the CreateService call AND the
        // explicitly created Parameters subkey registry entries are
        // deleted.

        Ret = DeleteService( SCService );

        if (!Ret)
                ErrorMessage("Could not delete device driver", 1);
        else
                printf( "\n%s driver has been successfully removed.\n", DriverName );


        
        // Close handles

        CloseServiceHandle( SCService );
        CloseServiceHandle( SCManager );



        
#ifdef DELETE_EVENTLOG_REGISTRY_ENTRIES

        // Remove associated registry entries
        //
        // Do not delete EventLog registry entries explicitly
        // created with indriver.exe.  This is so that System Log 
        // entries relating to the Par24b driver can still be 
        // interpreted even when the driver is no longer running.  
        // Except for making the System Log entries less informative,
        // deleting these registry entries by hand produces no bad effects.
        //
        // HKEY_LOCAL_MACHINE\System\CurrentControlSet\Services\EventLog\
        //                                                        System\Par24b
        //   EventMessageFile = C:\SR\PAR24B\DRIVER\par24b.sys;SystemRoot%\System32\IoLogMsg.dll
        //   TypesSupported   = 0x7L;
        //
        
        // By default, the code below is not compiled.  However, if you wish 
        // to have rmdriver.exe automatically delete the EventLog registry
        // entries, you can remove the #ifdef/#endif statements and move 
        // the three variable definitions to the top of main and recompile.

        LONG    Code;
        HKEY    ParentKey;
        LPCTSTR ParentKeyName;

        ParentKeyName = "System\\CurrentControlSet\\Services\\EventLog\\System";
                               
        Code = RegOpenKeyEx(
                            HKEY_LOCAL_MACHINE,      // top level key
                            ParentKeyName,           // event log parent key
                            0,                       // reserved
                            KEY_ALL_ACCESS,          // full access permission
                            &ParentKey               // handle to key
                           );

        if (Code != ERROR_SUCCESS)
                ErrorMessage("Could not open EventLog parent key", 1);


        Code = RegDeleteKey(
                            ParentKey,  // EventLog system file key
                            DriverName  // driver subkey to delete
                           );
        
        if (Code != ERROR_SUCCESS)
                ErrorMessage("Could not delete Par24b EventLog subkey", 0);

        
        Code = RegCloseKey( ParentKey );
        
        if (Code != ERROR_SUCCESS)
                ErrorMessage("Could not close EventLog parent key", 1);
#endif        
        
}

void ErrorMessage( char *msg, int fatal ) {

        DWORD Err;
        LPVOID lpMessageBuffer;
        
        // Display local message, error number, and system message
        
        Err = GetLastError();

        FormatMessage(
                      FORMAT_MESSAGE_ALLOCATE_BUFFER |
                      FORMAT_MESSAGE_FROM_SYSTEM,
                      NULL,
                      Err,
                      MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
                      (LPTSTR) &lpMessageBuffer,
                      0,
                      NULL
                     );

        printf( "\n%s.  Error #%d\n\n%s\n", msg, Err, lpMessageBuffer );

        LocalFree( lpMessageBuffer );



        
        // Give additional explanation for common errors
        
        if (Err == ERROR_INVALID_SERVICE_CONTROL) { // #1052
                printf( "\tThis error most likely indicates that the %s\n", REG_DRIVER_NAME );
                printf( "\tdevice driver could not be stopped because it\n" );
                printf( "\twas not currently running.  Removal continues.\n" );
                }

        if (Err == ERROR_SERVICE_DOES_NOT_EXIST) { // #1060
                printf( "\tThe %s device driver must be installed before\n", REG_DRIVER_NAME );
                printf( "\tit can be removed.  Use indriver.exe to install it.\n" );
                }
        

        // Clean up and exit on fatal errors
        
        if (fatal) {
                if (SCService)  CloseServiceHandle( SCService );
                if (SCManager)  CloseServiceHandle( SCManager );
                exit( 1 );
                }
}
