/* FILE: meter.c                Copyright (c), Symmetric Research, 1998-2001
 *
 * Command line program to demonstrate meter style acquisition with text
 * output.
 *
 * Parameters like sampling rate and channel calibrations are defined
 * constants below.  Recompile if you change these values.
 * 
 * Main is at the very bottom of the file.  It has the general sequence
 * of steps that the program runs through.
 *
 *
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>

#ifndef SROS_LINUX
#include <conio.h>
#endif

#include "par24b.h"






/* REV MESSAGE:
 *
 *
 *
 */
 
void Rev( void ) {

        /* Print the rev info. */

        printf( "PAR24B meter : (c) SR 1999-2001 : Rev {12/15/01}\n");

}


/* COMMAND LINE USAGE AND ERROR MESSAGES:
 *
 *
 *
 */

void Usage( void) {

        /* Print a command line usage message and quit. */

        printf( "USAGE: meter {counts,volts} [{bpp,epp,ecp/bpp,ecp/epp}]\n" );

        exit( 1);

}
void Error( char *msg) {

        /* Print and error message and quit. */

        printf( "ERROR: %s\n", msg);

        exit( 1);

}






/* PAR24B INITIALIZATION CONFIGURATION:
 *
 * These parameters have the overall configuration of the system like
 * the parallel port used and sampling rate.
 *
 * Setting SPS = 10.0 Hz and then averaging over NAVERAGE = 10 points
 * gives an effective sampling rate of 1 Hz.
 *
 * The number of channels is fixed at 8, don't change it ...
 *
 *
 *
 *
 */

#define NCHANNELS               8

#define PORT_ADDRESS            0x378
#define PORT_MODE               PAR24B_PORT_MODE_EPP

#define SPS                     10.0         /* samples per second */
#define NAVERAGE                10



        


/* VOLTAGE CALIBRATION INFO:
 *
 * When displaying the output as volts, a counts to voltage conversion
 * must be performed.  To perform this conversion, the system must be
 * calibrated.
 *
 * The formulas here do a simple straight line conversion.  Given the
 * counts at two different voltages, the straight line through the two
 * points is computed.
 * 
 * This of course could be enhanced by computing a least squares fit
 * line to many sampled points, but the two point formula here works
 * fairly well and is easy to implement.  Because the system has eight
 * independent A/D converters, a calibration formula is carried for each
 * channel.
 * 
 * To calibrate your system, give each channel a known voltage, V1, and
 * running in counts mode, record the counts for each channel.  Repeat
 * for a second voltage V2.  Enter the results in the defined constants
 * below and recompile.  The quality of the result of course depends on
 * the precision and stability of your voltage references V1, and V2.
 * 
 * 
 * 
 */

#define V1                      (double)0.000

#define C1_CHAN0                0x007FFD60L
#define C1_CHAN1                0x007FFD27L
#define C1_CHAN2                0x007FFC88L
#define C1_CHAN3                0x007FFCF0L
#define C1_CHAN4                0x007FFD51L
#define C1_CHAN5                0x007FFD32L
#define C1_CHAN6                0x007FFD8AL
#define C1_CHAN7                0x007FFC2FL

#define V2                      (double)+1.000

#define C2_CHAN0                0x008CB580L
#define C2_CHAN1                0x008CC500L
#define C2_CHAN2                0x008CC580L
#define C2_CHAN3                0x008CAB00L
#define C2_CHAN4                0x008CB580L
#define C2_CHAN5                0x008CB300L
#define C2_CHAN6                0x008CCD00L
#define C2_CHAN7                0x008CAE80L



/* slopes for each channel ... */

#define SL_CHAN0                (V2-V1)/(C2_CHAN0 - C1_CHAN0)
#define SL_CHAN1                (V2-V1)/(C2_CHAN1 - C1_CHAN1)
#define SL_CHAN2                (V2-V1)/(C2_CHAN2 - C1_CHAN2)
#define SL_CHAN3                (V2-V1)/(C2_CHAN3 - C1_CHAN3)
#define SL_CHAN4                (V2-V1)/(C2_CHAN4 - C1_CHAN4)
#define SL_CHAN5                (V2-V1)/(C2_CHAN5 - C1_CHAN5)
#define SL_CHAN6                (V2-V1)/(C2_CHAN6 - C1_CHAN6)
#define SL_CHAN7                (V2-V1)/(C2_CHAN7 - C1_CHAN7)


/* formulas for each channel ... */

#define CtoV_CHAN0( C)          ( SL_CHAN0*(C - C2_CHAN0) + V2 )
#define CtoV_CHAN1( C)          ( SL_CHAN1*(C - C2_CHAN1) + V2 )
#define CtoV_CHAN2( C)          ( SL_CHAN2*(C - C2_CHAN2) + V2 )
#define CtoV_CHAN3( C)          ( SL_CHAN3*(C - C2_CHAN3) + V2 )
#define CtoV_CHAN4( C)          ( SL_CHAN4*(C - C2_CHAN4) + V2 )
#define CtoV_CHAN5( C)          ( SL_CHAN5*(C - C2_CHAN5) + V2 )
#define CtoV_CHAN6( C)          ( SL_CHAN6*(C - C2_CHAN6) + V2 )
#define CtoV_CHAN7( C)          ( SL_CHAN7*(C - C2_CHAN7) + V2 )








/* DISPLAY FUNCTIONS:
 *
 *
 *
 *
 *
 */

#define COUNTS          0
#define VOLTS           1

#define CFMT  "%08lX, %08lX, %08lX, %08lX, %08lX, %08lX, %08lX, %08lX\n"
#define VFMT  "%8.4f, %8.4f, %8.4f, %8.4f, %8.4f, %8.4f, %8.4f, %8.4f\n"


void Display( int DisplayMode, long counts[]) {

        /* Display the data from 8 channels. */

        if ( DisplayMode == COUNTS ) {

                printf( CFMT,
                                counts[0],
                                counts[1],
                                counts[2],
                                counts[3],
                                counts[4],
                                counts[5],
                                counts[6],
                                counts[7]

                              );
                }

        else { /* DisplayMode == VOLTS */

                printf( VFMT,
                                CtoV_CHAN0( counts[0]),
                                CtoV_CHAN1( counts[1]),
                                CtoV_CHAN2( counts[2]),
                                CtoV_CHAN3( counts[3]),
                                CtoV_CHAN4( counts[4]),
                                CtoV_CHAN5( counts[5]),
                                CtoV_CHAN6( counts[6]),
                                CtoV_CHAN7( counts[7])

                              );
                }

}





/* KEYBOARD HELPER FUNCTION:
 *
 *
 *
 */
        
void KeyCheck( void ) {

        int c;


        /* Check for a keypress, quitting if found. */

        c = GetKeyCheck( );

        if ( c != EOF ) {

                PAR24B_Stop();
                PAR24B_Close();
                exit( 0 );
                }
}




/* STRING HELPER FUNCTION:
 *
 *
 *
 */

int StrEquals( char *src, char *dest) {


        /* Compare two strings for exact equality. */

        while ( *src && *dest && *src == *dest )

                { src++; dest++; }


        return ( *src == '\0' && *dest == '\0' ? 1 : 0);

}







/* MAIN:
 *
 * Initialize the PAR24B and start acquisition.
 *
 *
 *
 */

int main( int argc, char *argv[]) {

        char StrDisplayMode[128]; int DisplayMode, PortMode;

        int FilterPoint; unsigned int TrueFifoSize; long HI7190_CrValue;

           long counts[NCHANNELS], averages[NCHANNELS]; int naverage;


        int i;


        /* Display the rev. */

        Rev();


#ifdef SROS_LINUX

        if ( SetNonBlock( 1 ) == -1 )
                { printf( "Can't set non-blocking read\n"); exit( 1 ); }
#endif


        /* Get the DisplayMode from the command line. */

        if ( (argc != 2) && (argc != 3) )

                Usage();


        sscanf( argv[1], "%s", StrDisplayMode);

        if ( StrEquals( StrDisplayMode, "counts") )

                DisplayMode = COUNTS;

        else if ( StrEquals( StrDisplayMode, "volts") )

                DisplayMode = VOLTS;

        else
                Usage();



        
        // Get the optional PortMode from the command line.
        
        if ( argc < 3 )
                PortMode = PORT_MODE;

        else {                        
                if ( StrEquals( argv[2], "bpp" ) )
                        
                        PortMode = PAR24B_PORT_MODE_BPP;
                
                else if ( StrEquals( argv[2], "epp" ) )
                        
                        PortMode = PAR24B_PORT_MODE_EPP;
                
                else if ( StrEquals( argv[2], "ecp/bpp" ) )
                        
                        PortMode = PAR24B_PORT_MODE_ECP_BPP;
                
                else if ( StrEquals( argv[2], "ecp/epp" ) )
                        
                        PortMode = PAR24B_PORT_MODE_ECP_EPP;

                else
                        Usage();
                }





        /* Initialize the PAR24B. */

        PAR24B_SpsToFp( SPS, &FilterPoint );

        if ( !PAR24B_Init(
                
                        PORT_ADDRESS,
                        PortMode,
                        &TrueFifoSize,
                        PAR24B_DC_OFFSET,
                        FilterPoint,
                        PAR24B_GAIN_1,
                        PAR24B_CAL_MODE_SELF,
                        &HI7190_CrValue,
                        NULL

                        )

             ) { printf( "Initialization failed\n"); exit( 1 ); }

        else
                printf( "Initialization successful.  Press ENTER to quit.\n");



        
        /* Initialize the naverage count, and averages for each channel. */

        naverage = 0;

        for ( i = 0 ; i < NCHANNELS ; i++ )

                averages[ i] = 0;
        


        
        /* Start acquisition */

        PAR24B_Start();

polling_loop:



        /* If no data ready, check for key to quit and continue */

        if ( PAR24B_IsEmpty() )

                { KeyCheck(); goto polling_loop; }       



        
        /* Get the data that is ready */

        PAR24B_ReadFifo( counts);

        

        /* Average in the new sample from each channel. */

        for ( i = 0 ; i < NCHANNELS ; i++ )

                averages[i] += counts[i];



        /* Increment the number of samples averaged. */

        naverage++;

                        

        /* If the last sample has been averaged, normalize and display. */

        if ( naverage >= NAVERAGE ) {


                /* Normalize. */

                for ( i = 0 ; i < NCHANNELS ; i++ )

                        averages[i] /= NAVERAGE;


                /* Display the count averages for each channel. */

                Display( DisplayMode, averages);



                /* Reset the naverage count and averages. */

                naverage = 0;
                
                for ( i = 0 ; i < NCHANNELS ; i++ )

                        averages[i] = 0;

                }


                        

        /* Check for a keypress to quit. */

        KeyCheck();
        


        /* Continue looping. */

        goto polling_loop;


        return( 0 );
}
