// FILE: 24b.cpp                Copyright (c), Symmetric Research, 1997
//
// Source file for PAR24B specific portion of the Scope program
//
//	C24bApp
//	C24bWin
//	CHardwareDlg
//	CChannelDlg
//	CAcquireDlg
//
//

#include <io.h>		// needed for _access

#include "24b.h"


extern "C" {
#include "par24b.h"
}



// Global application object

C24bApp APP;


// General pointer to it (defined in scope.cpp)

extern CScopeApp *pAPP;



//***********************************************************************
//
// CLASS C24bApp
//
// This class, derived from CScopeApp, contains the PAR24B specific
// application portion of the Scope data acquisition program.  It works
// in conjunction with its parent class to deal with acquisition, files,
// and the registry.  The C24bWin class takes care of PAR24B specific
// main window functions.
//
//


BEGIN_MESSAGE_MAP(C24bApp, CScopeApp)
	//{{AFX_MSG_MAP(C24bApp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



int C24bApp::InitAppData() {
	
	TRACE0("C24bApp::InitAppData\n");

	unsigned int nvmax, nvtyp;
	double       dminsps, dmaxsps;

		

	// Initialize common global constants

	CScopeApp::InitAppData();


	

	// Set specific global application pointer

	pAPP = this;







	// Initialize max and min values and allocate scan arrays

	AppTitle.LoadString( IDR_SCOPE );
	
	PAR24B_FpToSps( PAR24B_FP_MAX, &dminsps);	// = 9.541F
	PAR24B_FpToSps( PAR24B_FP_MIN, &dmaxsps);	// = 1953.125F
	
	PAR24B_FifoSizeToNvalues(PAR24B_FIFO_SIZE_MAX, &nvmax);
	PAR24B_FifoSizeToNvalues(PAR24B_FIFO_SIZE_TYP, &nvtyp);

	DataPtSize  = sizeof(long);
	ROnsamples  = TRUE;

	MaxChannel  = PAR24B_CHANNELS;
	MaxSample   = (int)nvmax/MaxChannel;
	MinAmpShift = 0;
	MaxAmpShift = 24;
	MinSPS      = (float)dminsps;
	MaxSPS      = (float)dmaxsps;

	DefaultColor    = CS_GREEN;
	DefaultNdisplay = MaxChannel;
	DefaultNsamples = (int)nvtyp/MaxChannel;
	DefaultNblock   = 500 / DefaultNsamples + 1;
	
	Nplotbuffers = DefaultNblock;
	

	

	// Initialize additional specific constants
	
	AcqDataCoding = PAR24B_DC_OFFSET;
	pDataArray    = NULL;

	return( 1 );
}


CScopeWindow* C24bApp::NewScopeWindow() {

	TRACE0("C24bApp::NewScopeWindow\n");

	CScopeWindow* pWnd = (CScopeWindow*) new C24bWin;

	return( pWnd );
}

int C24bApp::ExitInstance() {

	TRACE0("C24bApp::ExitInstance\n");

		
	// Clean up any existing data array

	if (pDataArray)
		{ free(pDataArray); pDataArray = NULL; }

				
	return( CScopeApp::ExitInstance() );
}

int C24bApp::Par24bCorrectSps( float *sps ) {

	TRACE0("C24bApp::Par24bCorrectSps\n");
	
	int    fp1,  fp2;
	double sps0, sps1, sps2, diff1, diff2;
	
	sps0 = (double)*sps;
	
	PAR24B_SpsToFp( sps0, &fp1 );
	
	fp2 = fp1 + 1;
	
	PAR24B_FpToSps( fp1, &sps1 );
	PAR24B_FpToSps( fp2, &sps2 );

	diff1 = sps1 - sps0;
	diff2 = sps0 - sps2;

	if (diff1 < diff2) {
		*sps = (float)sps1;
		return(fp1);
		}
	else {
		*sps = (float)sps2;
		return(fp2);
		}

}


int C24bApp::AtodMeasure( int *ival ) {

	TRACE0("C24bApp::AtodMeasure\n");

	unsigned int fifosize;
	int          present, errorcode;
	long         ctrlreg;
	

	present = PAR24B_Init( HardPortAddr, HardPortMode, &fifosize, 
				 PAR24B_DC_OFFSET, PAR24B_FP_MIN,
				 PAR24B_GAIN_1, PAR24B_CAL_MODE_SELF, 
				 &ctrlreg, &errorcode );

	if (present)
		*ival = (int)fifosize;
	else
		*ival = 0;

	return( present );
}

int C24bApp::AtodInit() {
	
	TRACE0("C24bApp::AtodInit\n");

	unsigned int fifosize;
	int          init, fp, errorcode;
	long         ctrlreg;


	// Init A/D

	fp = Par24bCorrectSps( &AcqSPS );
	
	init = PAR24B_Init( HardPortAddr, HardPortMode, &fifosize,
				AcqDataCoding, fp, AcqGain, HardCalibration,
				&ctrlreg, &errorcode );



	// Set specific error information

	if (init && (fifosize != (unsigned int)HardFIFOsize)) {
	
		init = 0;
		ErrString = "Actual vs specified FIFO size mismatch.\n";
		}
		
	else if ( (errorcode >= PAR24B_IERROR_DC_OFFSET) &&
		  (errorcode <= PAR24B_IERROR_CAL_MODE)  )
		  
		ErrString = "Invalid parameter (dc,fp,gain,cal).\n";
		
	else if (errorcode != PAR24B_IERROR_SUCCESS)
	
		ErrString = "Check power, cable, and port address.\n";
		
	else
		ErrString = "";



	// Set user byte or general error info
							
	if (init)
		PAR24B_UserByte( AcqLedValue );
	else
		ErrString += "Could not initialize the PAR24B board.\n";


	return( init );
}


int C24bApp::AtodStart() {
	
	TRACE0("C24bApp::AtodStart\n");


	// Start A/D execution

	PAR24B_Start( );
	

	return( 1 );
}


int C24bApp::AtodCheckReady() {
	
	TRACE0("C24bApp::AtodCheckReady\n");

	return( PAR24B_IsReady() );
}


int C24bApp::AtodCheckOverflow() {
	
	TRACE0("C24bApp::AtodCheckOverflow\n");

	return( PAR24B_Overflow() );
}


int C24bApp::AtodGetData() {
	
	TRACE0("C24bApp::AtodGetData\n");

	int iret;

		
	// Twiddle the yellow LED to show its working

	AcqLedValue ^= 1;

	PAR24B_UserByte( AcqLedValue );


	
	// Read the data
	
	iret = PAR24B_GetData( pDataArray, (unsigned int) Nbufferpts );

	
	return( iret );
}


int C24bApp::AtodDemuxData() {

	TRACE0("C24bApp::AtodDemuxData\n");


	int i, j, na, nc, ns, kmux, kdmx, startpt;


	
	// Demux raw data values into the plottable demuxed array

	startpt = Iplotbuff * AcqNsamples;


	ns = AcqNsamplesPlot;
	na = AcqNsamples;
	nc = AcqNchannels;


	for ( j = 0 ; j < nc ; j++ ) {

		pTMPoffset[j] = 0.0;

		for ( i = 0 ; i < na ; i++ ) {

			kmux = i*nc+j;
			kdmx = j*ns+i+startpt;

			pDemuxArray[kdmx] = pDataArray[kmux];

			pTMPoffset[j] += (double)pDemuxArray[kdmx];
			
			}
		}



	// Use the offset computed from the first AD buffer for
	// all parts of the plot buffer
		
	if (ResetDC == TRUE) {

		// If AutoDC is TRUE, then always do ResetDC
		
		if (AutoDC == FALSE)
			ResetDC = FALSE;
			
		for ( j = 0 ; j < nc ; j++ )
			pDCoffset[j] = (long)(pTMPoffset[j] / (double)na);

		}

	
	return( 1 );
}

int C24bApp::AtodStop() {
	
	TRACE0("C24bApp::AtodStop\n");


	PAR24B_Stop();

	return( 1 );
}


int C24bApp::AllocateDataArray( void ) {

	TRACE0("C24bApp::AllocateDataArray\n");

	// Clean up any previous array and allocate new one
	
	if (pDataArray)
		{ free(pDataArray); pDataArray = NULL; }

	pDataArray = (long*) malloc(Nbufferbytes);

	

	if ( pDataArray )
		return( 1 );
	else
		return( 0 );
}


int C24bApp::TriggerCondition( void ) {

	TRACE0("C24bApp::TriggerCondition\n");

	// This function is called by the OnTimer function to decide
	// whether or not the currently acquired buffer of data should
	// be written out.
	//
	// Currently, this function always returns 1, so the acquired 
	// data is always saved.  However, you can modify this function
	// to perform any triggering computations required by your 
	// application.  Common algorithms include simple threshold
	// detection and sta/lta detection.
	//
	// Because this function is a member of the C24bApp class which
	// is derived from the CScopeApp class you have access to all the
	// member variables of both classes.  In particular, you may wish
	// to use
	//
	//	pDataArray	a pointer to the raw data
	//	AcqNchannels	the number of channels
	//	AcqNsamples	the number of samples per channel
	
	return( 1 );
}

	
int C24bApp::FillHdr() {


	// Fill in general header info

	CScopeApp::FillHdr();

	


	// Fill in 24b specific header info

	strcpy(Hdr.Id, "SR PAR24B");

	Hdr.DataCoding = AcqDataCoding;
	Hdr.Gain       = (double)(1<<AcqGain); 


	return( 1 );
}


int C24bApp::WriteBytes( CFile *outfile ) {

	TRACE0("C24bApp::WriteBytes\n");

	// Ensure valid file pointer
	
	if (!outfile) {
		ErrString = "Invalid outfile";
		ShowError();
		return( 0 );
		}


	// Write out 24B specific data bytes
	
	outfile->Write( pDataArray, Nbufferbytes );
	return( 1 );
}



int C24bApp::VerifyAppData() {

	TRACE0("C24bApp::VerifyAppData\n");

	unsigned int nv;
	int          i, fp;
	char         SigString[MAXSTR];

	

	// Verify common data
	
	CScopeApp::VerifyAppData();




	// Hardware parameter verification
	

	if ((HardPortMode != PAR24B_PORT_MODE_EPP)     &&
            (HardPortMode != PAR24B_PORT_MODE_BPP)     &&
            (HardPortMode != PAR24B_PORT_MODE_ECP_EPP) &&
            (HardPortMode != PAR24B_PORT_MODE_ECP_BPP)
           )
		HardPortMode = PAR24B_PORT_MODE_EPP;

	if (HardPortAddr < 0)
		HardPortAddr = PAR24B_PORT_ADDRESS_1;

	if ((HardCalibration<PAR24B_CAL_MODE_NONE) || (HardCalibration>PAR24B_CAL_MODE_SOIG))
		HardCalibration = PAR24B_CAL_MODE_SELF;
		
	if ((HardFIFOsize<PAR24B_FIFO_SIZE_MIN) || (HardFIFOsize>PAR24B_FIFO_SIZE_MAX))
		HardFIFOsize = PAR24B_FIFO_SIZE_TYP;

		
	// Update variables that depend on HardFIFOsize

	PAR24B_FifoSizeToNvalues( (unsigned int)HardFIFOsize, &nv);

	if (AcqNchannels > 0)
		AcqNsamples = (int)nv / AcqNchannels;
	
	if (AcqNsamples > 0)
		Nplotbuffers = 500 / AcqNsamples + 1;


		

		
	// Channel parameter verification

	for ( i = 0 ;  i < MaxChannel ; i++ ) {

		if (strlen(&pScanTitle[i*MAXTITLE]) > MAXTITLE) {
		
			sprintf( SigString, "CH%02d", i );
			strcpy( &pScanTitle[i*MAXTITLE], SigString );
			}

		// All channels are always scanned
		
		if (pScanActive[i] != CHECKED)
			pScanActive[i] = CHECKED;


		}


	    	    

	    
	// Acquisition parameter verification

	
	if ((AcqLedValue != 0) && (AcqLedValue != 1))
		AcqLedValue = 0;

	if ((AcqGain < PAR24B_GAIN_1) || (AcqGain > PAR24B_GAIN_128))
		AcqGain = PAR24B_GAIN_1;

	if ((AcqSPS < MinSPS) || (AcqSPS > MaxSPS))
		AcqSPS = 30.0F;


	fp = Par24bCorrectSps( &AcqSPS );



	return( 1 );
}


int C24bApp::RegistrySettings( int direction, int sectid ) {

	int i;

	static char SigString[MAXSTR], TitleString[MAXSTR], OnOffString[MAXSTR];



	// Create/Open Section Key
	
	if ( !RegistryOpenKey( sectid ) )
		return( 0 );





	// First work with common settings

	RegistryCommonSettings( direction, sectid );



		

	// Hardware Dialog Registry Settings
		
	if (sectid == REG_SECT_HARD) {

	    if (direction == REG_DIR_GET) {
		
		GetRegLong( "PortMode",   (long*)&HardPortMode,    PAR24B_PORT_MODE_EPP  );
		GetRegLong( "PortAddr",   (long*)&HardPortAddr,    PAR24B_PORT_ADDRESS_1 );
		GetRegLong( "Calibration",(long*)&HardCalibration, PAR24B_CAL_MODE_SELF  );
		GetRegLong( "FIFOsize",   (long*)&HardFIFOsize,    PAR24B_FIFO_SIZE_TYP  );

		}

	    else if (direction == REG_DIR_SAVE) {

		SaveRegLong( "PortMode",    HardPortMode    );
		SaveRegLong( "PortAddr",    HardPortAddr    );
		SaveRegLong( "Calibration", HardCalibration );
		SaveRegLong( "FIFOsize",    HardFIFOsize    );
			
		}

	    }


		


	// Channel Dialog Registry Settings
	
	else if (sectid == REG_SECT_CHAN) {

	    if (direction == REG_DIR_GET) {
			
		for ( i = 0 ;  i < MaxChannel ; i++ ) {

		    sprintf( SigString,   "CH%02d",   i );
		    sprintf( TitleString, "Title_CH%02d", i );
		    sprintf( OnOffString, "OnOff_CH%02d", i );
		    
		    GetRegChar( TitleString, &pScanTitle[i*MAXTITLE], SigString );
		    GetRegLong( OnOffString, &pScanActive[i],         CHECKED   );

		    }

		}

	    
	    else if (direction == REG_DIR_SAVE) {
			
		for ( i = 0 ;  i < MaxChannel ; i++ ) {

		    sprintf( TitleString, "Title_CH%02d", i );
		    sprintf( OnOffString, "OnOff_CH%02d", i );

		    SaveRegChar( TitleString, &pScanTitle[i*MAXTITLE] );
		    SaveRegLong( OnOffString, pScanActive[i] );
			
		    }
			
		}


	    }




	
	// Acquire Dialog Registry Settings
		
	else if (sectid == REG_SECT_ACQ) {
		
	    if (direction == REG_DIR_GET) {
		    
		GetRegLong(  "LEDvalue",  (long*)&AcqLedValue,  0             );
		GetRegLong(  "Gain",      (long*)&AcqGain,      PAR24B_GAIN_1 );
		GetRegFloat( "SPS",              &AcqSPS,       30.0F         );

		}

	    else if (direction == REG_DIR_SAVE) {

    		SaveRegLong(  "LEDvalue",  AcqLedValue  );
		SaveRegLong(  "Gain",      AcqGain      );
		SaveRegFloat( "SPS",       AcqSPS       );
			
		}

	    }






	
	// Close Section Key

	RegCloseKey( hkeySect );
	
	return( 1 );		
	
}







//***********************************************************************
//
// CLASS C24bWin
//
// This class, derived from CScopeWindow, contains the PAR24B specific
// main window portion of the Scope data acquisition program.  It primarily
// controls dialogs specific to the PAR24B while its parent class handles
// the bulk of the graphics drawing, menu operations, and main window
// functions.
//


BEGIN_MESSAGE_MAP( C24bWin, CScopeWindow )
	//{{AFX_MSG_MAP(C24bWin)
	ON_COMMAND(IDM_OPTIONS_HARDWARE,  OnHardwareDialog)
	ON_COMMAND(IDM_OPTIONS_CHANNEL,   OnChannelDialog)
	ON_COMMAND(IDM_OPTIONS_ACQUIRE,   OnAcquireDialog)
	ON_UPDATE_COMMAND_UI(IDM_OPTIONS_HARDWARE,  OnUpdateMenuStopped)
	ON_UPDATE_COMMAND_UI(IDM_OPTIONS_CHANNEL,   OnUpdateMenuStopped)
	ON_UPDATE_COMMAND_UI(IDM_OPTIONS_ACQUIRE,   OnUpdateMenuStopped)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



C24bWin::C24bWin()
	:CScopeWindow() {  // parent constructor

	TRACE0("C24bWin::C24bWin\n");
}

C24bWin::~C24bWin() {

	TRACE0("C24bWin::~C24bWin\n");
}


void C24bWin::PostNcDestroy() {

	TRACE0("C24bWin::PostNcDestroy\n");

	// call grandparent function to delete "this"

	CFrameWnd::PostNcDestroy();
}


void C24bWin::OnHardwareDialog() {

	TRACE0("C24bWin::OnHardwareDialog\n");

	int oldFifoSize;


	oldFifoSize = APP.HardFIFOsize;
	
	CHardwareDlg dlg(this);

	if (dlg.DoModal() == IDOK) {
	
		if (oldFifoSize != APP.HardFIFOsize) {
			pAPP->IsAllocated = FALSE;
			pAPP->ComputeBufferSizes();
			if (!ComputePlotPositions())
				return;
				
			SetScrollBar( SB_HORZ );
			
			pAPP->ResetPaint = TRUE;
			Invalidate();
			}
		
		}
		
	controlkey = FALSE;

}


void C24bWin::OnChannelDialog() {

	TRACE0("C24bWin::OnChannelDialog\n");

		
	CChannelDlg dlg(this);

	if (dlg.DoModal() == IDOK) {

		CClientDC dc(this);
		ComputeMaxTitleLength( &dc );
		if (!ComputePlotPositions())
			return;
		
		SetScrollBar( SB_BOTH );
		
		pAPP->ResetPaint = TRUE;
		Invalidate();

		}
	
		
	controlkey = FALSE;
}

void C24bWin::OnAcquireDialog() {

	TRACE0("C24bWin::OnAcquireDialog\n");

		
	CAcquireDlg dlg(this);

	if (dlg.DoModal() == IDOK) {


		}
	
		
	controlkey = FALSE;
}







//***********************************************************************
//
// CLASS CHardwareDlg
//
// This class, derived from CScopeDialog, presents and controls the PAR24B
// specific Hardware dialog.  This dialog allows the user to specify the 
// parallel port mode and address.
//


BEGIN_MESSAGE_MAP(CHardwareDlg, CScopeDialog)
	//{{AFX_MSG_MAP(CHardwareDlg)
	ON_MESSAGE(WM_EDITLOSTFOCUS,      OnEditLostFocus)
	ON_BN_CLICKED(IDC_BTN_W_HELP,     OnHelp)
	ON_BN_CLICKED(IDC_BTN_W_OK,       OnOK)
	ON_BN_CLICKED(IDC_BTN_W_CANCEL,   OnCancel)
	ON_BN_CLICKED(IDC_BTN_W_FIFOSIZE, OnMeasureFIFO)
	ON_CBN_SELCHANGE(IDC_CBO_W_PORTADDR, OnSelchangeAddress)
	ON_CBN_KILLFOCUS(IDC_CBO_W_PORTADDR, OnKillfocusAddress)
	ON_CBN_SELCHANGE(IDC_CBO_W_FIFOSIZE, OnSelchangeFifoSize)
	ON_CBN_KILLFOCUS(IDC_CBO_W_FIFOSIZE, OnKillfocusFifoSize)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



CHardwareDlg::CHardwareDlg(CWnd* pParent /*=NULL*/)
	    : CScopeDialog(CHardwareDlg::IDD, pParent) {

	TRACE0("CHardwareDlg::CHardwareDlg\n");

	//{{AFX_DATA_INIT(CHardwareDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_Index = 0;
	m_Addr  = PAR24B_PORT_ADDRESS_1;
	m_Fifo  = PAR24B_FIFO_SIZE_TYP;
}


void CHardwareDlg::DoDataExchange(CDataExchange* pDX) {

	TRACE0("CHardwareDlg::DoDataExchange\n");

	CScopeDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CHardwareDlg)
	DDX_Control(pDX, IDC_CBO_W_PORTADDR, m_CBO_PortAddr);
	DDX_Control(pDX, IDC_CBO_W_FIFOSIZE, m_CBO_FifoSize);
	//}}AFX_DATA_MAP
}


BOOL CHardwareDlg::OnInitDialog() {

	TRACE0("CHardwareDlg::OnInitDialog\n");

	int ibtn;

	
	CScopeDialog::OnInitDialog();
	


	// Init Port Mode Radio Buttons
	
        if (APP.HardPortMode == PAR24B_PORT_MODE_BPP)
                ibtn = IDC_OPT_W_MODEBPP;
        
        else if (APP.HardPortMode == PAR24B_PORT_MODE_EPP)
                ibtn = IDC_OPT_W_MODEEPP;
        
        else if (APP.HardPortMode == PAR24B_PORT_MODE_ECP_BPP)
                ibtn = IDC_OPT_W_MODEECPB;
        
        else if (APP.HardPortMode == PAR24B_PORT_MODE_ECP_EPP)
                ibtn = IDC_OPT_W_MODEECPE;
        
        else        
                ibtn = IDC_OPT_W_MODEEPP;


	CheckRadioButton( IDC_OPT_W_MODEBPP, IDC_OPT_W_MODEECPE, ibtn);



		
	// Init Port Address Combo Box

	m_Addr = APP.HardPortAddr;

	SetComboAddress();



	// Init Fifo Size Combo Box

	m_Fifo = APP.HardFIFOsize;

	SetComboFifoSize();



	
	// Init Calibration Radio Buttons
	
	if (APP.HardCalibration == PAR24B_CAL_MODE_NONE)
		
		ibtn = IDC_OPT_W_CALNONE;
	
	else if (APP.HardCalibration == PAR24B_CAL_MODE_SOIG)
		
		ibtn = IDC_OPT_W_CALSOIG;
	
	else
		ibtn = IDC_OPT_W_CALSELF;

		
	CheckRadioButton( IDC_OPT_W_CALNONE, IDC_OPT_W_CALSOIG, ibtn);
		
			
		
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CHardwareDlg::OnOK() {

	TRACE0("CHardwareDlg::OnOK\n");

	unsigned int nv;
	int          ibtn;


	if (!CheckFocus())
		return;


		
	CScopeDialog::OnOK();


	
	// Update Port Mode

	ibtn = GetCheckedRadioButton( IDC_OPT_W_MODEBPP, IDC_OPT_W_MODEECPE );

	if (ibtn == IDC_OPT_W_MODEBPP)
		APP.HardPortMode = PAR24B_PORT_MODE_BPP;
        
	else if (ibtn == IDC_OPT_W_MODEEPP)
		APP.HardPortMode = PAR24B_PORT_MODE_EPP;
        
	else if (ibtn == IDC_OPT_W_MODEECPB)
		APP.HardPortMode = PAR24B_PORT_MODE_ECP_BPP;
        
	else // if (ibtn == IDC_OPT_W_MODEECPE)
		APP.HardPortMode = PAR24B_PORT_MODE_ECP_EPP;



	
	// Update Port Address

	APP.HardPortAddr = m_Addr;



	
	// Update Fifo Size

	APP.HardFIFOsize = m_Fifo;

	PAR24B_FifoSizeToNvalues( (unsigned int)APP.HardFIFOsize, &nv);

	if (pAPP->AcqNchannels > 0)
		pAPP->AcqNsamples = (int)nv / pAPP->AcqNchannels;
	
	if (pAPP->AcqNsamples > 0)
		pAPP->Nplotbuffers = 500 / pAPP->AcqNsamples + 1;






	// Update Calibration


	ibtn = GetCheckedRadioButton( IDC_OPT_W_CALNONE, IDC_OPT_W_CALSOIG );

	if (ibtn == IDC_OPT_W_CALNONE)
		
		APP.HardCalibration = PAR24B_CAL_MODE_NONE;

	else if (ibtn == IDC_OPT_W_CALSOIG)
		
		APP.HardCalibration = PAR24B_CAL_MODE_SOIG;
			
	else
		APP.HardCalibration = PAR24B_CAL_MODE_SELF;
		
}

void CHardwareDlg::OnCancel() {

	TRACE0("CHardwareDlg::OnCancel\n");

	CScopeDialog::OnCancel();

}

void CHardwareDlg::OnHelp() {

	TRACE0("CHardwareDlg::OnHelp\n");

	
	if (!CheckFocus())
		return;

	
	pAPP->ShowHelp(IDR_HELP_HARD_TITLE, IDR_HELP_HARD_TEXT, this);
}

void CHardwareDlg::OnMeasureFIFO() {

	TRACE0("CHardwareDlg::OnMeasureFIFO\n");

	int present, msize;

	
	if (!CheckFocus())
		return;

	
	present = pAPP->AtodMeasure( &msize );

	if (!present) {

		CString Msg;
		Msg  = "PAR24B board must be on and working\n";
		Msg += "in order to measure the FIFO size.\n";
		AfxMessageBox( Msg, MB_OK );
		}
	
	else {
		CString Msg;
		Msg.Format("Found FIFO size of %d\n", msize);
		AfxMessageBox( Msg, MB_OK );
		
		m_Fifo = msize;
		SetComboFifoSize();
		}
		
}

LONG CHardwareDlg::OnEditLostFocus( UINT nID, LONG xxx ) {

	TRACE1("CHardwareDlg::OnEditLostFocus (%u)\n", nID);

	BOOL good, good1, good2;
	

	if (!DoValidate) {
		return( 1 );
		}


	// Don't allow validation when already doing validation

	DoValidate = FALSE;


		
	// Validate various edit controls
	
	switch ( nID ) {
		
				// AFX_IDC_FONTNAMES somehow has focus
		case 1001:	// after enter from typing in a DropDown
				// CBO.  So validate them both to be safe.

			good1 = ValidateAsHex( IDC_CBO_W_PORTADDR,0,0xFFFF,
						&m_Addr,"Port address" );
			if (good1)
				SetComboAddress();

			good2 = ValidateAsInt( IDC_CBO_W_FIFOSIZE,
						PAR24B_FIFO_SIZE_MIN,
						PAR24B_FIFO_SIZE_MAX,
						&m_Fifo,"Fifo size" );
			if (good2)
				SetComboFifoSize();

			good = (good1 && good2);

			break;
			
			
		case IDC_CBO_W_PORTADDR:

			good = ValidateAsHex( IDC_CBO_W_PORTADDR,
						0,
						0xFFFF,
						&m_Addr,
						"Port address" );
			if (good)
				SetComboAddress();
			
			break;


		case IDC_CBO_W_FIFOSIZE:

			good = ValidateAsInt( IDC_CBO_W_FIFOSIZE,
						PAR24B_FIFO_SIZE_MIN,
						PAR24B_FIFO_SIZE_MAX,
						&m_Fifo,
						"Fifo size" );
			if (good)
				SetComboFifoSize();
			
			break;


		default:
			good = TRUE;
			break;
						
			
		}


	// Re-allow validation

	DoValidate = TRUE;


	if (good)
		return( 1 );
	else
		return( 0 );
}


void CHardwareDlg::OnSelchangeAddress() {

	TRACE0("CHardwareDlg::OnSelchangeAddress\n");

	
	m_Index = m_CBO_PortAddr.GetCurSel();

	if ( (m_Index == CB_ERR) || (m_Index == -1) )

		GetDlgItemInt( IDC_CBO_W_PORTADDR, &m_Addr );

	else {
		m_CBO_PortAddr.GetLBText( m_Index, pAPP->szTemp );
		m_Addr = strtol( pAPP->szTemp, NULL, 16 );
		}
}

void CHardwareDlg::OnKillfocusAddress() {

	TRACE0("CHardwareDlg::OnKillfocusAddress\n");

	PostMessage( WM_EDITLOSTFOCUS, IDC_CBO_W_PORTADDR, 0 );
}

void CHardwareDlg::SetComboAddress() {

	TRACE0("CHardwareDlg::SetComboAddress\n");


	// Make string from m_Addr
		
	sprintf( pAPP->szTemp, "0x%X", m_Addr );

	
	// Find string in list
		
	m_Index = m_CBO_PortAddr.FindString( -1, pAPP->szTemp );

	

	if ( (m_Index == CB_ERR) || (m_Index == -1) )
	 
		SetDlgItemText( IDC_CBO_W_PORTADDR, pAPP->szTemp );
	else
		m_CBO_PortAddr.SetCurSel( m_Index );
}

void CHardwareDlg::OnSelchangeFifoSize() {

	TRACE0("CHardwareDlg::OnSelchangeFifoSize\n");

	
	m_Index = m_CBO_FifoSize.GetCurSel();

	if ( (m_Index == CB_ERR) || (m_Index == -1) )

		GetDlgItemInt( IDC_CBO_W_FIFOSIZE, &m_Fifo );

	else {
		m_CBO_FifoSize.GetLBText( m_Index, pAPP->szTemp );
		m_Fifo = strtol( pAPP->szTemp, NULL, 16 );
		}
}

void CHardwareDlg::OnKillfocusFifoSize() {

	TRACE0("CHardwareDlg::OnKillfocusFifoSize\n");

	PostMessage( WM_EDITLOSTFOCUS, IDC_CBO_W_FIFOSIZE, 0 );
}

void CHardwareDlg::SetComboFifoSize() {

	TRACE0("CHardwareDlg::SetComboFifoSize\n");


	// Make string from m_Fifo
		
	sprintf( pAPP->szTemp, "%d", m_Fifo );

	
	// Find string in list
		
	m_Index = m_CBO_FifoSize.FindString( -1, pAPP->szTemp );


	if ( (m_Index == CB_ERR) || (m_Index == -1) )
	 
		SetDlgItemText( IDC_CBO_W_FIFOSIZE, pAPP->szTemp );
	else
		m_CBO_FifoSize.SetCurSel( m_Index );
}


//***********************************************************************
//
// CLASS CChannelDlg
//
// This class, derived from CScopeDialog, presents and controls the PAR24B
// specific Channel dialog.  This dialog allows the user to specify
// the titles and activation status of the channels.
//


BEGIN_MESSAGE_MAP(CChannelDlg, CScopeDialog)
	//{{AFX_MSG_MAP(CChannelDlg)
	ON_MESSAGE(WM_EDITLOSTFOCUS,     OnEditLostFocus)
	ON_BN_CLICKED(IDC_BTN_C_OK,      OnOK)
	ON_BN_CLICKED(IDC_BTN_C_CANCEL,  OnCancel)
	ON_BN_CLICKED(IDC_BTN_C_HELP,    OnHelp)
	ON_BN_CLICKED(IDC_BTN_C_BACK,    OnBtnBack)
	ON_BN_CLICKED(IDC_BTN_C_NEXT,    OnBtnNext)
	ON_EN_CHANGE(IDC_TXT_C_CURR,     OnChangeTxtCurr)
	ON_EN_KILLFOCUS(IDC_TXT_C_CURR,  OnKillfocusTxtCurr)
	ON_EN_KILLFOCUS(IDC_TXT_C_TITLE, OnKillfocusTxtTitle)
	ON_LBN_SELCHANGE(IDC_LST_C_SCAN, OnSelchangeListScan)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


CChannelDlg::CChannelDlg(CWnd* pParent /*=NULL*/)
	   : CScopeDialog(CChannelDlg::IDD, pParent) {

	TRACE0("CChannelDlg::CChannelDlg\n");

	//{{AFX_DATA_INIT(CChannelDlg)
	m_CurrChan = 0;
	//}}AFX_DATA_INIT


	pTempTitle = NULL;

}

CChannelDlg::~CChannelDlg() {

	TRACE0("CChannelDlg::~CChannelDlg\n");

	// Clean up dialog title buffers
	
	if (pTempTitle)
		{ free(pTempTitle); pTempTitle = NULL; }

}

void CChannelDlg::DoDataExchange(CDataExchange* pDX) {

	TRACE0("CChannelDlg::DoDataExchange\n");

	CScopeDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CChannelDlg)
	DDX_Control(pDX, IDC_LST_C_SCAN, m_LSTscan);
	DDX_Text(pDX, IDC_TXT_C_CURR, m_CurrChan);
	//}}AFX_DATA_MAP
}



BOOL CChannelDlg::OnInitDialog() {

	TRACE0("CChannelDlg::OnInitDialog\n");
	
	int i;


	
	// Allocate dialog title buffers

	pTempTitle = (char*)malloc( pAPP->MaxChannel * MAXTITLE *sizeof(char));


	if (!pTempTitle) {
		pAPP->ErrString = "Could not allocate dialog title buffer";
		pAPP->ShowError();
		EndDialog( -1 );
		return( FALSE );
		}




	
	// Initialize all member variables from application globals


	CScopeDialog::OnInitDialog();

	
	m_Nscan = 0;

	for ( i = 0 ; i < pAPP->MaxChannel ; i++ ) {

		strcpy(&pTempTitle[i*MAXTITLE],&(pAPP->pScanTitle[i*MAXTITLE]));

		if (pAPP->pScanActive[i] == CHECKED) {

			m_Nscan++;
			AddItem( i );
			
			}
		}

	UpdateChannelData(0);



		
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CChannelDlg::OnHelp() {

	TRACE0("CChannelDlg::OnHelp\n");
	
	if (!CheckFocus())
		return;

			
	pAPP->ShowHelp(IDR_HELP_CHAN_TITLE, IDR_HELP_CHAN_TEXT, this);
}

void CChannelDlg::OnCancel() {
	
	TRACE0("CChannelDlg::OnCancel\n");
	
	CScopeDialog::OnCancel();
}

void CChannelDlg::OnOK() {

	TRACE0("CChannelDlg::OnOK\n");
	
	int ipos, i;
	
	
	if (!CheckFocus())
		return;

	
	CScopeDialog::OnOK();


	// Update globals

	ipos = 0;
	
	for ( i = 0 ; i < pAPP->MaxChannel ; i++ ) {
		
		if (pAPP->pScanActive[i] == CHECKED) {
			pAPP->pScanOrder[ipos] = i;
			strcpy(&(pAPP->pScanTitle[ipos*MAXTITLE]),
				&pTempTitle[i*MAXTITLE]);
			ipos++;
			}
		}


}

LONG CChannelDlg::OnEditLostFocus( UINT nID, LONG xxx ) {

	TRACE1("CChannelDlg::OnEditLostFocus (%u)\n", nID);

	int  ipos, icurr;
	BOOL good;
	

	if (!DoValidate) {
		return( 1 );
		}


	// Don't allow validation when already doing validation

	DoValidate = FALSE;
	
	// Validate various edit controls
	
	switch ( nID ) {

		case IDC_TXT_C_CURR:

			icurr = m_CurrChan;

			good = ValidateAsInt(IDC_TXT_C_CURR,
						0, 
						pAPP->MaxChannel - 1, 
						&icurr, 
						"Current channel");

			// Always update channel since icurr
			// will be new or reset from bad

			UpdateChannelData(icurr);
			
			break;




		case IDC_TXT_C_TITLE:
			
			GetDlgItemText(IDC_TXT_C_TITLE,
				&pTempTitle[m_CurrChan*MAXTITLE], MAXTITLE); 
	
			ipos = FindPos();
			m_LSTscan.DeleteString(ipos);
			AddItem( m_CurrChan );
			ipos = FindPos();
			m_LSTscan.SetCurSel( ipos );

			good = TRUE;
			
			break;


			

		default:
			good = TRUE;
			break;
						
			
		}


	// Re-allow validation

	DoValidate = TRUE;


	TRACE0("CChannelDlg::OnEditLostFocus - END\n");

	if (good)
		return( 1 );
	else
		return( 0 );
}

void CChannelDlg::OnBtnBack() {

	TRACE0("CChannelDlg::OnBtnBack\n");
	

	int icurr;
	
	if (!CheckFocus())
		return;

	
	icurr = m_CurrChan - 1;
	if (icurr < 0)
		icurr = pAPP->MaxChannel - 1;
	
	UpdateChannelData( icurr );
	
}

void CChannelDlg::OnBtnNext() {
	
	TRACE0("CChannelDlg::OnBtnNext\n");
	
	int icurr;

	if (!CheckFocus())
		return;

		
	icurr = m_CurrChan + 1;
	if (icurr > pAPP->MaxChannel - 1)
		icurr = 0;
	
	UpdateChannelData( icurr );
	
}

void CChannelDlg::OnChangeTxtCurr() {

	TRACE0("CChannelDlg::OnChangeTxtCurr\n");
	
	CWnd *pWnd = GetFocus();


	if (pWnd && (pWnd->GetDlgCtrlID() == IDC_TXT_C_CURR) ) 
		GetDlgItem( IDC_LAB_C_CURR )->EnableWindow( FALSE );
}


void CChannelDlg::OnKillfocusTxtCurr() {

	TRACE0("CChannelDlg::OnKillfocusTxtCurr\n");
	
	PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_C_CURR, 0 );
}

void CChannelDlg::OnKillfocusTxtTitle() {

	TRACE0("CChannelDlg::OnKillfocusTxtTitle\n");
	
	PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_C_TITLE, 0 );
}

void CChannelDlg::OnSelchangeListScan() {

	TRACE0("CChannelDlg::OnSelchangeListScan\n");
	
	int icurr, ipos;
	
        ipos = m_LSTscan.GetCurSel();
		icurr = m_LSTscan.GetItemData(ipos);
        if (icurr != m_CurrChan)
            UpdateChannelData(icurr);
	
	TRACE0("CChannelDlg::OnSelchangeListScan - END\n");
}

void CChannelDlg::UpdateChannelData( int icurr ) {
	
	TRACE1("CChannelDlg::UpdateChannelData (%d)\n", icurr);
	
	int ipos;
    
	m_CurrChan = icurr;
	
	SetDlgItemInt(  IDC_TXT_C_CURR,  m_CurrChan );
	SetDlgItemText( IDC_TXT_C_TITLE, &pTempTitle[m_CurrChan*MAXTITLE] );

	ipos = FindPos();
	m_LSTscan.SetCurSel( ipos );

	GetDlgItem( IDC_LAB_C_CURR )->EnableWindow( TRUE );

}


void CChannelDlg::AddItem( int ikey ) {
	
	TRACE0("CChannelDlg::AddItem\n");
	
	CString tmpString;
	int     ipos;
	
	tmpString.Format("%02d  ", ikey);
	tmpString += &pTempTitle[ikey*MAXTITLE];
	ipos = m_LSTscan.AddString(tmpString);
	m_LSTscan.SetItemData(ipos,ikey);
}

int CChannelDlg::FindPos() {
	
	TRACE0("CChannelDlg::FindPos\n");
	
	int ipos, ncount;


	ncount = m_LSTscan.GetCount();

	for ( ipos = 0 ; ipos < ncount ; ipos++ ) {
		if ((int)m_LSTscan.GetItemData(ipos) == m_CurrChan)
			return( ipos );
		}
	
	return( -1 );
}





//***********************************************************************
//
// CLASS CAcquireDlg
//
// This class, derived from CScopeDialog, presents and controls the PAR24B
// specific Acquisition dialog.  This dialog allows the user to specify
// the sampling rate and the gain.
//

BEGIN_MESSAGE_MAP(CAcquireDlg, CScopeDialog)
	//{{AFX_MSG_MAP(CAcquireDlg)
	ON_MESSAGE(WM_EDITLOSTFOCUS,    OnEditLostFocus)
	ON_BN_CLICKED(IDC_BTN_A_OK,     OnOK)
	ON_BN_CLICKED(IDC_BTN_A_CANCEL, OnCancel)
	ON_BN_CLICKED(IDC_BTN_A_HELP,   OnHelp)
	ON_EN_KILLFOCUS(IDC_TXT_A_SPS,  OnKillfocusSps)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



CAcquireDlg::CAcquireDlg(CWnd* pParent /* = NULL */ )
	: CScopeDialog(CAcquireDlg::IDD, pParent)      {

	TRACE0("CAcquireDlg::CAcquireDlg\n");

	m_SPS = 0.0F;

}

BOOL CAcquireDlg::OnInitDialog() {

	TRACE0("CAcquireDlg::OnInitDialog\n");

	CScopeDialog::OnInitDialog();


	// Set SPS

	m_SPS = pAPP->AcqSPS;
	
	SetDlgItemFloat( IDC_TXT_A_SPS, m_SPS );


	// Set Gain
	
	CComboBox *pCbo = (CComboBox*)GetDlgItem( IDC_CBO_A_GAIN );
	pCbo->SetCurSel(APP.AcqGain);
	
		
	return TRUE;
}

void CAcquireDlg::OnOK() {

	TRACE0("CAcquireDlg::OnOK\n");


	if (!CheckFocus())
		return;


	
	// Don't close, if SPS changed due to error
	
	if (!VerifySps())
		return;

		
	CScopeDialog::OnOK();


	// Get SPS
					
	pAPP->AcqSPS = m_SPS;


	// Get Gain
	
	CComboBox *pCbo = (CComboBox*)GetDlgItem( IDC_CBO_A_GAIN );
	APP.AcqGain = pCbo->GetCurSel();
	
}


void CAcquireDlg::OnCancel() {

	TRACE0("CAcquireDlg::OnCancel\n");


	CScopeDialog::OnCancel();
}

void CAcquireDlg::OnHelp() {

	TRACE0("CAcquireDlg::OnHelp\n");

	if (!CheckFocus())
		return;

	pAPP->ShowHelp(IDR_HELP_ACQ_TITLE, IDR_HELP_ACQ_TEXT, this);
}

LONG CAcquireDlg::OnEditLostFocus( UINT nID, LONG xxx ) {

	TRACE1("CAcquireDlg::OnEditLostFocus (%u)\n", nID);

	BOOL   good;
	

	if (!DoValidate) {
		return( 1 );
		}


	// Don't allow validation when already doing validation

	DoValidate = FALSE;
	
	// Validate various edit controls
	
	switch ( nID ) {

		case IDC_TXT_A_SPS:

			good = ValidateAsFloat( IDC_TXT_A_SPS,
						pAPP->MinSPS,
						pAPP->MaxSPS,
						&m_SPS,
						"Samples per second" );

			APP.Par24bCorrectSps( &m_SPS );
	
			SetDlgItemFloat( IDC_TXT_A_SPS, m_SPS );
				
			break;
			

		default:
			good = TRUE;
			break;
						
			
		}


	// Re-allow validation

	DoValidate = TRUE;


	if (good)
		return( 1 );
	else
		return( 0 );
}

void CAcquireDlg::OnKillfocusSps() {

	TRACE0("CAcquireDlg::OnKillfocusSps\n");

	PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_A_SPS, 0 );
}

int CAcquireDlg::VerifySps() {

	TRACE0("CAcquireDlg::VerifySps\n");
	
	float spsvalue;

	spsvalue = GetDlgItemFloat( IDC_TXT_A_SPS );
	
	if (spsvalue < pAPP->MinSPS) {
		SetDlgItemFloat( IDC_TXT_A_SPS, pAPP->MinSPS );
		::MessageBeep(0);
		return( 0 );
		}
	else if (spsvalue > pAPP->MaxSPS) {
		SetDlgItemFloat( IDC_TXT_A_SPS, pAPP->MaxSPS );
		::MessageBeep(0);
		return( 0 );
		}
	else
		return( 1 );
		
}
