// FILE: scope.cpp                Copyright (c), Symmetric Research, 1997-1999
//
// Source file for generic portion of the Scope program includes
//
//      CScopeApp       should be virtual (ie must declare derived class)
//      CScopeWindow    should be virtual (ie must declare derived class)
//      CScopeDialog    should be virtual (ie must declare derived class)
//      CRegistryDlg
//      COutputDlg
//      CDisplayDlg
//      CContentsDlg
//      CTopicDlg
//
//
// Change history:
//
//  2/10/98 WCT changed temporary file name to scope.tmp
//  3/16/98 WCT added DC offset correction to A64 which allowed menus and
//              accelerators to be moved up to the common scope level
//  3/16/98 WCT replot axes when drawing traces
//  3/17/98 WCT correct trace polarity of drawing so + is above zero line
//  3/25/98 WCT added auto DC state to title bar and check box to display
//              dialog, added menu separator, changed names reg/menu/code
//  4/29/98 WCT split AtodInit into separate init and start functions
//              changed pTMPoffset to float so no overflow for long buffers
//              added AtodMeasure function, current used for 24b
//              added FIFO size controls to 24b hardware dialog
//  5/20/98 WCT changed .hdr and .out file names to be all upper case
//              no more auto removal for single file, so default to none
// 11/09/98 WCT changed pTMPoffset float to double to avoid mantissa overflow
//              enlarged Ctrl+F DC Offset beep to distinguish from bad key beep
//  6/18/99 WCT add OnEnterSizeMove and OnExitSizeMove to compensate for very
//              bad looking display on NT when "window dragging" option is
//              set from Display Properties dialog
//              
//              
//

#include <direct.h>     // needed for _getcwd (GET Current Working Directory)
#include <io.h>         // needed for _access

#include "scope.h"


// Global pointer to application object

CScopeApp *pAPP;



//***********************************************************************
//
// CLASS CScopeApp
//
// This class, derived from CWinApp, contains the generic application 
// portion of the Scope data acquisition program.  You should not declare
// objects of this type.  Instead, declare application objects of one of 
// the derived classes, CA64App or C24bApp.  The former is for use with
// the DSPA64/DSPHLF acquisition system and the latter is for the PAR24B
// acquisition system.  The application portion contains code dealing 
// with acquisition, files, and the registry.  CScopeWindow takes care of
// the graphics and main window functions.
//
//

BEGIN_MESSAGE_MAP(CScopeApp, CWinApp)
        //{{AFX_MSG_MAP(CScopeApp)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()



BOOL CScopeApp::InitInstance() {
        
        TRACE0("CScopeApp::InitInstance\n");

        BOOL    bAutoRun;
        HANDLE  hSem; 
        CString SemName;


        // Call just in case this isn't a no-op
        
        CWinApp::InitInstance();


        
        // Set initial data

        InitAppData();


        
        // Allocate and fill scan arrays
                
        if (!SetScanDefaults())
                return(FALSE);


                
        // Load and verify default values from registry

        InitFromRegistry();



                

        // Check command line for:
        //
        //  1. Normal (empty)   2. AutoRun (/r)   3. Error (unknown)

        CString CmdLine = m_lpCmdLine;
        CmdLine.TrimLeft();
        CmdLine.TrimRight();
        
        if ( CmdLine.IsEmpty() )
                bAutoRun = FALSE;

        else if ( !CmdLine.CompareNoCase("/r") )
                bAutoRun = TRUE;

        else {
                CString UsageMsg;
                UsageMsg  = "For Normal mode, run scope with no arguments.\n";
                UsageMsg += "For AutoRun mode where acquisition starts immediately,\n";
                UsageMsg += "run scope with the /r switch.\n";
                AfxMessageBox( UsageMsg, MB_OK );
                ErrCode = 1;
                return( FALSE );
                }


                        

        // Check for multiple instances

        if (ForbidMultiple) {

                // Create or open a named semaphore.
                
                SemName.Format( "SemaphoreFor%s", (LPCTSTR)AppTitle );

                hSem = CreateSemaphore( NULL, 0, 1, SemName );

                
                // If existing semaphore was opened, close handle.

                if ((hSem != NULL) && (GetLastError() == ERROR_ALREADY_EXISTS)) {
                            
                        CloseHandle(hSem);

                        AfxMessageBox( "Another instance of this program is already running.\n", MB_OK );

                        return( FALSE );

                        } 
                        
                }


                



                
        
        // Create main window and check validity.
        //
        // If invalid, delete it and NULL out the pointer so
        // DestroyWindow is not called when InitInstance fails.
        // The window class is already gone because of failing 
        // during its construction phase.

        
        m_pMainWnd = (CWnd*)NewScopeWindow();

        if ( ! ::IsWindow(m_pMainWnd->m_hWnd) ) {

                delete m_pMainWnd;
                m_pMainWnd = NULL;
                
                return( FALSE );
                }
        

                
                
        // Show main window
        
        m_pMainWnd->ShowWindow(m_nCmdShow);
        m_pMainWnd->UpdateWindow();


        
        // Post run command message if AutoRun mode requested

        if (bAutoRun)
                m_pMainWnd->PostMessage( WM_COMMAND, IDM_PROGRAM_RUN, NULL );

        return( TRUE );

}


int CScopeApp::InitAppData() {

        TRACE0("CScopeApp::InitAppData\n");
        
        Enable3dControls(); // use 3d controls in dialogs


        // Set general global application pointer
        // Although declared as base class, actually points to 
        // the global object which is of the derived class

        pAPP = this;


        
        // Initialize common global constants (not stored in registry)

        ErrCode          = 0;
        CurrBlock        = 0;
        Iplotbuff        = 0;
        LastNbufferbytes = 0;
        
        DoRegistryDelete = FALSE;
        IsRunning        = FALSE;
        DoClipping       = FALSE;
        IsAllocated      = FALSE;
        ResetPaint       = TRUE;
        ForbidMultiple   = TRUE;
        AutoDC           = FALSE;
        ResetDC          = TRUE;

        pTMPoffset       = NULL;
        pDCoffset        = NULL;
        pDemuxArray      = NULL;
        
        hkeySect         = NULL;
        hWndPrev         = NULL;
        
        ErrString        = "Error:";
        szTemp[0]        = '\0';



                        
        // Get current working directory
        // This is curr dir from Dos prompt, and exe dir from Win Run...
        
        if (_getcwd( szTemp, MAXSTR ) != NULL)
                CurrDir = szTemp;
        else
                CurrDir = ".";


        // NOTE: Many of the other variables are assumed to be
        //       set from the derived class InitAppData call
        //       which should complete after having called this
        //       parent class version


        return( 1 );
}

CScopeWindow* CScopeApp::NewScopeWindow() {

        TRACE0("CScopeApp::NewScopeWindow\n");

        return( NULL );
}

int CScopeApp::ExitInstance() {
        
        TRACE0("CScopeApp::ExitInstance\n");


        // Save all registry values

        if (ErrCode == 0) {
                RegistrySettings( REG_DIR_SAVE, REG_SECT_SETTING );
                RegistrySettings( REG_DIR_SAVE, REG_SECT_HARD );
                RegistrySettings( REG_DIR_SAVE, REG_SECT_CHAN );
                RegistrySettings( REG_DIR_SAVE, REG_SECT_ACQ );
                RegistrySettings( REG_DIR_SAVE, REG_SECT_OUTPUT );
                RegistrySettings( REG_DIR_SAVE, REG_SECT_DISP );
                }

        if (DoRegistryDelete == TRUE)
                RegistryClear();


        // Clean up any existing common arrays

        if (pDemuxArray)
                 { free(pDemuxArray); pDemuxArray = NULL; }
        
        FreeScanArrays();

        
        return( CWinApp::ExitInstance() );
}

int CScopeApp::AtodMeasure( int *ival ) {
        TRACE0("CScopeApp::AtodMeasure\n");
        return( 0 );
}

int CScopeApp::AtodInit() {
        TRACE0("CScopeApp::AtodInit\n");
        return( 0 );
}

int CScopeApp::AtodStart() {
        TRACE0("CScopeApp::AtodStart\n");
        return( 0 );
}

int CScopeApp::AtodCheckReady() {
        TRACE0("CScopeApp::AtodCheckReady\n");
        return( 0 );
}

int CScopeApp::AtodCheckOverflow() {
        TRACE0("CScopeApp::AtodCheckOverflow\n");
        return( 0 );
}

int CScopeApp::AtodGetData() {
        TRACE0("CScopeApp::AtodGetData\n");
        return( 0 );
}

int CScopeApp::AtodDemuxData() {
        TRACE0("CScopeApp::AtodDemuxData\n");
        return( 0 );
}

int CScopeApp::AtodStop() {
        TRACE0("CScopeApp::AtodStop\n");
        return( 0 );
}

int CScopeApp::FreeScanArrays() {

        TRACE0("CScopeApp::FreeScanArrays\n");

        if (pTMPoffset)
                { free(pTMPoffset);  pTMPoffset  = NULL; }

        if (pDCoffset)
                { free(pDCoffset);   pDCoffset   = NULL; }

        if (pScanActive)
                { free(pScanActive); pScanActive = NULL; }
        
        if (pScanOrder)
                { free(pScanOrder);  pScanOrder  = NULL; }
        
        if (pScanTitle)
                { free(pScanTitle);  pScanTitle  = NULL; }

        
        return( 1 );
}


int CScopeApp::ComputeActiveChannels( ) {

        TRACE0("CScopeApp::ComputeActiveChannels\n");

        int i;



        // Count number of active channels and set their order
        
        AcqNchannels = 0;
        
        for ( i = 0 ; i < MaxChannel ; i++ ) {
        
                if (pScanActive[i] == CHECKED) {
                
                        pScanOrder[AcqNchannels] = i;
                        AcqNchannels++;
                        }
                }


        // Ensure at least one active channel

        if (AcqNchannels == 0) {
                pScanActive[0] = CHECKED;
                pScanOrder[0]  = 0;
                AcqNchannels = 1;
                }


                
        // Ensure valid display number
        
        if (PltNdisplay > AcqNchannels)
                PltNdisplay = AcqNchannels;


        return( 1 );
}

        
int CScopeApp::SetScanDefaults() {
        
        TRACE0("CScopeApp::SetScanDefaults\n");

        int i;


        // Clean up old scan buffers and offset

        FreeScanArrays();

        

        // Allocate new scan buffers and offset

        pTMPoffset  = (double*) malloc( MaxChannel * sizeof(double) );
        pDCoffset   = (long*)   malloc( MaxChannel * sizeof(long) );
        
        pScanActive = (long*)   malloc( MaxChannel * sizeof(long) );
        pScanOrder  = (int*)    malloc( MaxChannel * sizeof(int) );
        pScanTitle  = (char*)   malloc( MaxChannel * sizeof(char) * MAXTITLE );




        if (!pScanActive || !pScanOrder || !pScanTitle ||
                            !pTMPoffset || !pDCoffset) {
                ErrString = "Could not allocate scan buffers or offset";
                ExitError();
                return( 0 );
                }


        
        // Initialize

        for ( i = 0 ; i < MaxChannel ; i++ ) {
                
                pTMPoffset[i]  = 0.0;
                pDCoffset[i]   = 0;
                
                }

        return( 1 );
}


int CScopeApp::ComputeBufferSizes( void ) {
        
        TRACE0("CScopeApp::ComputeBufferSizes\n");

        Nbufferpts      = AcqNchannels * AcqNsamples;
        NbufferptsPlot  = Nbufferpts   * Nplotbuffers;
        AcqNsamplesPlot = AcqNsamples  * Nplotbuffers;
        
        Nbufferbytes    = Nbufferpts   * DataPtSize;

        return( 1 );
}


int CScopeApp::AllocateBuffer( void ) {
        
        TRACE0("CScopeApp::AllocateBuffer\n");

        int   i, j, iret;
        long *pData, joff;

        
        ComputeBufferSizes();


        // Only re-dim if array needs to get bigger

        if (LastNbufferbytes < Nbufferbytes) {
                
                LastNbufferbytes = Nbufferbytes;


                // Clean up any previous demux array and allocate new one
        
                if (pDemuxArray)
                        { free(pDemuxArray); pDemuxArray = NULL; }

                pDemuxArray = (long*) malloc(NbufferptsPlot * sizeof(long));


                
                // Repeat for hardware specific data array
                
                iret = AllocateDataArray();

                }
        else
                iret = 1;

                



                
        // Check for successful allocation

        if (!iret || !pDemuxArray) {
        
                IsAllocated = FALSE;

                AfxMessageBox( "Can not allocate data buffers" );
                
                }
                
        else
                IsAllocated = TRUE;
                

                

        // When allocated, clear array (even if remains allocated from before)
                        
        if (IsAllocated == TRUE) {

                pData = pDemuxArray;
        
                for ( j = 0 ; j < AcqNchannels ; j++ ) {

                        joff = pDCoffset[j];
                        
                        for ( i = 0 ; i < AcqNsamplesPlot ; i++ )
                
                                *pData++ = joff;
                        
                        }
                
                }


        
        return( IsAllocated );
}


int CScopeApp::AllocateDataArray( void ) {
        TRACE0("CScopeApp::AllocateDataArray\n");
        return( 0 );
}

int CScopeApp::TriggerCondition( void ) {
        TRACE0("CScopeApp::TriggerCondition\n");

        // This function is called by the OnTimer function to decide
        // whether or not the currently acquired buffer of data should
        // be written out.  You can perform common triggering computations
        // here, but most of the triggering work will likely occur in the
        // TriggerCondition function of the derived class (CA64App or C24bApp)
        // since those classes have a pointer to the raw data array.
        
        return( 1 );
}

int CScopeApp::FillHdr( void ) {
        
        TRACE0("CScopeApp::FillHdr\n");

        int i, ichan;
        
        
        // Initialize general header info
        
        for ( i = 0 ; i < 32 ; i++ )
                Hdr.Id[i] = '\0';

        Hdr.Channels    = AcqNchannels;
        Hdr.RecordPts   = AcqNsamples * AcqNblock;
        Hdr.WordSize    = DataPtSize;
        Hdr.DataCoding  = 1;
        
        Hdr.SampleRate  = AcqSPS;
        Hdr.Gain        = 1.0; 
        Hdr.FilterCoeff = 0.0;
        Hdr.FilterScale = 1.0;

        for ( i = 0 ; i < AcqNchannels ; i++ ) {
                
                ichan = pScanOrder[i];
                Hdr.ChannelList[i].channel = ichan;
                strcpy(Hdr.ChannelList[i].title, &pScanTitle[ichan*MAXTITLE]);
                
                }

        return( 1 );
}


int CScopeApp::WriteBytes( CFile *outfile ) {
        TRACE0("CScopeApp::WriteBytes\n");
        return( 0 );
}


int CScopeApp::RegistrySettings( int direction, int sectid ) {
        TRACE0("CScopeApp::RegistrySettings\n");
        return( 0 );
}


int CScopeApp::WriteHdr( void ) {
        
        TRACE0("CScopeApp::WriteHdr\n");

        CString filename;
        CFile   hdrfile;
        int     realsize;



        // No output desired

        if (AcqFile == FILE_NONE)
                return( TRUE );



        // Initialize header info
        
        FillHdr();




        // Output the header file

        filename = WorkDir;
        filename += "\\SCOPE.HDR";

        if (!hdrfile.Open( filename, CFile::modeCreate | CFile::modeReadWrite
                                   | CFile::shareExclusive, NULL ) ) {

                ErrString.Format("Could not output header file %s.",
                                                        (LPCTSTR) filename);
                ShowError();

                return( FALSE );

                }
        else {

                // This computation computes the real header size
                // by removing the allocated ChannelList portion
                // and then only including back the actual maximum
                // number of channels.  This allows allocating
                // enough space to service either the DSPA64 or the
                // PAR24B, but to only write out the pertinent space.

                realsize = (sizeof(Hdr) - sizeof(Hdr.ChannelList)) +
                           MaxChannel * sizeof(struct channel_data);

                hdrfile.Write( &Hdr, realsize );

                hdrfile.Flush();

                hdrfile.Close();

                return( TRUE );
                
                }
        
}


int CScopeApp::WriteData( void ) {
        
        TRACE0("CScopeApp::WriteData\n");

        int         iret, irename;
        CString     filename;
        CString     newname;
        CFile       outfile;
        CFileStatus rStatus;
        UINT        openflags;


        
        // No output desired

        if (AcqFile == FILE_NONE)
                return( TRUE );


        
        filename = WorkDir;
        filename += "\\SCOPE.TMP";

        // Open existing or open create or fail

        if (CurrBlock > 0)
                openflags = CFile::modeCreate | CFile::modeNoTruncate;
        else
                openflags = CFile::modeCreate;

                        
        iret = outfile.Open( filename, CFile::shareExclusive | openflags |
                                        CFile::modeReadWrite, NULL);

        if (!iret) {

                ErrString.Format("Could not output data file %s.",
                                                        (LPCTSTR) filename);
                ShowError();
                return( FALSE );
                
                }



        // Position
        
        outfile.SeekToEnd( );


        // Write out specific data bytes

        iret = WriteBytes( &outfile );
        

        outfile.Flush();

        outfile.Close();


        
        CurrBlock += 1;





        // Rename file depending on chosen naming scheme
        
        if (CurrBlock >= AcqNblock) {

                if (AcqFile == FILE_ONE) {

                        newname.Format( "%s\\SCOPE.OUT", (LPCTSTR)WorkDir );
                        irename = 1;
                        }
                
                else if (AcqFile == FILE_SEQ) {

                        newname.Format( "%s\\%08lX.OUT", (LPCTSTR)WorkDir, AcqSeqStart );
                        AcqSeqStart++;
                        irename = 1;
                        }
                
                else if (AcqFile == FILE_TIME) {
                        
                        newname.Format( "%s\\%08lX.OUT", (LPCTSTR)WorkDir, time(NULL) );
                        irename = 1;
                        }
                
                else // AcqFile == FILE_NONE

                        irename = 0;

                
                        
                if (irename == 1) {
                        
                        TRY {
                                CFile::Rename(filename, newname );
                                }
                        CATCH( CFileException, e ) {
                                ErrString.Format("Could not rename %s to %s.",
                                        (LPCTSTR)filename, (LPCTSTR)newname );
                                if (e->m_cause == CFileException::accessDenied)
                                        ErrString += "  Access denied.";
                                else if (e->m_cause == CFileException::fileNotFound)
                                        ErrString += "  File not found.";
                                else if (e->m_cause == CFileException::badPath)
                                        ErrString += "  Bad path.";
                                ShowError( );
                                return( FALSE );
                                }
                        END_CATCH

                        }
                

                CurrBlock = 0;
                }


        if (iret)
                return( TRUE );
        else
                return( FALSE );

}


int CScopeApp::InitFromRegistry( void ) {

        TRACE0("CScopeApp::InitFromRegistry\n");



        // Initialize hardware constants:
        //
        // A64: HardBaseio, HardJP60, HardXtal
        // 24B: HardPortMode, HardPortAddr, HardCalibration, HardFIFOsize
        // 4CH: HardPortName, HardPortAddr
        
        RegistrySettings( REG_DIR_GET, REG_SECT_HARD );


        

        // Initialize channel constants(uses MaxChannel,pScanTitle,pScanActive):
        //
        // pScanTitle, pScanActive
        
        RegistrySettings( REG_DIR_GET, REG_SECT_CHAN );



        

        // Initialize acquisition constants:
        //
        // A64: AcqNcount,AcqNormalize,AcqDSPenable,AcqOverflow,AcqCoeff,AcqScale
        // 24B: AcqLedValue, AcqGain, AcqSPS
        // 4CH: AcqLedValue, AcqGainLog, AcqTurboLog, AcqSPS
        
        RegistrySettings( REG_DIR_GET, REG_SECT_ACQ );


        

        
        // Initialize output constants (uses DefaultNsamples,Nplotbuffers,CurrDir):
        //
        // AcqNsamples, AcqNblock, AcqFile, AcqSeqStart, WorkDir
        
        RegistrySettings( REG_DIR_GET, REG_SECT_OUTPUT );


        


        // Initialize display constants (uses MaxAmpShift,DefaultNdisplay,DefaultColor)
        //
        // PltShiftBy, PltNdisplay, PltFirstSample,PltFirstTrace,PltColorScheme
        
        RegistrySettings( REG_DIR_GET, REG_SECT_DISP );




        

        // Initialize miscellaneous settings
        //
        // DoClipping,DoStatusBar,DoPlotTrace,DoRegistryDelete,AutoDC,ForbidMultiple
        
        RegistrySettings( REG_DIR_GET, REG_SECT_SETTING );





                
        

        // Ensure that all parameter values are valid

        VerifyAppData();


                                

        

                
        // Initialize active channels (uses MaxChannel,pScanActive,HardJP60(A64))
        //
        // AcqNchannels, pScanOrder, PltNdisplay
        // BoardAggSPS, MinSPS, MaxSPS, AcqSPS (A64)
        
        ComputeActiveChannels();




                

        // Initialize buffer sizes (uses AcqNchannels,AcqNsamples,Nplotbuffers):
        //
        // Nbufferpts, NbufferptsPlot, AcqNsamplesPlot
        
        ComputeBufferSizes();





        
        return( 1 );
}


int CScopeApp::VerifyAppData() {

        TRACE0("CScopeApp::VerifyAppData\n");

        int maxValue;


        // Program setting parameter verification

        if ((DoClipping < -1) || (DoClipping > 1))
                DoClipping = FALSE;
        
        if ((DoStatusBar < -1 ) || (DoStatusBar > 1))
                DoStatusBar = TRUE;
        
        if ((DoPlotTrace < -1 ) || (DoPlotTrace > 1))
                DoPlotTrace = TRUE;
        
        if ((ForbidMultiple < -1 ) || (ForbidMultiple > 1))
                ForbidMultiple = TRUE;
        
        if ((DoRegistryDelete < -1 ) || (DoRegistryDelete > 1))
                DoRegistryDelete = FALSE;
        
        if ((AutoDC < -1 ) || (AutoDC > 1))
                AutoDC = FALSE;
        




        // Output file parameter verification
        
        if ((AcqNsamples<=0) || ((MaxSample!=0) && (AcqNsamples>MaxSample)))
                AcqNsamples = DefaultNsamples;

        if (AcqNblock <= 0)
                AcqNblock = Nplotbuffers;

        if ((AcqFile < FILE_NONE) || (AcqFile > FILE_TIME))
                AcqFile = FILE_NONE;

        if (AcqSeqStart <= 0)
                AcqSeqStart = 0;

        if (_access(WorkDir,00) != 0)
                WorkDir = CurrDir;

        


        // Display parameter verification

        if ((PltShiftBy < 0) || (PltShiftBy > MaxAmpShift-1))
                PltShiftBy = MaxAmpShift/2;

        if ((PltNdisplay <= 0) || (PltNdisplay > MaxChannel))
                PltNdisplay = DefaultNdisplay;

        maxValue = AcqNsamplesPlot - GetSystemMetrics(SM_CXFULLSCREEN) - 1;
        if (maxValue < 0)  maxValue = 0;
        if ((PltFirstSample < 0) || (PltFirstSample > maxValue))
                PltFirstSample = 0;

        maxValue = MaxChannel-PltNdisplay;
        if ((PltFirstTrace < 0) || (PltFirstTrace > maxValue))
                PltFirstTrace = 0;

        if ((PltColorScheme < CS_MONO) || (PltColorScheme > CS_BROWN))
                PltColorScheme = DefaultColor;
                

        return( 1 );
}

int CScopeApp::RegistryOpenKey( int sectid ) {

        TRACE1("CScopeApp::RegistryOpenKey %d\n", sectid);

        int  iret;
        char *pszKeySect;


        
        // Create/Open Section Key
        

        if (sectid == REG_SECT_SETTING)
                
                pszKeySect = "Program Settings";
                
        else if (sectid == REG_SECT_HARD)
                
                pszKeySect = "Hardware Dialog";
        
        else if (sectid == REG_SECT_CHAN)
                
                pszKeySect = "Channel Dialog";
        
        else if (sectid == REG_SECT_ACQ)
                
                pszKeySect = "Acquire Dialog";
        
        else if (sectid == REG_SECT_OUTPUT)
                
                pszKeySect = "Output Dialog";

        else if (sectid == REG_SECT_DISP)
                
                pszKeySect = "Display Dialog";
        
        else
                return( 0 );

        
        sprintf( szTemp, "Software\\Symmetric Research\\%s\\%s",
                                                AppTitle, pszKeySect );

        iret = RegCreateKey( HKEY_CURRENT_USER, szTemp, &hkeySect );

        if (iret != ERROR_SUCCESS || !hkeySect)

                return( 0 );

                
        return( 1 );
}

int CScopeApp::RegistryClear( void ) {

        TRACE0("CScopeApp::RegistryClear\n");

        int iret;


        sprintf(szTemp,
                "Delete key HKEY_CURRENT_USER\\Software\\Symmetric Research\\%s",
                AppTitle);

        if (AfxMessageBox( szTemp, MB_OKCANCEL ) != IDOK)
                return( 0 );



                        
        sprintf(szTemp,"Software\\Symmetric Research\\%s",AppTitle);

        iret = RegDeleteKey( HKEY_CURRENT_USER, szTemp );



        if (iret == ERROR_SUCCESS)
                return( 1 );
        else
                return( 0 );
}

int CScopeApp::RegistryCommonSettings( int direction, int sectid ) {

        TRACE1("CScopeApp::RegistryCommonSettings %d\n", sectid);



        

        // Program Settings Registry Settings
                
        if (sectid == REG_SECT_SETTING) {
                
            if (direction == REG_DIR_GET) {
                    
                GetRegLong( "Clipping",       (long*)&DoClipping,       FALSE );
                GetRegLong( "StatusBar",      (long*)&DoStatusBar,      TRUE  );
                GetRegLong( "PlotTrace",      (long*)&DoPlotTrace,      TRUE  );
                GetRegLong( "ForbidMultiple", (long*)&ForbidMultiple,   TRUE  );
                GetRegLong( "RegistryDelete", (long*)&DoRegistryDelete, FALSE );
                GetRegLong( "AutoDC",         (long*)&AutoDC,           FALSE );
                
                }

            else if (direction == REG_DIR_SAVE) {

                SaveRegLong( "Clipping",       DoClipping       );
                SaveRegLong( "StatusBar",      DoStatusBar      );
                SaveRegLong( "PlotTrace",      DoPlotTrace      );
                SaveRegLong( "ForbidMultiple", ForbidMultiple   );
                SaveRegLong( "RegistryDelete", DoRegistryDelete );
                SaveRegLong( "AutoDC",         AutoDC           );
                
                }

            }



        
            
        // File Output Dialog Registry Settings
                
        else if (sectid == REG_SECT_OUTPUT) {
                
            if (direction == REG_DIR_GET) {
                    
                GetRegLong( "Nsamples", (long*)&AcqNsamples,  DefaultNsamples );
                GetRegLong( "Nblock",   (long*)&AcqNblock,    DefaultNblock   );
                GetRegLong( "FileOut",  (long*)&AcqFile,      FILE_NONE       );
                GetRegLong( "SeqStart", (long*)&AcqSeqStart,  0               );
                GetRegChar( "WorkDir",  WorkDir.GetBuffer(MAXSTR), (LPCTSTR)CurrDir );
                WorkDir.ReleaseBuffer();
                
                }

            else if (direction == REG_DIR_SAVE) {

                SaveRegLong( "Nsamples", AcqNsamples      );
                SaveRegLong( "Nblock",   AcqNblock        );
                SaveRegLong( "FileOut",  AcqFile          );
                SaveRegLong( "SeqStart", AcqSeqStart      );
                SaveRegChar( "WorkDir",  (LPCTSTR)WorkDir );
                
                }

            }




        // Display Dialog Registry Settings
                
        else if (sectid == REG_SECT_DISP) {

            if (direction == REG_DIR_GET) {
                
                GetRegLong( "ShiftBy",     (long*)&PltShiftBy,     MaxAmpShift/2   );
                GetRegLong( "Ndisplay",    (long*)&PltNdisplay,    DefaultNdisplay );
                GetRegLong( "FirstSample", (long*)&PltFirstSample, 0               );
                GetRegLong( "FirstTrace",  (long*)&PltFirstTrace,  0               );
                GetRegLong( "ColorScheme", (long*)&PltColorScheme, DefaultColor    );
                
                }

            else if (direction == REG_DIR_SAVE) {
                        
                SaveRegLong( "ShiftBy",     PltShiftBy     );
                SaveRegLong( "Ndisplay",    PltNdisplay    );
                SaveRegLong( "FirstSample", PltFirstSample );
                SaveRegLong( "FirstTrace",  PltFirstTrace  );
                SaveRegLong( "ColorScheme", PltColorScheme );
                        
                }

            }





        return( 1 );
        
}

int CScopeApp::SaveRegLong( const char *szName, const long value ) {

        TRACE0("CScopeApp::SaveRegLong\n");
                
        sprintf( szTemp, "%ld", value );

        return( SaveSetting( szName, szTemp ) );
}

int CScopeApp::SaveRegFloat( const char *szName, const float value ) {

        TRACE0("CScopeApp::SaveRegFloat\n");
                
        sprintf( szTemp, "%f", value );

        return( SaveSetting( szName, szTemp ) );
}


int CScopeApp::SaveRegChar( const char *szName, const char *value ) {

        TRACE0("CScopeApp::SaveRegChar\n");
                
        return( SaveSetting( szName, value ) );
}

        
int CScopeApp::SaveSetting( const char *szName, const char *szValue ) {

        TRACE0("CScopeApp::SaveSetting\n");
        
        DWORD cbData;

        /* Check for valid section key */

        if (!hkeySect)
                return( 0 );;


        /* Set registry value */
        
        cbData = (lstrlen(szValue) + 1) * sizeof(TCHAR);
                        
        RegSetValueEx( hkeySect, szName, 0, REG_SZ, (LPBYTE)szValue, cbData );

        return( 1 );
}


int CScopeApp::GetRegLong(const char *szName, long *value, const long defval ) {

        TRACE0("CScopeApp::GetRegLong\n");
        
        int nscan;

        if ( GetSetting( szName, szTemp ) )
                nscan = sscanf( szTemp, "%ld", value );
        else
                nscan = 0;

        if (nscan != 1)
                *value = defval;

        
        return( 1 );
}

int CScopeApp::GetRegFloat( const char *szName, float *value, const float defval ) {

        TRACE0("CScopeApp::GetRegFloat\n");
        
        int nscan;

        if ( GetSetting( szName, szTemp ) )
                nscan = sscanf( szTemp, "%f", value );
        else
                nscan = 0;

        if (nscan != 1)
                *value = defval;

        
        return( 1 );
}

int CScopeApp::GetRegChar( const char *szName, char *value, const char *defval ) {

        TRACE0("CScopeApp::GetRegChar\n");
        
        if ( GetSetting( szName, szTemp ) )
                
                strcpy( value, szTemp );
        else
                strcpy( value, defval );

        
        return( 1 );
}

int CScopeApp::GetSetting( const char *szName, char *szValue ) {

        TRACE0("CScopeApp::GetSetting\n");
        
        int   iret;
        DWORD cbData;


        /* Check for valid section key */

        if (!hkeySect)
                return( 0 );


        /* Read registry value */
        
        cbData = (MAXSTR) * sizeof(TCHAR);

        iret = RegQueryValueEx( hkeySect, szName, NULL, NULL,
                                        (LPBYTE)szValue, &cbData);

        if (iret == ERROR_SUCCESS)
                return( 1 );
        else
                return( 0 );

}

int CScopeApp::ShowHelp( int titleID, int textID, CWnd *caller ) {

        int iret;
        
        CTopicDlg dlg(titleID, textID, m_pMainWnd);
        iret = dlg.DoModal();

        if (caller)
                caller->SetFocus();
        
        return( iret );
}

int CScopeApp::LoadHelpTitles( CListBox *lbx ) {

        int pos;

        // Load general help titles at default location
        
        pos = AddResourceString(lbx, IDR_HELP_MAIN_TITLE,   IDR_HELP_MAIN_TEXT   );
        pos = AddResourceString(lbx, IDR_HELP_ATOD_TITLE,   IDR_HELP_ATOD_TEXT   );
        pos = AddResourceString(lbx, IDR_HELP_REG_TITLE,    IDR_HELP_REG_TEXT    );
        pos = AddResourceString(lbx, IDR_HELP_HARD_TITLE,   IDR_HELP_HARD_TEXT   );
        pos = AddResourceString(lbx, IDR_HELP_CHAN_TITLE,   IDR_HELP_CHAN_TEXT   );
        pos = AddResourceString(lbx, IDR_HELP_ACQ_TITLE,    IDR_HELP_ACQ_TEXT    );
        pos = AddResourceString(lbx, IDR_HELP_OUTPUT_TITLE, IDR_HELP_OUTPUT_TEXT );
        pos = AddResourceString(lbx, IDR_HELP_DISP_TITLE,   IDR_HELP_DISP_TEXT   );
        pos = AddResourceString(lbx, IDR_HELP_HELP_TITLE,   IDR_HELP_HELP_TEXT   );

        return( pos );
}

int CScopeApp::AddResourceString( CListBox *listbox,
                                  int titleID, int textID, int pos /*=-1*/ ) {

        CString htitle;

        htitle = (LPCSTR)::LockResource( 
                         ::LoadResource( NULL, 
                         ::FindResource( NULL, 
                            (LPCTSTR)titleID, RT_RCDATA)) );

        if (pos == -1)
                pos = listbox->AddString( htitle );
        else
                pos = listbox->InsertString( pos, htitle );
                
        listbox->SetItemData( pos, MAKELONG(titleID,textID) );

        return( pos );
}

void CScopeApp::ShowError( void ) {

        CScopeWindow *pWnd = (CScopeWindow*)m_pMainWnd;
        
        if (IsRunning)
                pWnd->OnStopProgram();

        AfxMessageBox( (LPCTSTR)ErrString );

        pWnd->Invalidate();
}

void CScopeApp::ExitError( void ) {
        
        TRACE0("CScopeApp::ExitError\n");

        ErrCode = 1;
        
        AfxMessageBox( (LPCTSTR)ErrString );

        
        if (m_pMainWnd != NULL)
                m_pMainWnd->DestroyWindow();

        else
                AfxPostQuitMessage( 0 );


        TRACE0("CScopeApp::ExitError - END\n");
}



//***********************************************************************
//
// CLASS CScopeWindow
//
// This class, derived from CFrameWnd, contains the generic main window
// portion of the Scope data acquisition program.  It provides the
// graphics drawing, menu operations, and other main window functions
// for the CScopeApp class which deals with acquisition, files, and the
// registry.  Window functions specific to the A64 and 24B versions are
// taken care of in the derived classes CA64Win and C24bWin.
//

BEGIN_MESSAGE_MAP( CScopeWindow, CFrameWnd )
        //{{AFX_MSG_MAP(CScopeWindow)
        ON_WM_CREATE()
        ON_WM_SIZE()
        ON_WM_PAINT()
        ON_WM_HSCROLL()
        ON_WM_VSCROLL()
        ON_WM_DESTROY()
        ON_WM_KEYDOWN()
        ON_WM_KEYUP()
        ON_WM_TIMER()
        ON_COMMAND(IDM_PROGRAM_RUN,         OnRunProgram)
        ON_COMMAND(IDM_PROGRAM_STOP,        OnStopProgram)
        ON_COMMAND(IDM_PROGRAM_EXIT,        OnExit)
        ON_COMMAND(IDM_OPTIONS_REG_FACTORY, OnRegFactoryDialog)
        ON_COMMAND(IDM_OPTIONS_REG_DELETE,  OnSetRegDelete)
        ON_COMMAND(IDM_OPTIONS_DISPLAY,     OnDisplayDialog)
        ON_COMMAND(IDM_OPTIONS_OUTPUT,      OnOutputDialog)
        ON_COMMAND(IDM_OPTIONS_CLIP,        OnSetClip)
        ON_COMMAND(IDM_OPTIONS_AUTODC,      OnSetAutoDC)
        ON_COMMAND(IDM_OPTIONS_UPDATEDC,    OnSetDC)
        ON_COMMAND( ID_VIEW_STATUS_BAR,     OnSetStatusBar)
        ON_COMMAND(IDM_VIEW_PLOTTRACE,      OnSetPlotTrace)
        ON_COMMAND(IDM_HELP_CONTENTS,       OnContentsDialog)
        ON_COMMAND(IDM_HELP_ABOUT,          OnAboutDialog)
        ON_COMMAND(IDM_HELP_DEBUG,          OnDebug)
        ON_UPDATE_COMMAND_UI(IDM_PROGRAM_STOP,        OnUpdateMenuRunning)
        ON_UPDATE_COMMAND_UI(IDM_PROGRAM_RUN,         OnUpdateMenuStopped)
        ON_UPDATE_COMMAND_UI(IDM_OPTIONS_REG_FACTORY, OnUpdateMenuStopped)
        ON_UPDATE_COMMAND_UI(IDM_OPTIONS_REG_DELETE,  OnUpdateRegDeleteCheck)
        ON_UPDATE_COMMAND_UI(IDM_OPTIONS_DISPLAY,     OnUpdateMenuStopped)
        ON_UPDATE_COMMAND_UI(IDM_OPTIONS_OUTPUT,      OnUpdateMenuStopped)
        ON_UPDATE_COMMAND_UI(IDM_OPTIONS_CLIP,        OnUpdateClipCheck)
        ON_UPDATE_COMMAND_UI(IDM_OPTIONS_AUTODC,      OnUpdateAutoDCCheck)
        ON_UPDATE_COMMAND_UI(IDM_VIEW_PLOTTRACE,      OnUpdatePlotTraceCheck)
        ON_UPDATE_COMMAND_UI(IDM_HELP_CONTENTS,       OnUpdateMenuStopped)
        ON_UPDATE_COMMAND_UI(IDM_HELP_ABOUT,          OnUpdateMenuStopped)
        //}}AFX_MSG_MAP
        ON_MESSAGE(WM_ENTERSIZEMOVE, OnEnterSizeMove)
        ON_MESSAGE(WM_EXITSIZEMOVE,  OnExitSizeMove)
END_MESSAGE_MAP()

static UINT indicators[] = { ID_SEPARATOR, };       // status line indicator


CScopeWindow::CScopeWindow() {

        TRACE0("CScopeWindow::CScopeWindow\n");


        // Initialize some variables

        pYclipmin = pYclipmax = pYlast = pYaxis = NULL;
        pPtArray  = pOldArray = NULL;
        CurrFont  = NULL;
        PenBack   = PenAxis = PenTitle = PenTrace = NULL;

        
        // Initialize dialog positions

        InitDialogPositions();
        

        // Create and load icon, menus, accelerators, etc
        
        LoadFrame( IDR_SCOPE, WS_OVERLAPPEDWINDOW | WS_VSCROLL | WS_HSCROLL );


        TRACE0("CScopeWindow::CScopeWindow - END\n");
}

CScopeWindow::~CScopeWindow() {

        TRACE0("CScopeWindow::~CScopeWindow\n");
}

BOOL CScopeWindow::PreCreateWindow(CREATESTRUCT& cs) {

        TRACE0("CScopeWindow::PreCreateWindow\n");

        // Modify default size
        
        cs.x  = 10;
        cs.y  = 20;
        cs.cx = 620;
        cs.cy = 450;

        return( CFrameWnd::PreCreateWindow(cs) );

}

void CScopeWindow::PostNcDestroy() {

        TRACE0("CScopeWindow::PostNcDestroy\n");

        // do nothing at this level, let child class delete "this"
}

int CScopeWindow::OnCreate(LPCREATESTRUCT lpCreateStruct) {

        TRACE0("CScopeWindow::OnCreate\n");

        int nsize, yppi, apoint;


        // Create parent class
                
        if (CFrameWnd::OnCreate(lpCreateStruct) == -1) {
                pAPP->ErrString = "CFrameWnd creation failed\n";
                pAPP->ExitError();
                return -1;
                }

        // Add status bar
        
        if (!m_wndStatusBar.Create(this) ||
                !m_wndStatusBar.SetIndicators(indicators,
                  sizeof(indicators)/sizeof(UINT))) {

                pAPP->ErrString = "Status bar creation failed\n";
                pAPP->ExitError();
                return -1;      // fail to create
                }

                

        // Set default colors, create pens

        SetColors();

        CreatePens();


        
        // Compute some important point sizes (2,8,20) as pixels
        
        CClientDC dc(this);

        yppi = dc.GetDeviceCaps( LOGPIXELSY );  // Y Pixels Per logical Inch

        minPoint = (yppi *  2) / 72;
        midPoint = (yppi *  8) / 72;
        maxPoint = (yppi * 20) / 72;

        apoint   = (yppi * 16) / 72 - 1;


        

        // Set up a font

        font.lfHeight         = 0;      // apoint;
        font.lfWidth          = 0;
        font.lfEscapement     = 0;
        font.lfOrientation    = 0;
        font.lfWeight         = FW_NORMAL;
        font.lfItalic         = FALSE;
        font.lfUnderline      = FALSE;
        font.lfStrikeOut      = FALSE;
        font.lfCharSet        = ANSI_CHARSET;
        font.lfOutPrecision   = OUT_DEFAULT_PRECIS;
        font.lfClipPrecision  = CLIP_DEFAULT_PRECIS;
        font.lfQuality        = DEFAULT_QUALITY;
        font.lfPitchAndFamily = VARIABLE_PITCH | FF_SWISS;
        
        strcpy(font.lfFaceName, "Arial");




        
        // Initialize plot arrays and variables

        LastPtsPlot = 0;
        pPtArray    = NULL;
        pOldArray   = NULL;
        



        // Allocate vertical plot position buffers
        
        nsize = pAPP->MaxChannel * sizeof( int );
        
        pYlast    = (int*) malloc( nsize );
        pYaxis    = (int*) malloc( nsize );
        pYclipmin = (int*) malloc( nsize );
        pYclipmax = (int*) malloc( nsize );


        if (!pYlast || !pYaxis || !pYclipmin || !pYclipmax) {
        
                pAPP->ErrString = "Could not allocate vertical position buffers.";
                pAPP->ExitError();
                return -1;  // failed create
                }



        
        
        // Set status bar 

        if (!pAPP->DoStatusBar)
                CFrameWnd::OnBarCheck(AFX_IDW_STATUS_BAR);


        
        // Compute plot positions (and allocate plot buffers if needed)

        if (!ComputePlotPositions())
                return -1;      // failed create



        // Set scroll bar ranges
        
        SetScrollBar( SB_BOTH );
        
        controlkey = FALSE;

        
        
        // Set title

        SetWindowTitle( );

        
        return 0;
}

void CScopeWindow::OnSize(UINT nType, int cx, int cy) {

        TRACE0("CScopeWindow::OnSize\n");

        CFrameWnd::OnSize(nType, cx, cy);
        
        if (!ComputePlotPositions())
                return;

}

void CScopeWindow::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) {

        TRACE0("CScopeWindow::OnHScroll\n");

        int first, line, page, maxstart;

        first    = pAPP->PltFirstSample;
        maxstart = pAPP->AcqNsamplesPlot - TraceWidth - 1;
        page     = TraceWidth;
        line     = maxstart / 20;
        
        if (line < 5)
                line = 5;
        if (maxstart < 0)
                maxstart = 0;


        switch (nSBCode) {
        
                case SB_PAGEUP:
                        first -= page;
                        break;
                        
                case SB_PAGEDOWN:
                        first += page;
                        break;
                        
                case SB_LINEUP:
                        first -= line;
                        break;
                        
                case SB_LINEDOWN:
                        first += line;
                        break;
                        
                case SB_TOP:
                        first = 0;
                        break;
                        
                case SB_BOTTOM:
                        first = maxstart;
                        break;
                        
                case SB_THUMBPOSITION:
                        first = nPos;
                        break;
                }


        // Ensure range is valid

        if (first < 0)
                first = 0;

        if (first > maxstart)
                first = maxstart;


                
                
        // Update if changed

        if (first != pAPP->PltFirstSample) {
        
                SetScrollPos( SB_HORZ, first, TRUE );
                
                pAPP->PltFirstSample = first;
                
                Invalidate();

                }

        
}

void CScopeWindow::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) {

        TRACE0("CScopeWindow::OnVScroll\n");

        int first, maxstart;

        first    = pAPP->PltFirstTrace;
        maxstart = pAPP->AcqNchannels - pAPP->PltNdisplay;

        switch (nSBCode) {
        
                case SB_PAGEUP:
                        first -= pAPP->PltNdisplay;
                        break;
                        
                case SB_PAGEDOWN:
                        first += pAPP->PltNdisplay;
                        break;
                        
                case SB_LINEUP:
                        first--;
                        break;
                        
                case SB_LINEDOWN:
                        first++;
                        break;
                        
                case SB_TOP:
                        first = 0;
                        break;
                        
                case SB_BOTTOM:
                        first = maxstart;
                        break;
                        
                case SB_THUMBPOSITION:
                        first = nPos;
                        break;
                }


        // Ensure range is valid

        if (first < 0)
                first = 0;

        if (first > maxstart)
                first = maxstart;


                
        // Update if changed

        if (first != pAPP->PltFirstTrace) {
        
                SetScrollPos( SB_VERT, first, TRUE );
                
                pAPP->PltFirstTrace = first;
                pAPP->ResetPaint    = TRUE;
                
                Invalidate();

                }

                        
}

void CScopeWindow::OnPaint() {

        TRACE0("CScopeWindow::OnPaint\n");
                
        // No paint when minimized
        
        if ( IsIconic() )
                return;

        
        // Delay paint when dragging for a move or resize
        
        if ( IsDragging ) {
                PaintDeferred = TRUE;
                return;
                }

        
        // Only listen to DoPlotTrace flag when running
        
        if (pAPP->IsRunning && !pAPP->DoPlotTrace)
                return;



        // Redraw plot  

        CPaintDC dc(this); // device context for painting

        DrawBackground( &dc );

        DrawTitles( &dc );
        
        DrawAxes( &dc );



        
        // Check for overflow

        if (pAPP->IsRunning && pAPP->AtodCheckOverflow())
                return;



        // Redraw traces

        pAPP->ResetPaint = TRUE;
        
        if (pAPP->IsAllocated)
                DrawTraces( &dc );

                

        // Do not call CFrameWnd::OnPaint() for painting messages
}

LRESULT CScopeWindow::OnEnterSizeMove( WPARAM wParam, LPARAM lParam ) {
        
        TRACE0("CScopeWindow::OnEnterSizeMove\n");

        // Prevent any painting while dragging for move/resize
        
        IsDragging = TRUE;
        return LRESULT(0);
}


LRESULT CScopeWindow::OnExitSizeMove( WPARAM wParam, LPARAM lParam ) {
        
        TRACE0("CScopeWindow::OnExitSizeMove\n");

        // Drag for move/resize is done, so allow painting and
        // force a paint if any have been skipped during the drag.
        
        IsDragging = FALSE;
        if (PaintDeferred) {
                PaintDeferred = FALSE;
                Invalidate(FALSE);
        }
        return LRESULT(0);
}


void CScopeWindow::SetClipLimits( int height, int traceHalf ) {

        TRACE0("CScopeWindow::SetClipLimits\n");

        int i;
        
        if (pAPP->DoClipping)
                for ( i = 0 ; i < pAPP->MaxChannel ; i++ ) {
                        pYclipmin[i] = pYaxis[i] - traceHalf;
                        pYclipmax[i] = pYaxis[i] + traceHalf;
                        }

        else
                for ( i = 0 ; i < pAPP->MaxChannel ; i++ ) {
                        pYclipmin[i] = 0;
                        pYclipmax[i] = height-1;
                        }
}


void CScopeWindow::DrawBackground( CDC *pDC ) {

        TRACE0("CScopeWindow::DrawBackground\n");

        int height, width;

        GetSize( &height, &width );

        pDC->FillSolidRect( 0, 0, width+1, height+1, ColorBack );
}


void CScopeWindow::DrawTitles( CDC *pDC ) {

        TRACE0("CScopeWindow::DrawTitles\n");
        
        int i, ch2, itrace, ititle;
        

        // clear title area

        
        // draw titles

        pDC->SelectObject( CurrFont );
        pDC->SetTextColor( ColorTitle );
        pDC->SetBkMode(TRANSPARENT);

        ch2    = cyChar/2;
        itrace = pAPP->PltFirstTrace;

        for ( i = 0 ; i < pAPP->PltNdisplay ; i++ ) {

                ititle = pAPP->pScanOrder[itrace];
                
                pDC->TextOut( HMARGIN, pYaxis[i]-ch2,
                                &(pAPP->pScanTitle[ititle*MAXTITLE]) );
                
                itrace++;
                
                }
        
}


void CScopeWindow::DrawAxes( CDC *pDC ) {

        TRACE0("CScopeWindow::DrawAxes\n");
        
        int i;

        
        pDC->SelectObject( PenAxis );
        
        for ( i = 0 ; i < pAPP->PltNdisplay ; i++ ) {

                pDC->MoveTo( TraceStart, pYaxis[i] );
                pDC->LineTo( TraceEnd+1, pYaxis[i] );
                
                }
}


void CScopeWindow::DrawTraces( CDC *pDC ) {

        TRACE0("CScopeWindow::DrawTraces\n");

        int npoints, j, k, kend, height, width;



        // Correct number of points based on TraceWidth and max plot points     

        kend = pAPP->PltFirstSample + TraceWidth;

        if (kend > pAPP->AcqNsamplesPlot)
                kend = pAPP->AcqNsamplesPlot;
        
        npoints = kend - pAPP->PltFirstSample;



        
        // Fill plot arrays from data arrays with clipping      

        FillTraces( npoints );

        


        // Blankout and draw new traces
        
        k = 0;
        
        if (pAPP->ResetPaint == TRUE) {

                pAPP->ResetPaint = FALSE;

                GetSize( &height, &width );
                
                pDC->FillSolidRect(TraceStart,0,TraceWidth,height,ColorBack);

                DrawAxes( pDC );
                
                pDC->SelectObject( PenTrace );
                
                for ( j = 0 ; j < pAPP->PltNdisplay ; j++ ) {

                        pDC->Polyline( &(pPtArray[k]), npoints );

                        k += npoints;

                        }
                }


                
        // Or erase old and draw new traces

        else {

                for ( j = 0 ; j < pAPP->PltNdisplay ; j++ ) {

                        pDC->SelectObject( PenBack );
                        pDC->Polyline( &(pOldArray[k]), npoints );


                        pDC->SelectObject( PenAxis );
                        pDC->MoveTo( TraceStart, pYaxis[j] );
                        pDC->LineTo( TraceEnd+1, pYaxis[j] );
                        

                        pDC->SelectObject( PenTrace );
                        pDC->Polyline( &(pPtArray[k]), npoints );

                        k += npoints;

                        }
                }

        
}

void CScopeWindow::FillTraces( int npoints ) {

        TRACE0("CScopeWindow::FillTraces\n");

        int kstart, jstart, jend, jtrace;
        int i, j, k, kdata, joff, xpos;
        int yval, yax, yscaled, yclipmin, yclipmax;


        
        // Prepare point array with all points (some new and some old)
        
        k      = 0;
        jtrace = pAPP->PltFirstTrace;
        kstart = pAPP->Iplotbuff * pAPP->AcqNsamples + pAPP->PltFirstSample;

        if (kstart > pAPP->AcqNsamplesPlot)
                kstart -= pAPP->AcqNsamplesPlot;

                        
        for ( j = 0 ; j < pAPP->PltNdisplay ; j++ ) {
                
                xpos     = TraceStart;
                yax      = pYaxis[j];
                yclipmin = pYclipmin[j];
                yclipmax = pYclipmax[j];
                
                joff     = pAPP->pDCoffset[jtrace];
                jstart   = jtrace*pAPP->AcqNsamplesPlot;
                jend     = jstart + pAPP->AcqNsamplesPlot;
                kdata    = jstart + kstart;

                for ( i = 0 ; i < npoints ; i++ ) {

                        // Fill old trace points

                        pOldArray[k].x = pPtArray[k].x;
                        pOldArray[k].y = pPtArray[k].y;


                        
                        // Remove DC offset
                        
                        yval = (pAPP->pDemuxArray[kdata] - joff);

                                                

                        // Scale, rounding towards zero as per DOS scope

                        if (yval >= 0)
                                yscaled = yax - (yval >> pAPP->PltShiftBy);
                        else
                                yscaled = yax + (-yval >> pAPP->PltShiftBy);

                        

                        // Clip

                        if (yscaled < yclipmin)
                                yscaled = yclipmin;
                        
                        else if (yscaled > yclipmax)
                                yscaled = yclipmax;


                                                
                        // Fill new trace points
                        
                        pPtArray[k].x = xpos++;
                        pPtArray[k].y = yscaled;


                        
                        
                        k++;
                        kdata++;
                        if (kdata >= jend)
                                kdata = jstart;

                        }
                
                jtrace++;

                }

}


void CScopeWindow::OnDestroy() {

        TRACE0("CScopeWindow::OnDestroy\n");
        

        // Clean up pens
        
        DeletePens();


        
        // Clean up last font

        DeleteFont();


                
        // Clean up plot arrays

        DeletePlotArrays();

        
                                
        // Clean up vertical plot position buffers

        DeleteVerticalBuffers();

                                


                        
        CFrameWnd::OnDestroy();
}


void CScopeWindow::OnTimer(UINT nIDEvent) {

        TRACE0("CScopeWindow::OnTimer\n");


        // check overflow

        if ( pAPP->AtodCheckOverflow() ) {
                pAPP->ErrString = "A/D Overflow occurred.";
                pAPP->ShowError();
                return;
                }



        
        
        // Check ready buffer and process if ready          

        if (pAPP->AtodCheckReady() != 0) {

                pAPP->AtodGetData();

                
                // write data if trigger condition is satisfied

                if (pAPP->TriggerCondition() == 1) {
                        
                        if (!pAPP->WriteData()) {
                                OnStopProgram();
                                return;
                                }
                        }

                
                // check overflow

                if ( pAPP->AtodCheckOverflow() ) {
                        pAPP->ErrString = "A/D Overflow occurred after write.";
                        pAPP->ShowError();
                        return;
                        }



                // demux data

                pAPP->AtodDemuxData();



                // increment plot buff index

                if (pAPP->Iplotbuff >= pAPP->Nplotbuffers-1)
                        pAPP->Iplotbuff = 0;
                else
                        pAPP->Iplotbuff++;
                
                
                // draw traces

                if (pAPP->DoPlotTrace) {
                        CClientDC dc(this);
                        DrawTraces( &dc );
                        }


                // check overflow

                if ( pAPP->AtodCheckOverflow() ) {
                        pAPP->ErrString = "A/D Overflow occurred after draw.";
                        pAPP->ShowError();
                        return;
                        }
                
                }


        
        
        
        CFrameWnd::OnTimer(nIDEvent);  // calls default which calls defwinproc
}



void CScopeWindow::OnRunProgram() {
        
        TRACE0("CScopeWindow::OnRunProgram\n");
        
        int interval;



        // Initialize A/D board

        if ( ! pAPP->AtodInit() ) {
                
                pAPP->IsAllocated = FALSE;      // stops trace draw
                pAPP->ShowError();
                return;
                
                }

                
        
        
        // Allocate buffer

        if ( ! pAPP->AllocateBuffer() )
                return;


        
        
        // Compute timer interval
        
        if (pAPP->AcqSPS > 0.0001)
                interval = (int)(MSPERSEC*pAPP->AcqNsamples/
                                                (pAPP->AcqSPS*TICKINBUFF));
        else
                interval = 20;

        
        
        // Set block counter
        
        pAPP->CurrBlock = 0;
        

        
        // Output header

        pAPP->WriteHdr();



        

        // Begin A/D execution

        pAPP->AtodStart();

        
        // Enable timer 

        if ( ! SetTimer( IDT_TIMER, interval, NULL ) ) {
                
                pAPP->AtodStop();
                AfxMessageBox( "Not enough timers.  Close some other apps." );
                return;
                }

        
        pAPP->IsRunning = TRUE;

        
        // Set title

        SetWindowTitle( );
                
        
}


void CScopeWindow::OnStopProgram() {

        TRACE0("CScopeWindow::OnStopProgram\n");

        if (pAPP->IsRunning) {

                pAPP->AtodStop();
                KillTimer(IDT_TIMER);
        
                pAPP->IsRunning = FALSE;
                SetWindowTitle( );
                }
}

void CScopeWindow::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) {
        
        TRACE0("CScopeWindow::OnKeyUp\n");

        
        // Keep track of control key
        
        if (nChar == VK_CONTROL)
                
                controlkey = FALSE;


        

        // Default handling
        
        CFrameWnd::OnKeyUp(nChar, nRepCnt, nFlags);
}

void CScopeWindow::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) {

        TRACE0("CScopeWindow::OnKeyDown\n");

        int scancode;




        // Keep track of control key
        
        if (nChar == VK_CONTROL)
                
                controlkey = TRUE;

                

        // Translate scan codes into a reliable virtkey value

        scancode = nFlags & 0x00FF;

        if ( (scancode == 0x0C) || (scancode == 0x33) ) // -_ or ,< keys
                nChar = VK_SUBTRACT;
                
        if ( (scancode == 0x0D) || (scancode == 0x34) ) // =+ or .> keys
                nChar = VK_ADD;
                


        
        switch (nChar) {
                
            // Increase amplitude (decrease shift)
                
            case 0xBB:          // =+ key
            case 0xBE:          // .> key
            case VK_ADD:        // numpad + key

                pAPP->PltShiftBy -= nRepCnt;
                
                if (pAPP->PltShiftBy < pAPP->MinAmpShift)
                        pAPP->PltShiftBy = pAPP->MinAmpShift;

                UpdateTitleTraces();

                break;



                
            // Decrease amplitude (increase shift)
                
            case 0xBD:          // -_ key
            case 0xBC:          // ,< key
            case VK_SUBTRACT:   // numpad - key
                
                pAPP->PltShiftBy += nRepCnt;
                
                if (pAPP->PltShiftBy > pAPP->MaxAmpShift-1)
                        pAPP->PltShiftBy = pAPP->MaxAmpShift-1;
                                
                UpdateTitleTraces();
                
                break;


                


            // Process arrow keys for scrolling

            case VK_UP:
                SendMessage(WM_VSCROLL, MAKELONG(SB_LINEUP, 0), 0L);
                break;

            case VK_DOWN:
                SendMessage(WM_VSCROLL, MAKELONG(SB_LINEDOWN, 0), 0L);
                break;

            case VK_LEFT:
                SendMessage(WM_HSCROLL, MAKELONG(SB_LINEUP, 0), 0L);
                break;

            case VK_RIGHT:
                SendMessage(WM_HSCROLL, MAKELONG(SB_LINEDOWN, 0), 0L);
                break;



            // Process page up/down keys for scrolling
        
            case VK_PRIOR:    //PAGEUP key

                if (controlkey)
                        SendMessage(WM_HSCROLL, MAKELONG(SB_PAGEUP, 0), 0L);
                else
                        SendMessage(WM_VSCROLL, MAKELONG(SB_PAGEUP, 0), 0L);
                break;

            case VK_NEXT:     // PAGEDOWN key

                if (controlkey)
                        SendMessage(WM_HSCROLL, MAKELONG(SB_PAGEDOWN, 0), 0L);
                else
                        SendMessage(WM_VSCROLL, MAKELONG(SB_PAGEDOWN, 0), 0L);
                break;




            // Process home/end keys for scrolling

            case VK_HOME:

                if (controlkey)
                        SendMessage(WM_VSCROLL, MAKELONG(SB_TOP, 0), 0L);
                else
                        SendMessage(WM_HSCROLL, MAKELONG(SB_TOP, 0), 0L);
                break;

            case VK_END:

                if (controlkey)
                        SendMessage(WM_VSCROLL, MAKELONG(SB_BOTTOM, 0), 0L);
                else
                        SendMessage(WM_HSCROLL, MAKELONG(SB_BOTTOM, 0), 0L);
                break;


                
         }



        
        // Default handling
        
        CFrameWnd::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CScopeWindow::OnExit() {

        TRACE0("CScopeWindow::OnExit\n");

        if (pAPP->IsRunning) {
                AfxMessageBox("Please stop before exiting\n");
                return;
                }


        this->DestroyWindow();
}

void CScopeWindow::OnRegFactoryDialog() {

        TRACE0("CScopeWindow::OnRegFactoryDialog\n");
        
        CRegistryDlg dlg(this);

        
        if (dlg.DoModal() == IDOK) {

                CClientDC dc(this);
                ComputeMaxTitleLength( &dc );
                
                UpdateColorScheme();
                if (!ComputePlotPositions())
                        return;
                Invalidate();
                SetWindowTitle();
                }
        
        controlkey = FALSE;
}

void CScopeWindow::OnDisplayDialog() {

        TRACE0("CScopeWindow::OnDisplayDialog\n");
        
        int         oldColorScheme, oldNdisplay;
        CDisplayDlg dlg(this);

        oldColorScheme = pAPP->PltColorScheme;
        oldNdisplay    = pAPP->PltNdisplay;
        
        if (dlg.DoModal() == IDOK) {

                if (oldColorScheme != pAPP->PltColorScheme)
                        UpdateColorScheme();
                if (oldNdisplay != pAPP->PltNdisplay) {
                        pAPP->ComputeBufferSizes();
                        if (!ComputePlotPositions())
                                return;
                        }
                
                SetScrollBar( SB_BOTH );
                
                Invalidate();                    // cause repaint
                SetWindowTitle();
                }
        
        controlkey = FALSE;
}

void CScopeWindow::OnOutputDialog() {

        TRACE0("CScopeWindow::OnOutputDialog\n");

        int oldNsamples;

        oldNsamples = pAPP->AcqNsamples;
        
        COutputDlg dlg( pAPP->ROnsamples, this );

        if (dlg.DoModal() == IDOK) {

                if (oldNsamples < pAPP->AcqNsamples)
                        pAPP->IsAllocated = FALSE;

                pAPP->ComputeBufferSizes();
                
                SetScrollBar( SB_HORZ );
                
                }
        
        controlkey = FALSE;
}

void CScopeWindow::OnSetClip() {

        TRACE0("CScopeWindow::OnSetClip\n");
        
        int height, width;
        
        pAPP->DoClipping = !pAPP->DoClipping;
        
        GetSize( &height, &width );
                
        SetClipLimits( height, (pYaxis[1]-pYaxis[0])/2 );

        UpdateTitleTraces();
}

void CScopeWindow::OnSetDC() {

        TRACE0("CScopeWindow::OnSetDC\n");
        

        pAPP->ResetDC = TRUE;
        
        MessageBeep(MB_ICONASTERISK);
        
#if !defined( SROS_WINNT)                // One beep is enough for NT       
        MessageBeep(MB_ICONASTERISK);
        MessageBeep(MB_ICONASTERISK);
#endif        
}

void CScopeWindow::OnSetAutoDC() {

        TRACE0("CScopeWindow::OnSetAutoDC\n");
        
        pAPP->AutoDC  = !pAPP->AutoDC;
        pAPP->ResetDC = TRUE;

        SetWindowTitle();
}



void CScopeWindow::OnSetRegDelete() {

        TRACE0("CScopeWindow::OnSetRegDelete\n");
        
        pAPP->DoRegistryDelete = !pAPP->DoRegistryDelete;

}

void CScopeWindow::OnSetStatusBar() {

        TRACE0("CScopeWindow::OnSetStatusBar\n");

        pAPP->DoStatusBar = !pAPP->DoStatusBar;
        
        CFrameWnd::OnBarCheck(AFX_IDW_STATUS_BAR);
        
        if (!ComputePlotPositions())
                return;
        Invalidate();
}

void CScopeWindow::OnSetPlotTrace() {

        TRACE0("CScopeWindow::OnSetPlotTrace\n");
        
        pAPP->DoPlotTrace = !pAPP->DoPlotTrace;

        SetWindowTitle();

        if (pAPP->DoPlotTrace)
                Invalidate();
}

void CScopeWindow::UpdateTitleTraces() {

        TRACE0("CScopeWindow::UpdateTitleTraces\n");
        
        // Update title
        
        SetWindowTitle( );


        // Update traces if not running
        
        if (!pAPP->IsRunning && pAPP->IsAllocated) {
                CClientDC dc(this);
                DrawTraces( &dc );
                }
}


void CScopeWindow::UpdateColorScheme() {

        TRACE0("CScopeWindow::UpdateColorScheme\n");
        
        // Remove old pens

        DeletePens();


        // Set colors according to current scheme

        SetColors();


        // Create new pens

        CreatePens();
}


void CScopeWindow::SetColors() {

        TRACE0("CScopeWindow::SetColors\n");
        
        switch (pAPP->PltColorScheme) {

        case CS_MONO:

                ColorBack  = RGB(  0,   0,   0);        // Black
                ColorAxis  = RGB(192, 192, 192);        // White Dim
                ColorTitle = RGB(192, 192, 192);        // White Dim
                ColorTrace = RGB(255, 255, 255);        // White
                break;


        case CS_GREEN:

                ColorBack  = RGB(  0,   0,   0);        // Black
                ColorAxis  = RGB(128, 128, 128);        // Gray
                ColorTitle = RGB(  0, 128,   0);        // Green Dark
                ColorTrace = RGB(  0, 255,   0);        // Green
                break;


        case CS_BLUE:

                ColorBack  = RGB(  0,   0, 128);        // Blue Dark
                ColorAxis  = RGB(128, 128, 128);        // Gray
                ColorTitle = RGB(255, 255, 255);        // White
                ColorTrace = RGB(  0, 255, 255);        // Cyan
                break;

                
        case CS_BROWN:

                ColorBack  = RGB(128,   0,   0);        // Brown
                ColorAxis  = RGB(128, 128, 128);        // Gray
                ColorTitle = RGB(255, 255,   0);        // Yellow
                ColorTrace = RGB(255, 255, 255);        // White
                break;

                
        default:
                ColorBack  = RGB(  0,   0,   0);        // Black
                ColorAxis  = RGB(128, 128, 128);        // Gray
                ColorTitle = RGB(  0, 128,   0);        // Green Dark
                ColorTrace = RGB(  0, 255,   0);        // Green
                break;
                
        }


}

void CScopeWindow::CreatePens() {

        TRACE0("CScopeWindow::CreatePens\n");
                
        PenBack  = new CPen( PS_SOLID, 1, ColorBack  );
        PenAxis  = new CPen( PS_SOLID, 1, ColorAxis  );
        PenTitle = new CPen( PS_SOLID, 1, ColorTitle );
        PenTrace = new CPen( PS_SOLID, 1, ColorTrace );
}
        
void CScopeWindow::DeletePens() {

        TRACE0("CScopeWindow::DeletePens\n");
                
        // Make sure pens are not selected
        
        CClientDC dc(this);
        dc.SelectObject(GetStockObject(BLACK_PEN));


        // Delete them (destructor calls DeleteObject)
        
        delete PenBack;   PenBack  = NULL;
        delete PenAxis;   PenAxis  = NULL;
        delete PenTitle;  PenTitle = NULL;
        delete PenTrace;  PenTrace = NULL;

}

void CScopeWindow::CreateFont() {

        TRACE0("CScopeWindow::CreateFont\n");

        if (CurrFont != NULL)
                DeleteFont();


        // Compensate for including Internal Leading here
        
        if (font.lfHeight > 0)
                font.lfHeight = -font.lfHeight;



        
        CurrFont = new CFont();
        CurrFont->CreateFontIndirect( &font );
        
        CurrFont->GetLogFont( &font );



        
        // Uncompensate for including Internal Leading here
        
        if (font.lfHeight < 0)
                font.lfHeight = -font.lfHeight;

}

void CScopeWindow::DeleteFont() {

        TRACE0("CScopeWindow::DeleteFont\n");
                
        // Make sure font is not selected
        
        CClientDC dc(this);
        dc.SelectObject(GetStockObject(SYSTEM_FIXED_FONT));


        // Delete it (destructor calls DeleteObject)

        delete CurrFont;        CurrFont = NULL;

}

void CScopeWindow::DeletePlotArrays( ) {
        
        TRACE0("CScopeWindow::DeletePlotArrays\n");

        if (pPtArray)
                { free(pPtArray);  pPtArray  = NULL; }

        if (pOldArray)
                { free(pOldArray); pOldArray = NULL; }

}       

void CScopeWindow::DeleteVerticalBuffers( ) {
        
        TRACE0("CScopeWindow::DeleteVerticalBuffers\n");
        
        if (pYlast)
                { free(pYlast);    pYlast    = NULL; }
        
        if (pYaxis)
                { free(pYaxis);    pYaxis    = NULL; }
        
        if (pYclipmin)
                { free(pYclipmin); pYclipmin = NULL; }
        
        if (pYclipmax)
                { free(pYclipmax); pYclipmax = NULL; }

}

void CScopeWindow::UpdateTextSize( CDC *pDC ) {
        
        TRACE0("CScopeWindow::UpdateTextSize\n");

        
        TEXTMETRIC tm;


        // Update text dimensions
        
        pDC->SelectObject( CurrFont );
        pDC->GetTextMetrics( &tm );
        
        cxChar = tm.tmAveCharWidth;
        cyChar = tm.tmHeight + tm.tmExternalLeading;

}
        

        
void CScopeWindow::OnContentsDialog() {

        TRACE0("CScopeWindow::OnContentsDialog\n");
                
        CContentsDlg dlg(this);
        dlg.DoModal();
        
        controlkey = FALSE;
}

void CScopeWindow::OnAboutDialog() {

        TRACE0("CScopeWindow::OnAboutDialog\n");
                
        CAboutDlg dlg(this);
        dlg.DoModal();
        
        controlkey = FALSE;
}

void CScopeWindow::OnDebug() {

        TRACE0("CScopeWindow::OnDebug\n");


        CString msg;

                
        CClientDC dc(this);

        int lpy  = dc.GetDeviceCaps( LOGPIXELSY );
        int vres = dc.GetDeviceCaps( VERTRES );

        msg.Format("LOGPIXELSY = %d, VERTRES = %d\n",lpy,vres);
        AfxMessageBox(msg);
}


void CScopeWindow::OnUpdateMenuRunning(CCmdUI* pCmdUI) {

        TRACE0("CScopeWindow::OnUpdateMenuRunning\n");
        
        pCmdUI->Enable( pAPP->IsRunning );
}

void CScopeWindow::OnUpdateMenuStopped(CCmdUI* pCmdUI) {

        TRACE0("CScopeWindow::OnUpdateMenuStopped\n");
                
        pCmdUI->Enable( !pAPP->IsRunning );
}

void CScopeWindow::OnUpdateRegDeleteCheck(CCmdUI* pCmdUI) {

        TRACE0("CScopeWindow::OnUpdateRegDeleteCheck\n");
        
        pCmdUI->SetCheck( pAPP->DoRegistryDelete );
}

void CScopeWindow::OnUpdateClipCheck(CCmdUI* pCmdUI) {

        TRACE0("CScopeWindow::OnUpdateClipCheck\n");
        
        pCmdUI->SetCheck( pAPP->DoClipping );
}

void CScopeWindow::OnUpdateAutoDCCheck(CCmdUI* pCmdUI) {

        TRACE0("CScopeWindow::OnUpdateAutoDCCheck\n");
        
        pCmdUI->SetCheck( pAPP->AutoDC );
}

void CScopeWindow::OnUpdatePlotTraceCheck(CCmdUI* pCmdUI) {

        TRACE0("CScopeWindow::OnUpdatePlotTraceCheck\n");
        
        pCmdUI->SetCheck( pAPP->DoPlotTrace );
}

void CScopeWindow::SetScrollBar( int nBar ) {

        TRACE0("CScopeWindow::SetScrollBar\n");


        SCROLLINFO sif;

        sif.cbSize = sizeof(SCROLLINFO);
        sif.fMask  = SIF_ALL;
        

        if ((nBar == SB_VERT) || (nBar == SB_BOTH)) {

                sif.nMin  = 0;
                sif.nMax  = pAPP->AcqNchannels - 1;
                sif.nPage = pAPP->PltNdisplay;
                sif.nPos  = pAPP->PltFirstTrace;


                SetScrollInfo( SB_VERT, &sif, TRUE );
                
                }


        if ((nBar == SB_HORZ) || (nBar == SB_BOTH)) {

                sif.nMin  = 0;
                sif.nMax  = pAPP->AcqNsamplesPlot - 1;
                sif.nPage = TraceWidth;
                sif.nPos  = pAPP->PltFirstSample;


                SetScrollInfo( SB_HORZ, &sif, TRUE );

                }

}

void CScopeWindow::SetWindowTitle() {

        TRACE0("CScopeWindow::SetWindowTitle\n");
        
        CString title;
        CString bitstr;


        bitstr.Format("%d bits", pAPP->MaxAmpShift - pAPP->PltShiftBy);

                
        title = pAPP->AppTitle;

        if (pAPP->IsRunning == TRUE)
                title += " - Running (Ctrl+S to stop)";

        if (pAPP->DoPlotTrace == TRUE) {
                if (pAPP->DoClipping == TRUE)
                        title += " - Clipping ";
                else
                        title += " - Showing ";
                }
        else
                title += " - No Plot ";

        title += bitstr;

        if (pAPP->AutoDC == TRUE)
                title += " - Auto DC";


                                
        SetWindowText( title );
}

void CScopeWindow::GetSize( int *height, int *width ) {
        
        TRACE0("CScopeWindow::GetSize\n");

        RECT r, rs;
        
        GetClientRect(&r);

        if (m_wndStatusBar.IsWindowVisible())
                m_wndStatusBar.GetClientRect( &rs );
        else
                rs.bottom = 0;


        *height = r.bottom - rs.bottom;
        *width  = r.right;
}

int CScopeWindow::ChangeFontSize( int newsize ) {

        TRACE0("CScopeWindow::ChangeFontSize\n");



        // Enforce reasonable sizes
        
        if (newsize < minPoint)
                newsize = minPoint;
        if (newsize > maxPoint)
                newsize = maxPoint;


        
        // Test for no change
        
        if (font.lfHeight == newsize)
                return( 0 );


        
        // Create font with new size

        font.lfWidth  = 0;
        font.lfHeight = newsize;


        if (newsize < midPoint)
                strcpy(font.lfFaceName,"Small Fonts");
        else
                strcpy(font.lfFaceName,"Arial");
        

        CreateFont();





        // Refigure text size and title length

        CClientDC dc(this);
        
        UpdateTextSize( &dc );
        
        ComputeMaxTitleLength( &dc );


        return( 1 );
        
}

int CScopeWindow::ComputeMaxTitleLength( CDC *pDC ) {

        TRACE0("CScopeWindow::ComputeMaxTitleLength\n");

        int       i, ncount;
        CSize     sz(0,0);
        LPCTSTR   pstr;

        MaxTitleLength = 0;
        
        pDC->SelectObject( CurrFont );

        
        for ( i = 0 ; i < pAPP->MaxChannel ; i++ ) {


                if (pAPP->pScanActive[i] == CHECKED) {

                        pstr   = &(pAPP->pScanTitle[i*MAXTITLE]);
                        ncount = strlen( pstr );

                        sz     = pDC->GetTextExtent( pstr, ncount );

                        if (MaxTitleLength < sz.cx)
                                MaxTitleLength = sz.cx;
                        }
        
                }

        return( MaxTitleLength );

}

int CScopeWindow::ComputePlotPositions( ) {

        TRACE0("CScopeWindow::ComputePlotPositions\n");

        int    totalHeight, traceHalf, evenHeight, halfExcess;
        int    height, width, sbextra, maxFirstSample;
        int    i, j, xpos, yax;
        POINT *pArray;
        
        SCROLLINFO sif;


        // Always check the return value of this function and return
        // immediately if it fails.  This is because failure means that
        // the allocate plot buffer function has failed and the destroy
        // window function may have been called so the entire CScopeWindow
        // is deleted.  You can't access any class variables at that point,
        // so all functions should just unwind with returns.

                


        // Get size of window

        GetSize( &height, &width );



        // Always compute totalHeight assuming that horizontal scroll bar
        // is visible to avoid race condition where size without bar
        // increases font which reduces width so that bar is needed, but
        // size with bar has smaller font that increases width so bar
        // is not needed, etc

        GetScrollInfo( SB_HORZ, &sif );
        
        if ((int)sif.nPage >= sif.nMax)
                sbextra = GetSystemMetrics( SM_CYHSCROLL );
        else
                sbextra = 0;

        

        // Compute trace height

        totalHeight = height - 2 * VMARGIN - sbextra;
        TraceHeight = (int)((float)totalHeight / pAPP->PltNdisplay);

        
        ChangeFontSize( TraceHeight );

        


        // Compute trace width

        TraceStart = HMARGIN + MaxTitleLength + HMARGIN;
        TraceEnd   = width - HMARGIN;
        TraceWidth = TraceEnd - TraceStart;

        if (TraceWidth < 1)
                TraceWidth = 1;



        
        // Compute trace height positions

        traceHalf   = TraceHeight / 2;
        evenHeight  = TraceHeight * pAPP->PltNdisplay;
        halfExcess  = (totalHeight - evenHeight) / 2;

        pYaxis[0] = VMARGIN +  halfExcess + traceHalf;
        pYlast[0] = pYaxis[0];

        for ( i = 1 ; i < pAPP->MaxChannel ; i++ ) {
                pYaxis[i] = pYaxis[i - 1] + TraceHeight;
                pYlast[i] = pYaxis[i];
                }


        // Update clipping
        
        SetClipLimits( height, traceHalf );




        // If needed, reallocate plot buffers

        if (!AllocatePlotBuffers())
                return( 0 );



        // Reset horizontal scroll bar range
        
        maxFirstSample = pAPP->AcqNsamplesPlot - TraceWidth - 1;
        if (maxFirstSample < 0)
                maxFirstSample = 0;
        if (pAPP->PltFirstSample > maxFirstSample)
                pAPP->PltFirstSample = maxFirstSample;
                        
        SetScrollBar( SB_HORZ );
        
        
                
        // Fill up polyline positions

        pAPP->ResetPaint = TRUE;
        
        pArray = pPtArray;

        for ( j = 0 ; j < pAPP->AcqNchannels; j++ ) {

                xpos = TraceStart;
                yax  = pYaxis[j];
                
                for ( i = 0 ; i < pAPP->AcqNsamplesPlot ; i++ ) {
                        
                        pArray->x = xpos++;
                        pArray->y = yax;
                        
                        pArray++;
                        }
                }

        // Need to replot
        
        Invalidate( FALSE );

        return( 1 );
}


int CScopeWindow::AllocatePlotBuffers() {

        TRACE0("CScopeWindow::AllocatePlotBuffers\n");


        // Only reallocate if need to increase size
        
        if (LastPtsPlot < pAPP->NbufferptsPlot) {

                LastPtsPlot = pAPP->NbufferptsPlot;


                // Clean up any previous plot arrays
                
                DeletePlotArrays( );


                // Allocate current arrays
                
                pPtArray  = (POINT*) malloc(pAPP->NbufferptsPlot * sizeof(POINT));
                pOldArray = (POINT*) malloc(pAPP->NbufferptsPlot * sizeof(POINT));

                if (!pPtArray || !pOldArray) {
                        pAPP->ErrString = "Can not allocate plot buffers\n";
                        pAPP->ExitError();
                        return( 0 );
                        }
                                
                
                }


        // Fail with NULL arrays even if haven't reallocated
        if (!pPtArray || !pOldArray)
                return( 0 );

        return( 1 );

}


void CScopeWindow::InitDialogPositions( void ) {
        
        TRACE0("CScopeWindow::InitDialogPositions\n");

                
        // These dialogs are identical for all A/D systems

        RegistryPosTop  = DLG_POS_INIT;
        RegistryPosLeft = DLG_POS_INIT;
        
        DisplayPosTop   = DLG_POS_INIT;
        DisplayPosLeft  = DLG_POS_INIT;
        
        OutputPosTop    = DLG_POS_INIT;
        OutputPosLeft   = DLG_POS_INIT;
        
        ContentsPosTop  = DLG_POS_INIT;
        ContentsPosLeft = DLG_POS_INIT;
        
        TopicPosTop     = DLG_POS_INIT;
        TopicPosLeft    = DLG_POS_INIT;
        TopicPosBottom  = DLG_POS_INIT;
        TopicPosRight   = DLG_POS_INIT;


        // These dialogs vary by specific A/D system
        
        HardwarePosTop  = DLG_POS_INIT;
        HardwarePosLeft = DLG_POS_INIT;

        ChannelPosTop   = DLG_POS_INIT;
        ChannelPosLeft  = DLG_POS_INIT;
        
        AcquirePosTop   = DLG_POS_INIT;
        AcquirePosLeft  = DLG_POS_INIT;
        
}

void CScopeWindow::GetDialogPosition( RECT *r, int nID ) {
        
        TRACE0("CScopeWindow::GetDialogPosition\n");
        
        int height, width;

        
        // Keep original size
        
        height = r->bottom - r->top;
        width  = r->right  - r->left;

        
        // Get saved dialog positions
                
        switch ( nID ) {

                                
                // These dialogs are identical for all A/D systems
                
                case IDD_REGISTRY:
                        r->top    = RegistryPosTop;
                        r->left   = RegistryPosLeft;
                        r->bottom = r->top  + height;
                        r->right  = r->left + width;
                        break;
                        
                case IDD_OUTPUT:
                        r->top    = OutputPosTop;
                        r->left   = OutputPosLeft;
                        r->bottom = r->top  + height;
                        r->right  = r->left + width;
                        break;
                        
                case IDD_DISPLAY:
                        r->top    = DisplayPosTop;
                        r->left   = DisplayPosLeft;
                        r->bottom = r->top  + height;
                        r->right  = r->left + width;
                        break;
                        
                case IDD_HELPCONTENTS:
                        r->top    = ContentsPosTop;
                        r->left   = ContentsPosLeft;
                        r->bottom = r->top  + height;
                        r->right  = r->left + width;
                        break;
                        
                case IDD_HELPTOPIC:
                        r->top    = TopicPosTop;
                        r->left   = TopicPosLeft;
                        r->bottom = TopicPosBottom;
                        r->right  = TopicPosRight;
                        break;

                                                
                // These dialogs vary by specific A/D system
                        
                case IDD_HARDWARE:
                        r->top    = HardwarePosTop;
                        r->left   = HardwarePosLeft;
                        r->bottom = r->top  + height;
                        r->right  = r->left + width;
                        break;
                        
                case IDD_CHANNEL:
                        r->top    = ChannelPosTop;
                        r->left   = ChannelPosLeft;
                        r->bottom = r->top  + height;
                        r->right  = r->left + width;
                        break;
                        
                case IDD_ACQUIRE:
                        r->top    = AcquirePosTop;
                        r->left   = AcquirePosLeft;
                        r->bottom = r->top  + height;
                        r->right  = r->left + width;
                        break;

                                
                default:
                        r->top    = DLG_POS_INIT;
                        r->left   = DLG_POS_INIT;
                        r->bottom = DLG_POS_INIT;
                        r->right  = DLG_POS_INIT;
                        break;
                }


}


void CScopeWindow::SaveDialogPosition( RECT *r, int nID ) {
        
        TRACE0("CScopeWindow::SaveDialogPosition\n");


        // Save dialog positions
                
        switch ( nID ) {
                
                // These dialogs are identical for all A/D systems
                
                case IDD_REGISTRY:
                        RegistryPosTop  = r->top;
                        RegistryPosLeft = r->left;
                        break;
                        
                case IDD_OUTPUT:
                        OutputPosTop    = r->top;
                        OutputPosLeft   = r->left;
                        break;
                        
                case IDD_DISPLAY:
                        DisplayPosTop   = r->top;
                        DisplayPosLeft  = r->left;
                        break;
                        
                case IDD_HELPCONTENTS:
                        ContentsPosTop  = r->top;
                        ContentsPosLeft = r->left;
                        break;
                        
                case IDD_HELPTOPIC:
                        TopicPosTop     = r->top;
                        TopicPosLeft    = r->left;
                        TopicPosBottom  = r->bottom;
                        TopicPosRight   = r->right;
                        break;

                                                
                // These dialogs vary by specific A/D system
        
                case IDD_HARDWARE:
                        HardwarePosTop  = r->top;
                        HardwarePosLeft = r->left;
                        break;
                        
                case IDD_CHANNEL:
                        ChannelPosTop   = r->top;
                        ChannelPosLeft  = r->left;
                        break;
                        
                case IDD_ACQUIRE:
                        AcquirePosTop   = r->top;
                        AcquirePosLeft  = r->left;
                        break;

                }
}





//***********************************************************************
//
// CLASS CScopeDialog
//
// This class, derived from CDialog, contains a number of validation and
// convenience functions available to all the scope dialogs.  Basically,
// it is just an extension of the CDialog class
//

BEGIN_MESSAGE_MAP(CScopeDialog, CDialog)
        //{{AFX_MSG_MAP(CScopeDialog)
        ON_WM_ACTIVATEAPP()
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()




CScopeDialog::CScopeDialog(UINT nIdTemplate, CWnd* pParent /*=NULL*/)
             : CDialog(nIdTemplate, pParent) {

        TRACE0("CScopeDialog::CScopeDialog\n");


        DialogId   = nIdTemplate;
        CancelId   = IDCANCEL;
        DoValidate = TRUE;
}

BOOL CScopeDialog::OnInitDialog() {

        TRACE0("CScopeDialog::OnInitDialog\n");

        CDialog::OnInitDialog();


        // Return to saved position, if any
        
        RestorePosition();


        
        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CScopeDialog::OnOK() {

        TRACE0("CScopeDialog::OnOK\n");

        RememberPosition();

        CDialog::OnOK();
}

void CScopeDialog::OnCancel() {

        TRACE0("CScopeDialog::OnCancel\n");

        RememberPosition();

        CDialog::OnCancel();

        DoValidate = FALSE;
}

void CScopeDialog::OnActivateApp( BOOL bActive, HTASK hTask ) {

        TRACE0("CScopeDialog::OnActivateApp\n");
        
        CDialog::OnActivateApp( bActive, hTask );

        DoValidate = bActive;
}

LONG CScopeDialog::OnEditLostFocus( UINT nID, LONG xxx ) {

        TRACE1("CScopeDialog::OnEditLostFocus (%u)\n", nID);

        MessageBeep( 0 );
        
        return( 1 );
}

int CScopeDialog::CheckFocus() {

        TRACE0("CScopeDialog::CheckFocus\n");

        CWnd *pWnd;
        int   iret;

        // Call this function from inside button click routines.
        // Even though the click routine is being executed, the
        // previous control may still have the focus.  If it was
        // an edit control, it needs to go through the "losing
        // focus" validation function.
                
        pWnd = GetFocus();
        
        if (pWnd)
                iret = OnEditLostFocus(pWnd->GetDlgCtrlID(), 0L );
        else
                iret = 1;
        

        return( iret );

}

int CScopeDialog::ValidateAsInt( int nID, int minval, int maxval, int *theval, char *txt ) {

        TRACE0("CScopeDialog::ValidateAsInt\n");
                
        BOOL good;
        int  oldval, newval;


        oldval = *theval;
        
        newval = (int)GetDlgItemInt(nID, &good);

        if ((good) && (newval >= minval) && (newval <= maxval) ) {
                *theval = newval;
                return( TRUE );
                }
        else {
                CString msg;
                msg.Format("%s must be an integer between %d and %d", (LPCTSTR)txt, minval, maxval);
                AfxMessageBox(msg);
                SetDlgItemInt(nID, oldval);
                GotoDlgCtrl( GetDlgItem(nID) );
                return( FALSE );
                }

}

int CScopeDialog::ValidateAsHex( int nID, int minval, int maxval, int *theval, char *txt ) {

        TRACE0("CScopeDialog::ValidateAsHex\n");
        
        CString str;
        BOOL    good;
        int     oldval, newval;


        oldval = *theval;
        
        GetDlgItemText( nID, str );
        
        newval = strtol( str, NULL, 0 );
        
        if ( (newval == 0) || (newval == LONG_MAX) || (newval == LONG_MIN) )
                good = FALSE;
        else
                good = TRUE;

        if ((good) && (newval >= minval) && (newval <= maxval) ) {
                *theval = newval;
                return( TRUE );
                }
        else {
                str.Format("%s must be an integer between %d and %d", (LPCTSTR)txt, minval, maxval);
                AfxMessageBox(str);
                
                SetDlgItemHex( nID, oldval );
                GotoDlgCtrl( GetDlgItem(nID) );
                return( FALSE );
                }

}


int CScopeDialog::ValidateAsFloat( int nID, float minval, float maxval, float *theval, char *txt ) {

        TRACE0("CScopeDialog::ValidateAsFloat\n");
                
        CString btntxt;
        BOOL    good;
        float   oldval, newval;


        oldval = *theval;
        
        GetDlgItemText(nID, btntxt);
        newval = (float)strtod(btntxt,NULL);
        if (newval != 0.0)
                good = TRUE;
        else
                good = FALSE;

        if ((good) && (newval >= minval) && (newval <= maxval) ) {
                *theval = newval;
                return( TRUE );
                }
        else {
                CString msg;
                msg.Format("%s must be a float between %f and %f", (LPCTSTR)txt, minval, maxval);
                AfxMessageBox(msg);
                btntxt.Format("%f", oldval);
                SetDlgItemText(nID, btntxt);
                GotoDlgCtrl( GetDlgItem(nID) );
                return( FALSE );
                }

}


int CScopeDialog::SetDlgItemHex( int nID, int val ) {
        
        TRACE0("CScopeDialog::SetDlgItemHex\n");
        
        CString txt;
        txt.Format("0x%X", val);
        SetDlgItemText(nID, txt);

        return(TRUE);
}


int CScopeDialog::SetDlgItemFloat( int nID, float val ) {

        TRACE0("CScopeDialog::SetDlgItemFloat\n");
        
        CString txt;

        txt.Format("%.3f", val);
        SetDlgItemText( nID, txt );

        return(TRUE);
}


float CScopeDialog::GetDlgItemFloat(int nID, BOOL *lpTrans ) {
        
        CString txt;
        float   newval;
        
        GetDlgItemText(nID, txt);
        newval = (float)strtod(txt,NULL);

        
        // Fill error flag if desired
        
        if (lpTrans != NULL) {
                if (newval == 0.0)
                        *lpTrans = FALSE;
                else
                        *lpTrans = TRUE;
                }

        return( newval );
}


BOOL CScopeDialog::SetTextReadOnly( int nID ) {

        TRACE0("CScopeDialog::SetTextReadOnly\n");
        
        CEdit *pTxt = (CEdit*)GetDlgItem( nID );
        if (pTxt)
                return( pTxt->SetReadOnly( TRUE ) );
        else
                return( FALSE );
}

void CScopeDialog::RestorePosition() {
        
        TRACE0("CScopeDialog::RestorePosition\n");


        CScopeWindow *pWnd = (CScopeWindow *)m_pParentWnd;
        
        if (!pWnd)
                return;


        // Get current positions

        RECT r;
        
        GetWindowRect( &r );


        
        // Update with saved positions

        pWnd->GetDialogPosition( &r, DialogId );


        
        // If no uninitialized positions, then move dialog

        if ((r.top    != DLG_POS_INIT) &&
            (r.left   != DLG_POS_INIT) &&
            (r.bottom != DLG_POS_INIT) &
            (r.right  != DLG_POS_INIT))

                MoveWindow( &r );


}

void CScopeDialog::RememberPosition() {
        
        TRACE0("CScopeDialog::RememberPosition\n");


        CScopeWindow *pWnd = (CScopeWindow *)m_pParentWnd;
        
        if (!pWnd)
                return;

        

        // Get current position and have main window save it
        
        RECT r;
        
        GetWindowRect( &r );

        pWnd->SaveDialogPosition( &r, DialogId );
}




//***********************************************************************
//
// CLASS CAboutDlg
//
// This class, derived from CDialog, displays version information about
// the current program.
//

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
        //{{AFX_MSG_MAP(CAboutDlg)
        ON_BN_CLICKED(IDC_BTN_X_OK,  OnOK)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()




CAboutDlg::CAboutDlg(CWnd* pParent /*=NULL*/)
          : CDialog(CAboutDlg::IDD, pParent) {

        TRACE0("CAboutDlg::CAboutDlg\n");
        
}

BOOL CAboutDlg::OnInitDialog() {
        
        TRACE0("CAboutDlg::OnInitDialog\n");

        CDialog::OnInitDialog();


        // Set dialog title based on app title

        sprintf( pAPP->szTemp, "About %s", pAPP->AppTitle );
                
        SetWindowText( pAPP->szTemp );

        

        // Set version text based on app title

        CStatic *pWnd = (CStatic *) GetDlgItem( IDC_LAB_X_VERSION );
        
        if (pWnd) {

                sprintf( pAPP->szTemp, "%s Version 11/98", pAPP->AppTitle );
                
                pWnd->SetWindowText( pAPP->szTemp );
                
                }
        

        
        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CAboutDlg::OnOK() {

        TRACE0("CAboutDlg::OnOK\n");
        
        CDialog::OnOK();
}





//***********************************************************************
//
// CLASS CRegistryDlg
//
// This class, derived from CScopeDialog, allows the user to press the 
// big reset button and reset the registry entries.

BEGIN_MESSAGE_MAP(CRegistryDlg, CScopeDialog)
        //{{AFX_MSG_MAP(CRegistryDlg)
        ON_BN_CLICKED(IDC_BTN_R_YES,  OnBtnYes)
        ON_BN_CLICKED(IDC_BTN_R_NO,   OnBtnNo)
        ON_BN_CLICKED(IDC_BTN_R_HELP, OnHelp)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()


CRegistryDlg::CRegistryDlg(CWnd* pParent /*=NULL*/)
                  : CScopeDialog(CRegistryDlg::IDD, pParent) {

        TRACE0("CRegistryDlg::CRegistryDlg\n");

        //{{AFX_DATA_INIT(CRegistryDlg)
        //}}AFX_DATA_INIT
                                  
}

BOOL CRegistryDlg::OnInitDialog() {

        TRACE0("CRegistryDlg::OnInitDialog\n");
        
        CScopeDialog::OnInitDialog();

        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}


void CRegistryDlg::DoDataExchange(CDataExchange* pDX) {

        TRACE0("CRegistryDlg::DoDataExchange\n");
        
        CScopeDialog::DoDataExchange(pDX);
        
        //{{AFX_DATA_MAP(CRegistryDlg)
        //}}AFX_DATA_MAP
}


void CRegistryDlg::OnBtnYes() {
        
        TRACE0("CRegistryDlg::OnBtnYes\n");

        CScopeDialog::OnOK();


        // Delete all current values

        if (!pAPP->RegistryClear()) {
                AfxMessageBox("Registry values not cleared\n");
                return;
                }




        // Restore factory defaults

        pAPP->InitFromRegistry();

}

void CRegistryDlg::OnBtnNo() {

        TRACE0("CRegistryDlg::OnBtnNo\n");

        CScopeDialog::OnCancel();
        
}

void CRegistryDlg::OnHelp() {
        
        TRACE0("CRegistryDlg::OnHelp\n");

        pAPP->ShowHelp(IDR_HELP_REG_TITLE, IDR_HELP_REG_TEXT, this);
                
}




//***********************************************************************
//
// CLASS COutputDlg
//
// This class, derived from CScopeDialog, presents and controls the Output 
// file dialog.  This dialog allows the user to specify the naming 
// convention and directory for the output binary data files.  A binary 
// header file called SCOPE.HDR is also written to that directory.  The 
// user also has some limited control over the size of the resulting files.
//

BEGIN_MESSAGE_MAP(COutputDlg, CScopeDialog)
        //{{AFX_MSG_MAP(COutputDlg)
        ON_MESSAGE(WM_EDITLOSTFOCUS,        OnEditLostFocus)
        ON_BN_CLICKED(IDC_BTN_F_OK,         OnOK)
        ON_BN_CLICKED(IDC_BTN_F_CANCEL,     OnCancel)
        ON_BN_CLICKED(IDC_BTN_F_HELP,       OnHelp)
        ON_BN_CLICKED(IDC_OPT_F_FILE_NONE,  OnOptFileOut)
        ON_BN_CLICKED(IDC_OPT_F_FILE_ONE,   OnOptFileOut)
        ON_BN_CLICKED(IDC_OPT_F_FILE_SEQ,   OnOptFileOut)
        ON_BN_CLICKED(IDC_OPT_F_FILE_TIME,  OnOptFileOut)
        ON_EN_KILLFOCUS(IDC_TXT_F_NBLOCK,   OnKillfocusTxtNblock)
        ON_EN_KILLFOCUS(IDC_TXT_F_NSAMPLE,  OnKillfocusTxtNsample)
        ON_EN_KILLFOCUS(IDC_TXT_F_SEQSTART, OnKillfocusTxtSeqStart)
        ON_EN_KILLFOCUS(IDC_TXT_F_WORKDIR,  OnKillfocusTxtWorkDir)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()




COutputDlg::COutputDlg(CWnd* pParent /*=NULL*/)
              : CScopeDialog(COutputDlg::IDD, pParent) {

        TRACE0("COutputDlg::COutputDlg\n");

        InitData();     
                
}


COutputDlg::COutputDlg(BOOL ROnsamples, CWnd* pParent /*=NULL*/)
              : CScopeDialog(COutputDlg::IDD, pParent) {

        TRACE0("COutputDlg::COutputDlg\n");

        InitData();
                
        m_ROnsamples = ROnsamples;
}


void COutputDlg::DoDataExchange(CDataExchange* pDX) {

        TRACE0("COutputDlg::DoDataExchange\n");
        
        CScopeDialog::DoDataExchange(pDX);
        
        //{{AFX_DATA_MAP(COutputDlg)
        DDX_Text(pDX, IDC_TXT_F_NBLOCK,   m_Nblock);
        DDX_Text(pDX, IDC_TXT_F_NSAMPLE,  m_Nsamples);
        DDX_Text(pDX, IDC_TXT_F_SEQSTART, m_SeqStart);
        DDX_Text(pDX, IDC_TXT_F_WORKDIR,  m_WorkDir);
        //}}AFX_DATA_MAP
}

void COutputDlg::InitData(void) {

        TRACE0("COutputDlg::InitData\n");
        
        //{{AFX_DATA_INIT(COutputDlg)
        m_ROnsamples = FALSE;
        m_Nblock     = 0;
        m_Nsamples   = 0;
        m_FileOut    = 0;
        m_SeqStart   = 0;
        m_WorkDir    = _T("");
        //}}AFX_DATA_INIT
}

BOOL COutputDlg::OnInitDialog() {

        TRACE0("COutputDlg::OnInitDialog\n");
        
        
        // Initialize all member variables from application globals

        m_Nsamples  = pAPP->AcqNsamples;
        m_Nblock    = pAPP->AcqNblock;
        m_FileOut   = pAPP->AcqFile;
        m_SeqStart  = pAPP->AcqSeqStart;
        m_WorkDir   = pAPP->WorkDir;

                
        
        CScopeDialog::OnInitDialog();



        
        // Maybe set Nsamples field to be read only

        if (m_ROnsamples)
                SetTextReadOnly( IDC_TXT_F_NSAMPLE );

        
        
        // Update filesize
        
        ComputeFileSize();



        // Set naming convention radio button
        
        CheckRadioButton( IDC_OPT_F_FILE_NONE, IDC_OPT_F_FILE_TIME,
                                        (IDC_OPT_F_FILE_NONE + m_FileOut) );

        OnOptFileOut();



        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}


void COutputDlg::OnCancel() {

        TRACE0("COutputDlg::OnCancel\n");
                
        CScopeDialog::OnCancel();
}

void COutputDlg::OnOK() {

        TRACE0("COutputDlg::OnOK\n");

                        
        if (!CheckFocus())
                return;

        
        CScopeDialog::OnOK();


        // Update globals

        pAPP->AcqNsamples  = m_Nsamples;
        pAPP->AcqNblock    = m_Nblock;
        pAPP->AcqFile      = m_FileOut;
        pAPP->AcqSeqStart  = m_SeqStart;
        pAPP->WorkDir      = m_WorkDir;
}

void COutputDlg::OnHelp() {

        TRACE0("COutputDlg::OnHelp\n");
                
        if (!CheckFocus())
                return;

        pAPP->ShowHelp(IDR_HELP_OUTPUT_TITLE, IDR_HELP_OUTPUT_TEXT, this);
}

LONG COutputDlg::OnEditLostFocus( UINT nID, LONG xxx ) {

        TRACE1("COutputDlg::OnEditLostFocus (%u)\n", nID);

        CString tmpDir;
        BOOL    good;
        

        if (!DoValidate) {
                return( 1 );
                }


        // Don't allow validation when already doing validation

        DoValidate = FALSE;
        
        // Validate various edit controls
        
        switch ( nID ) {

                case IDC_TXT_F_NSAMPLE:

                        good = ValidateAsInt( IDC_TXT_F_NSAMPLE,
                                                0,
                                                9999,
                                                &m_Nsamples,
                                                "Samples per trace" );
                        
                        if (good) 
                                ComputeFileSize();
                        
                        break;




                case IDC_TXT_F_NBLOCK:
                        
                        good = ValidateAsInt( IDC_TXT_F_NBLOCK,
                                                0,
                                                999,
                                                &m_Nblock,
                                                "Blocks per file" );
        
                        if (good) 
                                ComputeFileSize();
                        
                        break;


                        

                case IDC_TXT_F_WORKDIR:

                        GetDlgItemText( IDC_TXT_F_WORKDIR, tmpDir );

                        if (_access(tmpDir,00) != 0) {
                                AfxMessageBox( "Working directory must already exist" );
                                SetDlgItemText( IDC_TXT_F_WORKDIR, m_WorkDir );
                                GotoDlgCtrl( GetDlgItem(IDC_TXT_F_WORKDIR) );
                                good = FALSE;
                                }
                        else {
                                GetDlgItemText( IDC_TXT_F_WORKDIR, m_WorkDir );
                                good = TRUE;
                                }
                        
                        break;





                        
                case IDC_TXT_F_SEQSTART:

                        good = ValidateAsInt( IDC_TXT_F_SEQSTART,
                                                0,
                                                0xFFFF,
                                                &m_SeqStart,
                                                "Starting sequential file" );

                        break;

                        

                default:
                        good = TRUE;
                        break;
                                                
                        
                }


        // Re-allow validation

        DoValidate = TRUE;


        if (good)
                return( 1 );
        else
                return( 0 );
}


void COutputDlg::OnKillfocusTxtNsample() {

        TRACE0("COutputDlg::OnKillfocusTxtNsample\n");
                
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_F_NSAMPLE, 0 );
        
}

void COutputDlg::OnKillfocusTxtNblock() {

        TRACE0("COutputDlg::OnKillfocusTxtNblock\n");
                
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_F_NBLOCK, 0 );
}

void COutputDlg::ComputeFileSize() {

        TRACE0("COutputDlg::ComputeFileSize\n");
                
        
        long fs;

        fs = pAPP->AcqNchannels * m_Nsamples * m_Nblock * pAPP->DataPtSize;
        SetDlgItemInt( IDC_LAB_F_SIZEINBYTES, fs );
}

void COutputDlg::OnOptFileOut() {

        TRACE0("COutputDlg::OnOptFileOut\n");
                
        BOOL SeqState;
        int  chkbtn;

        chkbtn = GetCheckedRadioButton( IDC_OPT_F_FILE_NONE, IDC_OPT_F_FILE_TIME );
        m_FileOut = chkbtn - IDC_OPT_F_FILE_NONE;

        if (chkbtn == IDC_OPT_F_FILE_SEQ)
                        SeqState = TRUE;
        else
                        SeqState = FALSE;

            
        GetDlgItem(IDC_LAB_F_SEQSTART)->EnableWindow( SeqState );
        GetDlgItem(IDC_TXT_F_SEQSTART)->EnableWindow( SeqState );
}


void COutputDlg::OnKillfocusTxtWorkDir() {

        TRACE0("COutputDlg::OnKillfocusTxtWorkDir\n");
                
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_F_WORKDIR, 0 );
}


void COutputDlg::OnKillfocusTxtSeqStart() {
        
        TRACE0("COutputDlg::OnKillfocusTxtSeqStart\n");
                
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_F_SEQSTART, 0 );
}



//***********************************************************************
//
// CLASS CDisplayDlg
//
// This class, derived from CScopeDialog, presents and controls the Display
// dialog.  This dialog allows the user to specify various display options
// such as the number of traces to be displayed, the scaling applied
// to those traces, and the overall color scheme.

BEGIN_MESSAGE_MAP(CDisplayDlg, CScopeDialog)
        //{{AFX_MSG_MAP(CDisplayDlg)
        ON_MESSAGE(WM_EDITLOSTFOCUS,           OnEditLostFocus)
        ON_BN_CLICKED(IDC_BTN_D_OK,            OnOK)
        ON_BN_CLICKED(IDC_BTN_D_CANCEL,        OnCancel)
        ON_BN_CLICKED(IDC_BTN_D_HELP,          OnHelp)
        ON_EN_KILLFOCUS(IDC_TXT_D_SHIFTBY,     OnKillfocusTxtShiftby)
        ON_EN_KILLFOCUS(IDC_TXT_D_NDISPLAY,    OnKillfocusTxtNdisplay)
        ON_EN_KILLFOCUS(IDC_TXT_D_FIRSTSAMPLE, OnKillfocusTxtFirstsample)
        ON_EN_KILLFOCUS(IDC_TXT_D_FIRSTTRACE,  OnKillfocusTxtFirsttrace)
        ON_BN_CLICKED(IDC_CHK_D_CLIPPING,      OnChkClipping)
        ON_BN_CLICKED(IDC_CHK_D_AUTODC,        OnChkAutoDC)
        ON_BN_CLICKED(IDC_OPT_D_COLORMONO,     OnOptColorScheme)
        ON_BN_CLICKED(IDC_OPT_D_COLORGREEN,    OnOptColorScheme)
        ON_BN_CLICKED(IDC_OPT_D_COLORBLUE,     OnOptColorScheme)
        ON_BN_CLICKED(IDC_OPT_D_COLORBROWN,    OnOptColorScheme)
        //}}AFX_MSG_MAP
//      ON_CONTROL_RANGE(BN_CLICKED, IDC_OPT_D_COLORMONO, IDC_OPT_D_COLORBROWN, OnOptColorScheme)
END_MESSAGE_MAP()




CDisplayDlg::CDisplayDlg(CWnd* pParent /*=NULL*/)
           : CScopeDialog(CDisplayDlg::IDD, pParent) {
                
        //{{AFX_DATA_INIT(CDisplayDlg)
        m_ShiftBy     = 0;
        m_Ndisplay    = 0;
        m_FirstSample = 0;
        m_FirstTrace  = 0;
        m_ColorScheme = 0;
        m_TraceWidth  = 0;
        m_Clipping    = FALSE;
        m_AutoDC      = FALSE;
        //}}AFX_DATA_INIT
}


void CDisplayDlg::DoDataExchange(CDataExchange* pDX)
{
        CScopeDialog::DoDataExchange(pDX);
        //{{AFX_DATA_MAP(CDisplayDlg)
        DDX_Control(pDX, IDC_TXT_D_SHIFTBY,     m_TXT_ShiftBy);
        DDX_Control(pDX, IDC_TXT_D_NDISPLAY,    m_TXT_Ndisplay);
        DDX_Control(pDX, IDC_TXT_D_FIRSTSAMPLE, m_TXT_FirstSample);
        DDX_Control(pDX, IDC_TXT_D_FIRSTTRACE,  m_TXT_FirstTrace);
        DDX_Control(pDX, IDC_LAB_D_PLOTBITS,    m_LAB_PlotBits);
        DDX_Control(pDX, IDC_LAB_D_NSCAN,       m_LAB_Nscan);
        DDX_Control(pDX, IDC_LAB_D_LASTSAMPLE,  m_LAB_LastSample);
        DDX_Control(pDX, IDC_CHK_D_CLIPPING,    m_CHK_Clipping);
        DDX_Control(pDX, IDC_CHK_D_AUTODC,      m_CHK_AutoDC);
        DDX_Control(pDX, IDC_OPT_D_COLORMONO,   m_OPT_ColorMono);
        DDX_Control(pDX, IDC_OPT_D_COLORGREEN,  m_OPT_ColorGreen);
        DDX_Control(pDX, IDC_OPT_D_COLORBLUE,   m_OPT_ColorBlue);
        DDX_Control(pDX, IDC_OPT_D_COLORBROWN,  m_OPT_ColorBrown);
        DDX_Text(pDX, IDC_TXT_D_SHIFTBY,     m_ShiftBy);
        DDX_Text(pDX, IDC_TXT_D_NDISPLAY,    m_Ndisplay);
        DDX_Text(pDX, IDC_TXT_D_FIRSTSAMPLE, m_FirstSample);
        DDX_Text(pDX, IDC_TXT_D_FIRSTTRACE,  m_FirstTrace);
        //}}AFX_DATA_MAP
}


BOOL CDisplayDlg::OnInitDialog() {

        
        // Initialize all member variables from application globals

        m_ShiftBy     = pAPP->PltShiftBy;
        m_Ndisplay    = pAPP->PltNdisplay;
        m_FirstSample = pAPP->PltFirstSample;
        m_FirstTrace  = pAPP->PltFirstTrace;
        m_ColorScheme = pAPP->PltColorScheme;
        m_Clipping    = pAPP->DoClipping;
        m_AutoDC      = pAPP->AutoDC;


        CScopeWindow *pWnd = (CScopeWindow *)m_pParentWnd;
        if (pWnd)
                m_TraceWidth = pWnd->TraceWidth;
        else
                m_TraceWidth = 500;



        
        // Initialize base class and initialize text boxes (via DDX)

        CScopeDialog::OnInitDialog();
        

        // Initialize labels and radio buttons

        SetPlotBitsLabel();
        SetNscanLabel();
        SetLastSampleLabel();
        SetColorSchemeRadio();

        if (m_Clipping)
                m_CHK_Clipping.SetCheck(CHECKED);
        else
                m_CHK_Clipping.SetCheck(UNCHECKED);


        if (m_AutoDC)
                m_CHK_AutoDC.SetCheck(CHECKED);
        else
                m_CHK_AutoDC.SetCheck(UNCHECKED);


        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CDisplayDlg::SetPlotBitsLabel( ) {

        CString str;
        int nscan;

        nscan = pAPP->MaxAmpShift - m_ShiftBy;
        str.Format("Plot %d Bits", nscan );
        CStatic *Label = (CStatic*)GetDlgItem(IDC_LAB_D_PLOTBITS);
        Label->SetWindowText( str );
}

void CDisplayDlg::SetNscanLabel( ) {
        CString str;
        str.Format("Scan %d Traces", pAPP->AcqNchannels);
        m_LAB_Nscan.SetWindowText( str );
}

void CDisplayDlg::SetLastSampleLabel( ) {

        sprintf(pAPP->szTemp, "Last is %d", (m_FirstSample + m_TraceWidth));
        m_LAB_LastSample.SetWindowText( pAPP->szTemp );
}

void CDisplayDlg::SetColorSchemeRadio( ) {

        TRACE0("CDisplayDlg::SetColorSchemeRadio\n");

        int ibtn;

        ibtn = IDC_OPT_D_COLORMONO + m_ColorScheme;
        CheckRadioButton( IDC_OPT_D_COLORMONO, IDC_OPT_D_COLORBROWN, ibtn );
}

void CDisplayDlg::OnCancel() {

        TRACE0("CDisplayDlg::OnCancel\n");

        CScopeDialog::OnCancel();
}

void CDisplayDlg::OnOK() {

        TRACE0("CDisplayDlg::OnOK\n");


        if (!CheckFocus())
                return;

        
        CScopeDialog::OnOK();


        
        // Update globals

        pAPP->PltShiftBy     = m_ShiftBy;
        pAPP->PltNdisplay    = m_Ndisplay;
        pAPP->PltFirstSample = m_FirstSample;
        pAPP->PltFirstTrace  = m_FirstTrace;
        pAPP->PltColorScheme = m_ColorScheme;
        pAPP->DoClipping     = m_Clipping;
        pAPP->AutoDC         = m_AutoDC;

}

void CDisplayDlg::OnHelp() {

        TRACE0("CDisplayDlg::OnHelp\n");

        if (!CheckFocus())
                return;

        
        pAPP->ShowHelp(IDR_HELP_DISP_TITLE, IDR_HELP_DISP_TEXT, this);
}

LONG CDisplayDlg::OnEditLostFocus( UINT nID, LONG xxx ) {

        TRACE1("CDisplayDlg::OnEditLostFocus (%u)\n", nID);

        int  maxFirst;
        BOOL good;
        

        if (!DoValidate) {
                return( 1 );
                }


        // Don't allow validation when already doing validation

        DoValidate = FALSE;
        
        // Validate various edit controls
        
        switch ( nID ) {

                case IDC_TXT_D_SHIFTBY:

                        good = ValidateAsInt( IDC_TXT_D_SHIFTBY,
                                                pAPP->MinAmpShift, 
                                                pAPP->MaxAmpShift-1,
                                                &m_ShiftBy,
                                                "Shift by" );
                        if (good)
                                SetPlotBitsLabel();
                        
                        break;




                case IDC_TXT_D_NDISPLAY:
                        
                        good = ValidateAsInt( IDC_TXT_D_NDISPLAY,
                                                1,
                                                pAPP->AcqNchannels,
                                                &m_Ndisplay,
                                                "Number to display" );
        
                        if (good) {
//                              AdjustFirstTrace();
                                maxFirst = pAPP->AcqNchannels - m_Ndisplay;
                                if (m_FirstTrace > maxFirst) {
                                        m_FirstTrace = maxFirst;
                                        SetDlgItemInt(IDC_TXT_D_FIRSTTRACE,
                                                        m_FirstTrace);
                                        }
                                }
                        
                        break;


                        

                case IDC_TXT_D_FIRSTSAMPLE:

                        maxFirst = pAPP->AcqNsamplesPlot - m_TraceWidth - 1;
                        if (maxFirst < 0)
                                maxFirst = 0;

                        good = ValidateAsInt( IDC_TXT_D_FIRSTSAMPLE,
                                                0,
                                                maxFirst,
                                                &m_FirstSample,
                                                "First sample" );
        
                        if (good)
                                SetLastSampleLabel();
                        
                        break;





                        
                case IDC_TXT_D_FIRSTTRACE:

                        maxFirst = pAPP->AcqNchannels - m_Ndisplay;
        
                        good = ValidateAsInt( IDC_TXT_D_FIRSTTRACE,
                                                0,
                                                maxFirst,
                                                &m_FirstTrace,
                                                "First trace" );

                        break;

                default:
                        good = TRUE;
                        break;
                                                
                        
                }


        // Re-allow validation

        DoValidate = TRUE;


        if (good)
                return( 1 );
        else
                return( 0 );
}


void CDisplayDlg::OnKillfocusTxtShiftby() {
        
        TRACE0("CDisplayDlg::OnKillfocusTxtShiftby\n");

        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_D_SHIFTBY, 0 );
}

void CDisplayDlg::OnKillfocusTxtNdisplay() {

        TRACE0("CDisplayDlg::OnKillfocusTxtNdisplay\n");

        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_D_NDISPLAY, 0 );
}


void CDisplayDlg::OnKillfocusTxtFirstsample() {
        
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_D_FIRSTSAMPLE, 0 );
}

void CDisplayDlg::OnKillfocusTxtFirsttrace() {

        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_D_FIRSTTRACE, 0 );
}

void CDisplayDlg::OnChkClipping(  ) {

        m_Clipping = !m_Clipping;
}

void CDisplayDlg::OnChkAutoDC(  ) {

        m_AutoDC = !m_AutoDC;
}

void CDisplayDlg::OnOptColorScheme(  ) {

        int ibtn;

        ibtn = GetCheckedRadioButton( IDC_OPT_D_COLORMONO, IDC_OPT_D_COLORBROWN );
        m_ColorScheme = ibtn - IDC_OPT_D_COLORMONO;
}







//***********************************************************************
//
// CLASS CContentsDlg
//
// This class, derived from CScopeDialog, presents and controls the Help 
// Contents dialog.  This dialog lists the topics for which help text is 
// available and allows the user to select and view this text.

BEGIN_MESSAGE_MAP(CContentsDlg, CScopeDialog)
        //{{AFX_MSG_MAP(CContentsDlg)
        ON_BN_CLICKED(IDC_BTN_H_DISPLAY,   OnBtnDisplay)
        ON_BN_CLICKED(IDC_BTN_H_HELP,      OnHelp)
        ON_BN_CLICKED(IDC_BTN_H_CLOSE,     OnCancel)
        ON_LBN_DBLCLK(IDC_LST_H_TOPICS,    OnDblclkTopics)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()


CContentsDlg::CContentsDlg(CWnd* pParent /*=NULL*/)
            : CScopeDialog(CContentsDlg::IDD, pParent) {

}


void CContentsDlg::DoDataExchange(CDataExchange* pDX) {

        CScopeDialog::DoDataExchange(pDX);
        //{{AFX_DATA_MAP(CContentsDlg)
        DDX_Control(pDX, IDC_LST_H_TOPICS, m_LST_topics);
        //}}AFX_DATA_MAP
}


void CContentsDlg::OnBtnDisplay() {

        int   pos, titleID, textID;
        DWORD item;

                
        titleID = IDR_HELP_UNKNOWN_TITLE;
        textID  = IDR_HELP_UNKNOWN_TEXT;
        
        pos = m_LST_topics.GetCurSel();
        
        if (pos != LB_ERR) {

                item    = m_LST_topics.GetItemData(pos);
                titleID = LOWORD(item);
                textID  = HIWORD(item);
                
                }
        
        pAPP->ShowHelp(titleID, textID, this);
}

void CContentsDlg::OnHelp() {
        
        pAPP->ShowHelp(IDR_HELP_HELP_TITLE, IDR_HELP_HELP_TEXT, this);
}

void CContentsDlg::OnCancel() {

        CScopeDialog::OnCancel();
}

BOOL CContentsDlg::OnInitDialog() {


        CScopeDialog::OnInitDialog();

        
        m_LST_topics.ResetContent();

        pAPP->LoadHelpTitles( &m_LST_topics );

        m_LST_topics.SetCurSel(0);
        
        
        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CContentsDlg::OnDblclkTopics() {

        OnBtnDisplay();
}






//***********************************************************************
//
// CLASS CTopicDlg
//
// This class, derived from CScopeDialog, presents and controls the Help 
// Topic dialog.  This dialog displays help text for the selected topic.  
// This may have been selected directly by the user from the Help 
// Contents dialog or it may have been selected automatically by the 
// program based on context.

BEGIN_MESSAGE_MAP(CTopicDlg, CScopeDialog)
        //{{AFX_MSG_MAP(CTopicDlg)
        ON_WM_SIZE()
        ON_EN_SETFOCUS(IDC_TXT_T_TOPICTEXT, OnSetfocusTopic)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()


CTopicDlg::CTopicDlg(CWnd* pParent /*=NULL*/)
          : CScopeDialog(CTopicDlg::IDD, pParent) {

        TRACE0("CTopicDlg::CTopicDlg\n");               
                  
        //{{AFX_DATA_INIT(CTopicDlg)
        //}}AFX_DATA_INIT
                  
        m_TitleID = IDR_HELP_UNKNOWN_TITLE;
        m_TextID  = IDR_HELP_UNKNOWN_TEXT;
}

CTopicDlg::CTopicDlg(int TitleID, int TextID, CWnd* pParent /*=NULL*/)
          : CScopeDialog(CTopicDlg::IDD, pParent) {

        TRACE0("CTopicDlg::CTopicDlg\n");               

        m_TitleID = TitleID;
        m_TextID  = TextID;
}

void CTopicDlg::DoDataExchange(CDataExchange* pDX) {

        CScopeDialog::DoDataExchange(pDX);
        
        //{{AFX_DATA_MAP(CTopicDlg)
        DDX_Control(pDX, IDC_TXT_T_TOPICTEXT, m_TXT_Topic);
        //}}AFX_DATA_MAP
}


BOOL CTopicDlg::OnInitDialog() {

        TRACE0("CTopicDlg::OnInitDialog\n");
                
        // Initialize all member variables from application globals


        CScopeDialog::OnInitDialog();


        
        // Set title

        htitle = (LPCSTR)::LockResource( 
                         ::LoadResource( NULL, 
                         ::FindResource( NULL, 
                            (LPCTSTR)m_TitleID, RT_RCDATA)) );
        SetWindowText(htitle);


        

        // Set text
        
        htext = (LPCSTR)::LockResource( 
                        ::LoadResource( NULL, 
                        ::FindResource( NULL, 
                            (LPCTSTR)m_TextID, RT_RCDATA)) );
        m_TXT_Topic.SetWindowText(htext);
                
        
        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}



void CTopicDlg::OnSize(UINT nType, int cx, int cy) {

        TRACE0("CTopicDlg::OnSize\n");
        
        CScopeDialog::OnSize(nType, cx, cy);
        
        m_TXT_Topic.MoveWindow(0,0,cx,cy);
}

void CTopicDlg::OnSetfocusTopic() {

        TRACE0("CTopicDlg::OnSetfocusTopic\n");

        m_TXT_Topic.SetSel(0,0,FALSE);
}

